@@ -74,7 +74,7 @@
   (cond ((= (string-length s) 1)
 	 (cond ((== s "*") " ")
 	       ((in? s '("+" "-" "=")) (string-append " " s " "))
-	       ((char-alphabetic? (string-ref s 0)) `(h:i ,s))
+	       ((char-alphabetic? (string-ref s 0)) `(h:var ,s))
 	       (else s)))
 	((string-starts? s "<cal-")
 	 `(h:font (@ (face "Zapf Chancery")) ,(tmhtml-sub-token s 5)))
@@ -83,7 +83,7 @@
 	((string-starts? s "<frak-")
 	 `(h:u ,(tmhtml-sub-token s 6)))
 	((string-starts? s "<bbb-") `(h:u (h:b ,(tmhtml-sub-token s 5))))
-	((string-starts? s "<b-") `(h:b (h:i ,(tmhtml-sub-token s 3))))
+	((string-starts? s "<b-") `(h:b (h:var ,(tmhtml-sub-token s 3))))
 	((string-starts? s "<")
 	 (with encoded (cork->utf8 s)
 	   (utf8->html (if (== s encoded)
@@ -544,6 +544,7 @@
 	   (and (func? (car l) 'h:i) (tmhtml-short? (cdar l)))
 	   (and (func? (car l) 'h:b) (tmhtml-short? (cdar l)))
 	   (and (func? (car l) 'h:u) (tmhtml-short? (cdar l)))
+	   (and (func? (car l) 'h:var) (tmhtml-short? (cdar l)))
 	   (and (func? (car l) 'h:font) (tmhtml-short? (cdar l))))))
 
 (define (tmhtml-wide l)
@@ -839,7 +840,7 @@
 	((== x '("cell-halign" "c")) "text-align: center")
 	((== x '("cell-halign" "r")) "text-align: right")
 	((== x '("cell-valign" "t")) "vertical-align: top")
-	((== x '("cell-valign" "c")) "vertical-align: center")
+	((== x '("cell-valign" "c")) "vertical-align: middle")
 	((== x '("cell-valign" "b")) "vertical-align: bottom")
 	((== x '("cell-valign" "B")) "vertical-align: baseline")
 	((== (car x) "cell-background")
