@@ -60,10 +60,13 @@
   (gggeq "\\geq\\negmedspace\\negmedspace\\geq\\negmedspace\\negmedspace\\geq")
 
   ;; extra literal symbols
+  (btimes "{\\mbox{\\rotatebox[origin=c]{90}{$\\ltimes$}}}")
   (Backepsilon "{\\mbox{\\rotatebox[origin=c]{180}{E}}}")
   (Mho "{\\mbox{\\rotatebox[origin=c]{180}{$\\Omega$}}}")
+  (mathcatalan "C")
   (mathd "\\mathrm{d}")
   (mathe "\\mathrm{e}")
+  (matheuler "\\gamma")
   (mathi "\\mathrm{i}")
   (mathpi "\\pi")
   (Alpha "\\mathrm{A}")
@@ -94,6 +97,7 @@
   (dottimes "\\mathaccent95{\\times}")
   (dotoplus "\\mathaccent95{\\oplus}")
   (dototimes "\\mathaccent95{\\otimes}")
+  (into "\\rightarrow")
   (longequal "{=\\!\\!=}")
   (longhookrightarrow "{\\lhook\\joinrel\\relbar\\joinrel\\rightarrow}")
   (longhookleftarrow "{\\leftarrow\\joinrel\\relbar\\joinrel\\rhook}")
@@ -102,6 +106,8 @@
   (triangleup "\\triangle")
   (precdot "{\\prec\\hspace{-0.6em}\\cdot}\\;\\,")
   (preceqdot "{\\preccurlyeq\\hspace{-0.6em}\\cdot}\\;\\,")
+  (llangle "{\\langle\\!\\langle}")
+  (rrangle "{\\rangle\\!\\rangle}")
   (join "\\Join")
   (um "-")
   (upl "+")
@@ -113,6 +119,10 @@
   (minusassign "-\\!\\!=")
   (timesassign "\times\\!\\!=")
   (overassign "/\\!\\!=")
+  (lflux "\\ll")
+  (gflux "\\gg")
+  (colons "\\,:\\,")
+  (transtype "\\,:\\!!>")
   (udots "{\\mathinner{\\mskip1mu\\raise1pt\\vbox{\\kern7pt\\hbox{.}}\\mskip2mu\\raise4pt\\hbox{.}\\mskip2mu\\raise7pt\\hbox{.}\\mskip1mu}}"))
 
 (drd-rules
@@ -182,7 +192,8 @@
 
 (drd-table latex-texmacs-3%
   (subsubindex (index (!append 1 "!" 2 "!" 3)))
-  (tmref 1))
+  (tmref 1)
+  (glossaryentry (!append (item (!option (!append 1 (hfill)))) 2 (dotfill) 3)))
 
 (drd-table latex-texmacs-4%
   (subsubsubindex (index (!append 1 "!" 2 "!" 3 "!" 4))))
@@ -227,17 +238,43 @@
 			       "\\setlength{\\itemindent}{\\parindent}"
 			       "\\setlength{\\parsep}{\\parskip}"))
     (!append "\\item[]"
-	     ---))))
+	     ---)))
+  ("tmparsep"
+   (!append (begingroup) "\\setlength{\\parskip}{" 1 "}"
+	     ---
+	     (endgroup)))
+  ("tmindent"
+   ((!begin "tmparmod" "1.5em" "0pt" "0pt") ---))
+  ("elsequation" ((!begin "eqnarray") (!append --- "&&")))
+  ("elsequation*" ((!begin "eqnarray*") (!append --- "&&")))
+  ("theglossary"
+   ((!begin "list" "" (!append "\\setlength{\\labelwidth}{6.5em}"
+			       "\\setlength{\\leftindent}{7em}"
+			       "\\small")) ---)))
 
 (drd-table latex-texmacs-env-arity%
   ("proof" 0)
   ("proof*" 1)
-  ("tmparmod" 3))
+  ("tmparmod" 3)
+  ("tmparsep" 1)
+  ("tmindent" 0)
+  ("proof" 0)
+  ("elsequation" 0)
+  ("elsequation*" 0)
+  ("theglossary" 1))
 
-(drd-group latex-texmacs-tag% begin-proof begin-proof* begin-tmparmod)
-(drd-group latex-environment-0% begin-proof)
-(drd-group latex-environment-1% begin-proof*)
-(drd-group latex-environment-3% begin-tmparmod)
+(drd-group latex-texmacs-tag%
+  begin-proof begin-proof* begin-tmparmod begin-tmparsep begin-tmindent
+  begin-elsequation begin-elsequation* begin-theglossary)
+
+(drd-group latex-environment-0%
+  begin-proof begin-tmindent begin-elsequation begin-elsequation*)
+
+(drd-group latex-environment-1%
+  begin-proof* begin-theglossary)
+
+(drd-group latex-environment-3%
+  begin-tmparmod begin-tmparsep)
 
 (define-macro (latex-texmacs-itemize env lab)
   (with env-sym (string->symbol (string-append "begin-" env))
@@ -265,6 +302,16 @@
        (drd-group latex-texmacs-tag% ,env-sym)
        (drd-group latex-list% ,env-sym))))
 
+(define-macro (latex-texmacs-description env)
+  (with env-sym (string->symbol (string-append "begin-" env))
+    `(begin
+       (drd-table latex-texmacs-environment%
+	 (,env ((!begin "description") ---)))
+       (drd-table latex-texmacs-env-arity% (,env 0))
+       ;;;
+       (drd-group latex-texmacs-tag% ,env-sym)
+       (drd-group latex-list% ,env-sym))))
+
 (latex-texmacs-itemize "itemizeminus" "$-$")
 (latex-texmacs-itemize "itemizedot" "$\\bullet$")
 (latex-texmacs-itemize "itemizearrow" "$\\rightarrow$")
@@ -273,6 +320,10 @@
 (latex-texmacs-enumerate "enumerateromancap" "I.")
 (latex-texmacs-enumerate "enumeratealpha" "a{\\textup{)}}")
 (latex-texmacs-enumerate "enumeratealphacap" "A.")
+(latex-texmacs-description "descriptioncompact")
+(latex-texmacs-description "descriptionaligned")
+(latex-texmacs-description "descriptiondash")
+(latex-texmacs-description "descriptionlong")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Extra preamble definitions which are needed to export certain macros
@@ -331,11 +382,10 @@
 (latex-texmacs-theorem "proposition" "Proposition")
 (latex-texmacs-theorem "lemma" "Lemma")
 (latex-texmacs-theorem "corollary" "Corollary")
-(latex-texmacs-theorem "lemma" "Lemma")
 (latex-texmacs-theorem "axiom" "Axiom")
-(latex-texmacs-theorem "lemma" "Lemma")
 (latex-texmacs-theorem "definition" "Definition")
 (latex-texmacs-theorem "notation" "Notation")
+(latex-texmacs-theorem "conjecture" "Conjecture")
 (latex-texmacs-remark "remark" "Remark")
 (latex-texmacs-remark "note" "Note")
 (latex-texmacs-remark "example" "Example")
@@ -427,8 +477,14 @@
   ("" "\\^A")
   ("" "{\\aa}")
   ("" "{\\AA}")
+  ("" "\\~A")
+  ("" "\\~a")
+  ("" "{\\AE}")
+  ("" "{\\ae}")
   ("" "\\c{c}")
   ("" "\\c{C}")
+  ("" "{\\dh}")
+  ("" "{\\DH}")
   ("" "\\`e")
   ("" "\\`E")
   ("" "\\'e")
@@ -438,13 +494,15 @@
   ("" "\\^e")
   ("" "\\^E")
   ("" "\\`{\\i}")
-  ("" "\\`{\\I}")
+  ("" "\\`I")
   ("" "\\'{\\i}")
-  ("" "\\'{\\I}")
+  ("" "\\'I")
   ("" "\\\"{\\i}")
-  ("" "\\\"{\\I}")
+  ("" "\\\"I")
   ("" "\\^{\\i}")
-  ("" "\\^{\\I}")
+  ("" "\\^I")
+  ("" "\\~N")
+  ("" "\\~n")
   ("" "\\`o")
   ("" "\\`O")
   ("" "\\'o")
@@ -453,6 +511,13 @@
   ("" "\\\"O")
   ("" "\\^o")
   ("" "\\^O")
+  ("" "\\~O")
+  ("" "\\~o")
+  ("" "{\\O}")
+  ("" "{\\o}")
+  ("" "{\\ss}")
+  ("" "{\\th}")
+  ("" "{\\TH}")
   ("" "\\`u")
   ("" "\\`U")
   ("" "\\'u")
@@ -467,7 +532,7 @@
   ("" "\\\"Y")
   ("" "!`")
   ("" "?`")
-  ("" "{\\ss}"))
+  ("" "?`"))
 
 (drd-table cyrillic-catcodes%
   ("" "\\CYRA")
