@@ -40,7 +40,15 @@
     (receive
 	(tm-uses tm-init tm-preamble)
 	(latex-preamble doc-misc style lan init)
+      (if (and (== lan "japanese") (== style "article"))
+	  (set! style "jarticle"))
+      (if (and (== lan "japanese") (== style "book"))
+	  (set! style "jbook"))
       (output-verbatim "\\documentclass{" style "}\n")
+      (if (== lan "korean")
+	  (output-verbatim "\\usepackage{dhucs}\n"))
+      (if (in? lan '("chinese" "taiwanese"))
+	  (output-verbatim "\\usepackage{CJK}\n"))
       (output-verbatim tm-uses)
       (for-each texout-usepackage (cdr styles))
       (output-verbatim tm-init)
@@ -179,15 +187,15 @@
 	(else #f)))
 
 (define (texout-double-math? x)
-  (or (and (match? x '((:or !document !concat) :1))
+  (or (and (match? x '((:or !document !concat) :%1))
 	   (texout-double-math? (cadr x)))
-      (and (match? x '((!begin :1) :1))
+      (and (match? x '((!begin :%1) :%1))
 	   (in? (cadar x) '("eqnarray" "eqnarray*" "leqnarray*")))))
 
 (define (texout-math x)
   (cond ((texout-empty? x) (noop))
 	((texout-double-math? x) (texout x))
-	((match? x '((!begin "center") :1))
+	((match? x '((!begin "center") :%1))
 	 (texout `((!begin "equation") ,(cadr x))))
 	((and (output-test-end? "$") (not (output-test-end? "\\$")))
 	 (output-remove 1)
@@ -253,6 +261,7 @@
 
 (define (texout x)
   (cond ((string? x) (output-text x))
+	((== (car x) '!widechar) (output-text (symbol->string (cadr x))))
 	((== (car x) '!file) (texout-file (cdr x)))
 	((== (car x) '!document) (texout-document (cdr x)))
 	((== (car x) '!paragraph) (texout-paragraph (cdr x)))
