@@ -22,20 +22,60 @@
 	(convert latex latex-tools)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Initialization
+;; Global variables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define tmtex-style "generic")
+(define tmtex-packages '())
 (define tmtex-env (make-ahash-table))
 (define tmtex-serial 0)
+(define tmtex-auto-produce 0)
+(define tmtex-auto-consume 0)
 (define tmtex-image-root-url (string->url "image"))
 (define tmtex-image-root-string "image")
 (define tmtex-appendices? #f)
 (define tmtex-replace-style? #t)
 (define tmtex-indirect-bib? #f)
+(define tmtex-oriental? #f)
+(define tmtex-chinese? #f)
+(define tmtex-japanese? #f)
+(define tmtex-korean? #f)
+(define tmtex-taiwanese? #f)
 
-(tm-define (tmtex-initialize opts)
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Style
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(texmacs-modes
+  (elsevier-style% (in? tmtex-style '("elsart")))
+  (natbib-package% (in? "cite-author-year" tmtex-packages)))
+
+(define (tmtex-set-style style body)
+  (set! tmtex-style (car style))
+  (set! tmtex-packages (cdr style))
+  (if (elsevier-style?) (init-elsevier body)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Language
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmtex-set-language lan)
+  (set! tmtex-chinese? (== lan "chinese"))
+  (set! tmtex-japanese? (== lan "japanese"))
+  (set! tmtex-korean? (== lan "korean"))
+  (set! tmtex-taiwanese? (== lan "taiwanese"))
+  (set! tmtex-oriental? (or tmtex-chinese? tmtex-japanese?
+			    tmtex-korean? tmtex-taiwanese?)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Initialization from options
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmtex-initialize opts)
   (set! tmtex-env (make-ahash-table))
   (set! tmtex-serial 0)
+  (set! tmtex-auto-produce 0)
+  (set! tmtex-auto-consume 0)
   (if (== (url-suffix current-save-target) "tex")
       (begin
 	(set! tmtex-image-root-url (url-unglue current-save-target 4))
@@ -63,7 +103,7 @@
   (block* ("" "c" "" #t))
   (tabular ("" "l" "" #f))
   (tabular* ("" "c" "" #f))
-  (matrix ((#{left\(}#) "c" (#{right\)}#) #f))
+  (matrix ((,(string->symbol "left(")) "c" (,(string->symbol "right)")) #f))
   (det ((left|) "c" (right|) #f))
   (choice ((left\{) "l" (right.) #f)))
 
@@ -134,15 +174,15 @@
 (define (tmtex-math-mode?)
   (== (tmtex-env-get "mode") "math"))
 
-(define (tmtex-env-set var val)
+(tm-define (tmtex-env-set var val)
   (ahash-set! tmtex-env var (cons val (tmtex-env-list var))))
 
-(define (tmtex-env-reset var)
+(tm-define (tmtex-env-reset var)
   (let ((val (tmtex-env-list var)))
     (if (nnull? val)
 	(ahash-set! tmtex-env var (cdr val)))))
 
-(define (tmtex-env-assign var val)
+(tm-define (tmtex-env-assign var val)
   (tmtex-env-reset var)
   (tmtex-env-set var val))
 
@@ -167,7 +207,8 @@
 	((func? (car l) '!concat) (append (cdar l) (tex-concat-list (cdr l))))
 	(else (cons (car l) (tex-concat-list (cdr l))))))
 
-(define (tex-concat l)
+(tm-define (tex-concat l)
+  (:synopsis "Horizontal concatenation of list of LaTeX expressions")
   (let ((r (tex-concat-similar (tex-concat-list l))))
     (if (null? r) ""
 	(if (null? (cdr r)) (car r)
@@ -179,8 +220,8 @@
 	 (tex-concat-strings (cons (string-append (car l) (cadr l)) (cddr l))))
 	(else (cons (car l) (tex-concat-strings (cdr l))))))
 
-(define (tex-concat* l)
-  "Variant of tex-concat for which adjecent strings are concatenated"
+(tm-define (tex-concat* l)
+  (:synopsis "Variant of tex-concat which concatenates adjacent strings")
   (tex-concat (tex-concat-strings l)))
 
 (define tex-apply
@@ -213,6 +254,24 @@
 	((string-starts? s "b-cal-")
 	 (tex-apply 'tmmathbf (tmtex-modified-token 'mathcal s 6)))
 	((string-starts? s "b-") (tmtex-modified-token 'tmmathbf s 2))
+	((and (string-starts? s "#") tmtex-oriental?)
+	 (cond (tmtex-japanese?
+		(let* ((qs (string-append "<" s ">"))
+		       (cv (string-convert qs "Cork" "ISO-2022-JP"))
+		       (ex (list->string (list #\33 #\50 #\102))))
+		  (set! cv (string-append cv ex))
+		  (list '!widechar (string->symbol cv))))
+	       (tmtex-korean?
+		(let* ((qs (string-append "<" s ">"))
+		       (cv (string-convert qs "Cork" "UTF-8")))
+		  (list '!widechar (string->symbol cv))))
+	       ((or tmtex-chinese? tmtex-taiwanese?)
+		(let* ((qs (string-append "<" s ">"))
+		       ;;(cv (string-convert qs "Cork" "cp936")) ; Chinese?
+		       ;;(cv (string-convert qs "Cork" "cp950")) ; Taiwanese ?
+		       (cv (string-convert qs "Cork" "UTF-8")))
+		  (set! cv (string-append cv ex))
+		  (list '!widechar (string->symbol cv))))))
 	(else (let ((ss (list (string->symbol s))))
 		(cond ((not (drd-in? (car ss) latex-symbol%))
 		       (display* "TeXmacs] non converted symbol: " s "\n")
@@ -339,8 +398,8 @@
 	((== x "seminar") "slides")
 	((in? x '("tmarticle" "tmdoc" "mmxdoc")) "article")
 	((in? x '("tmbook" "tmmanual")) "book")
-	;;((in? x '("acmconf" "amsart" "svjour" "elsart")) x)
-	;;((in? x '("jsc")) "elsart")
+	;;((in? x '("acmconf" "amsart" "svjour")) x)
+	((in? x '("elsart" "jsc")) "elsart")
 	((in? x '("acmconf" "amsart")) x)
 	((in? x '("svjour" "elsart" "jsc")) "article")
 	((not tmtex-replace-style?) x)
@@ -798,6 +857,10 @@
       arg
       (list (list '!begin "tmparmod" x y z) arg)))
 
+(define (tmtex-make-parsep x arg)
+  (set! x (tmlength->texlength x))
+  (list (list '!begin "tmparsep" x) arg))
+
 (define (tmtex-with-one var val arg)
   (if (== var "mode")
       (let ((old (tmtex-env-get-previous "mode")))
@@ -816,6 +879,7 @@
 	      ((== "par-left" var) (tmtex-make-parmod val "0pt" "0pt" arg))
 	      ((== "par-right" var) (tmtex-make-parmod "0pt" val "0pt" arg))
 	      ((== "par-first" var) (tmtex-make-parmod "0pt" "0pt" val arg))
+	      ((== "par-par-sep" var) (tmtex-make-parsep val arg))
 	      ((== var "color")
 	      	(list '!group (tex-concat (list (list 'color val) " " arg))))
 	      (else arg)))))
@@ -911,16 +975,10 @@
   (list 'pageref (force-string (car l))))
 
 (define (tmtex-specific l)
-  (cond ((== (car l) "latex") (tmtex (cadr l)))
+  (cond ((== (car l) "latex") (tmtex-tt (cadr l)))
 	((== (car l) "image") (tmtex-eps (cadr l)))
 	(else "")))
 
-(define (tmtex-hyperlink l)
-  (tmtex-function 'tmhlink l))
-
-(define (tmtex-action l)
-  (list 'tmaction (tmtex (car l)) (tmtex (cadr l))))
-
 (define (tmtex-eps-names)
   (set! tmtex-serial (+ tmtex-serial 1))
   (let* ((postfix (string-append "-" (number->string tmtex-serial) ".eps"))
@@ -932,7 +990,7 @@
   (if (tmtex-math-mode?) (set! x `(with "mode" "math" ,x)))
   (receive (name-url name-string) (tmtex-eps-names)
     (print-snippet name-url x)
-    (list 'epsfig (string-append "file=" name-string))))
+    (list 'includegraphics name-string)))
 
 (define (tmtex-graphics l)
   (tmtex-eps (cons 'graphics l)))
@@ -942,12 +1000,12 @@
     (if (or (string-ends? name ".ps")
 	    (string-ends? name ".eps")
 	    (not (url-exists? u)))
-	(list 'epsfig (string-append "file=" name))
+	(list 'includegraphics name)
 	(let* ((suffix (url-suffix u))
 	       (fm (string-append (format-from-suffix suffix) "-file")))
 	  (receive (name-url name-string) (tmtex-eps-names)
 	    (convert-to-file u fm "postscript-file" name-url)
-	    (list 'epsfig (string-append "file=" name-string)))))))
+	    (list 'includegraphics name-string))))))
 
 (define (tmtex-postscript l)
   (let* ((fig (tmtex-as-eps (force-string (car l))))
@@ -977,7 +1035,8 @@
 		      ((null? r) (car l))
 		      (else (append (car l) (list sep) r)))))))
 
-(define (tmtex-select-data expr tag)
+(tm-define (tmtex-select-data expr tag)
+  (:synopsis "Get data matching @tag in @expr with nice separators")
   (let* ((data (select expr (list tag)))
 	 (sep (if (== tag 'author-address) '(!nextline) "; "))
 	 (fun (lambda (x)
@@ -1005,7 +1064,7 @@
     (tex-concat* (tmtex-data-assemble '(!nextline)
 				      (list name* address)))))
 
-(define (tmtex-doc-data s l)
+(tm-define (tmtex-doc-data s l)
   (let* ((tag (cons s l))
 	 (title (tmtex-select-data tag 'doc-title))
 	 (authors (map tmtex-make-author (select tag '(doc-author-data))))
@@ -1022,6 +1081,15 @@
 		  (author ,(tex-concat author*))
 		  (maketitle)))))
 
+(define (tmtex-doc-data-wrapper s l)
+  (tmtex-doc-data s l))
+
+(tm-define (tmtex-abstract s l)
+  (tmtex-std-env s l))
+
+(define (tmtex-abstract-wrapper s l)
+  (tmtex-abstract s l))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; TeXmacs style primitives
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1058,6 +1126,9 @@
       (list 'tmtexttt (tmtex (car l)))))
 ;;(list '!verb (tmtex-tt (car l)))))
 
+(define (tmtex-indent s l)
+  (list (list '!begin "tmindent") (tmtex (car l))))
+
 (define (tmtex-list-env s l)
   (let* ((r (string-replace s "-" ""))
 	 (t (cond ((== r "enumerateRoman") "enumerateromancap")
@@ -1089,7 +1160,7 @@
 (define (tmtex-Huge s l)
   (list 'Huge (tmtex (car l))))
 
-(define (tmtex-equation s l)
+(tm-define (tmtex-equation s l)
   (tmtex-env-set "mode" "math")
   (let ((r (tmtex (car l))))
     (tmtex-env-reset "mode")
@@ -1097,6 +1168,9 @@
 	(list (list '!begin s) r)
 	(list '!eqn r))))
 
+(define (tmtex-equation-wrapper s l)
+  (tmtex-equation s l))
+
 (define (tmtex-eqnarray s l)
   (tmtex-env-set "mode" "math")
   (let ((r (tmtex-table-apply (string->symbol s) (car l))))
@@ -1114,7 +1188,7 @@
 
 (define (tmtex-bib-sub doc)
   (cond ((nlist? doc) doc)
-	((match? doc '(concat (bibitem* :1) (label :string?) :*))
+	((match? doc '(concat (bibitem* :%1) (label :string?) :*))
 	 (let* ((l (cadr (caddr doc)))
 		(s (if (string-starts? l "bib-") (string-drop l 4) l)))
 	   (cons* 'concat (list 'bibitem* (cadadr doc) s) (cdddr doc))))
@@ -1123,7 +1197,7 @@
 
 (define (tmtex-bib-max l)
   (cond ((npair? l) "")
-	((match? l '(bibitem* :string? :1)) (cadr l))
+	((match? l '(bibitem* :string? :%1)) (cadr l))
 	(else (let* ((s1 (tmtex-bib-max (car l)))
 		     (s2 (tmtex-bib-max (cdr l))))
 		(if (< (string-length s1) (string-length s2)) s2 s1)))))
@@ -1179,14 +1253,14 @@
 (define (tmtex-output s l)
   (list '!group (list 'ttfamily (tmtex (car l)))))
 
-(define (tmtex-cite-list l)
-  (cond ((null? l) "")
-	((nstring? (car l)) (tmtex-cite-list (cdr l)))
-	((null? (cdr l)) (car l))
-	(else (string-append (car l) "," (tmtex-cite-list (cdr l))))))
+(define (tmtex-hlink s l)
+  (list 'href (tmtex (cadr l)) (tmtex (car l))))
 
-(define (tmtex-cite s l)
-  (tex-apply (string->symbol s) (tmtex-cite-list l)))
+(define (tmtex-slink s l)
+  (tmtex-function 'url l))
+
+(define (tmtex-action s l)
+  (list 'tmaction (tmtex (car l)) (tmtex (cadr l))))
 
 (define (tmtex-choose s l)
   (list 'binom (tmtex (car l)) (tmtex (cadr l))))
@@ -1209,6 +1283,84 @@
 (define ((tmtex-rename into) s l)
   (tmtex-apply into (tmtex-list l)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Citations
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmtex-cite-list l)
+  (cond ((null? l) "")
+	((nstring? (car l)) (tmtex-cite-list (cdr l)))
+	((null? (cdr l)) (car l))
+	(else (string-append (car l) "," (tmtex-cite-list (cdr l))))))
+
+(tm-define (tmtex-cite s l)
+  (tex-apply 'cite (tmtex-cite-list l)))
+
+(tm-define (tmtex-cite s l)
+  (:mode natbib-package?)
+  (tex-apply 'citep (tmtex-cite-list l)))
+
+(define (tmtex-nocite s l)
+  (tex-apply 'nocite (tmtex-cite-list l)))
+
+(tm-define (tmtex-cite-detail s l)
+  (tex-apply 'cite `(!option ,(tmtex (cadr l))) (tmtex (car l))))
+
+(tm-define (tmtex-cite-detail s l)
+  (:mode natbib-package?)
+  (tex-apply 'citetext `(!concat (citealp ,(tmtex (car l))) ", "
+				 ,(tmtex (cadr l)))))
+
+(define (tmtex-cite-raw s l)
+  (tex-apply 'citealp (tmtex-cite-list l)))
+
+(define (tmtex-cite-raw* s l)
+  (tex-apply 'citealp* (tmtex-cite-list l)))
+
+(define (tmtex-cite-textual s l)
+  (tex-apply 'citet (tmtex-cite-list l)))
+
+(define (tmtex-cite-textual* s l)
+  (tex-apply 'citet* (tmtex-cite-list l)))
+
+(define (tmtex-cite-parenthesized s l)
+  (tex-apply 'citep (tmtex-cite-list l)))
+
+(define (tmtex-cite-parenthesized* s l)
+  (tex-apply 'citep* (tmtex-cite-list l)))
+
+(define (tmtex-render-cite s l)
+  (tex-apply 'citetext (tmtex (car l))))
+
+(define (tmtex-cite-author s l)
+  (tex-apply 'citeauthor (tmtex (car l))))
+
+(define (tmtex-cite-author* s l)
+  (tex-apply 'citeauthor* (tmtex (car l))))
+
+(define (tmtex-cite-year s l)
+  (tex-apply 'citeyear (tmtex (car l))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Glossaries
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmtex-glossary s l)
+  (with nr (+ tmtex-auto-produce 1)
+    (set! tmtex-auto-produce nr)
+    `(label ,(string-append "autolab" (number->string nr)))))
+
+(define (tmtex-glossary-entry s l)
+  (with nr (+ tmtex-auto-consume 1)
+    (with lab (string-append "autolab" (number->string nr))
+      (set! tmtex-auto-consume nr)
+      `(glossaryentry ,(tmtex (car l)) ,(tmtex (cadr l)) (pageref ,lab)))))
+
+(define (tmtex-the-glossary s l)
+  `(!document
+      (,(if (latex-book-style?) 'chapter* 'section*) "Glossary")
+      ((!begin "theglossary" ,(car l)) ,(tmtex (cadr l)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The main conversion routines
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1237,7 +1389,9 @@
 			 (map-in-order tmtex l)))))))
 
 (define (tmtex-compound l)
-  (tmtex-apply (string->symbol (car l)) (cdr l)))
+  (if (string? (car l))
+      (tmtex-apply (string->symbol (car l)) (cdr l))
+      ""))
 
 (define (tmtex-list l)
   (map-in-order tmtex l))
@@ -1352,8 +1506,6 @@
   (pageref tmtex-pageref)
   (write tmtex-noop)
   (specific tmtex-specific)
-  (hlink tmtex-hyperlink)
-  (action tmtex-action)
   ((:or tag meaning flag) tmtex-noop)
 
   ((:or anim-compose anim-repeat anim-constant
@@ -1379,20 +1531,23 @@
   ((:or hide-preamble show-preamble) (,tmtex-default -1))
   (hide-part (,tmtex-hide-part -1))
   (show-part (,tmtex-show-part -1))
-  (doc-data (,tmtex-doc-data -1))
+  (doc-data (,tmtex-doc-data-wrapper -1))
   ((:or doc-title doc-author-data doc-date doc-note
 	doc-keywords doc-AMS-class) (,tmtex-default -1))
   ((:or author-name author-address author-note
 	author-email author-homepage) (,tmtex-default -1))
-  (abstract (,tmtex-std-env 1))
+  (abstract (,tmtex-abstract-wrapper 1))
   (appendix (,tmtex-appendix 1))
   ((:or theorem proposition lemma corollary proof axiom definition
-	notation conjecture remark note example exercise warning
+	notation conjecture remark note example exercise problem warning
 	convention quote-env quotation verse)
    (,tmtex-std-env 1))
   ((:or verbatim code) (,tmtex-verbatim 1))
-  ((:or center indent) (,tmtex-std-env 1))
-  ((:or description itemize itemize-minus itemize-dot itemize-arrow
+  (center (,tmtex-std-env 1))
+  (indent (,tmtex-indent 1))
+  ((:or description description-compact description-aligned
+	description-dash description-long
+	itemize itemize-minus itemize-dot itemize-arrow
 	enumerate enumerate-numeric enumerate-roman enumerate-Roman
 	enumerate-alpha enumerate-Alpha)
    (,tmtex-list-env 1))
@@ -1410,10 +1565,14 @@
   (really-huge (,tmtex-Huge 1))
 
   (math (,tmtex-math 1))
-  ((:or equation equation*) (,tmtex-equation 1))
+  ((:or equation equation*) (,tmtex-equation-wrapper 1))
   ((:or eqnarray eqnarray* leqnarray*) (,tmtex-eqnarray 1))
   (eq-number (,tmtex-default -1))
-  ((:or the-index the-glossary) (,tmtex-dummy -1))
+  (the-index (,tmtex-dummy -1))
+  (glossary (,tmtex-glossary 1))
+  (glossary-explain (,tmtex-glossary 2))
+  (glossary-2 (,tmtex-glossary-entry 3))
+  (the-glossary (,tmtex-the-glossary 2))
   ((:or table-of-contents) (,tmtex-toc 2))
   (bibliography (,tmtex-bib 4))
   (thebibliography (,tmtex-thebibliography 2))
@@ -1427,13 +1586,28 @@
   (session (,tmtex-session 1))
   (input (,tmtex-input 2))
   (output (,tmtex-output 1))
-  ((:or cite nocite) (,tmtex-cite -1))
+  (hlink (,tmtex-hlink 2))
+  (action (,tmtex-action 2))
+  (slink (,tmtex-slink 1))
   (choose (,tmtex-choose 2))
   ((:or strong em tt name samp abbr dfn kbd var acronym person)
    (,tmtex-modifier 1))
   (menu (,tmtex-menu -1))
   (with-TeXmacs-text (,(tmtex-rename 'withTeXmacstext) 0))
-  (made-by-TeXmacs (,(tmtex-rename 'madebyTeXmacs) 0)))
+  (made-by-TeXmacs (,(tmtex-rename 'madebyTeXmacs) 0))
+  (cite (,tmtex-cite -1))
+  (nocite (,tmtex-nocite -1))
+  (cite-detail (,tmtex-cite-detail 2))
+  (cite-raw (,tmtex-cite-raw -1))
+  (cite-raw* (,tmtex-cite-raw* -1))
+  (cite-textual (,tmtex-cite-textual -1))
+  (cite-textual* (,tmtex-cite-textual* -1))
+  (cite-parenthesized (,tmtex-cite-parenthesized -1))
+  (cite-parenthesized* (,tmtex-cite-parenthesized* -1))
+  (render-cite (,tmtex-render-cite 1))
+  ((:or cite-author cite-author-link) (,tmtex-cite-author 1))
+  ((:or cite-author* cite-author*-link) (,tmtex-cite-author* 1))
+  ((:or cite-year cite-year-link) (,tmtex-cite-year 1)))
 
 (drd-group tmtex-protected%
   a b c d i j k l o r t u v H L O P S
@@ -1447,9 +1621,13 @@
 
 (define tmtex-user-defs-table (make-ahash-table))
 
+(define (user-definition? x)
+  (and (func? x 'assign 2)
+       (string? (cadr x))))
+
 (define (collect-user-defs-sub t)
   (cond ((npair? t) (noop))
-	((macro-definition? t)
+	((user-definition? t)
 	 (ahash-set! tmtex-user-defs-table (string->symbol (cadr t)) #t))
 	(else (for-each collect-user-defs-sub (cdr t)))))
 
@@ -1457,7 +1635,7 @@
   (if (== (get-preference "texmacs->latex:expand-user-macros") "on") '()
       (begin
 	(set! tmtex-user-defs-table (make-ahash-table))
-	(collect-user-defs-sub t)
+	(collect-user-defs-sub (cons 'document (tmtex-filter-preamble t)))
 	(ahash-set->list tmtex-user-defs-table))))
 
 (define (as-string sym)
@@ -1495,6 +1673,11 @@
   (if (== (car sty) "generic") (set! sty (cons "letter" (cdr sty))))
   sty)
 
+(define (find? x what)
+  (cond ((== x what) #t)
+	((nlist? x) #f)
+	(else (list-or (map (cut find? <> what) x)))))
+
 (tm-define (texmacs->latex x opts)
   (if (tmfile? x)
       (let* ((body (tmfile-extract x 'body))
@@ -1503,9 +1686,14 @@
 	     (lan (tmfile-init x "language"))
 	     (init (tmfile-extract x 'initial))
 	     (doc (list '!file body style lan init (get-texmacs-path))))
-	(latex-set-language lan)
 	(latex-set-style main-style)
-	(texmacs->latex doc opts))
+	(latex-set-language lan)
+	(tmtex-set-style style body)
+	(tmtex-set-language lan)
+	(with result (texmacs->latex doc opts)
+	  (tmtex-set-style '("generic") "")
+	  (tmtex-set-language "english")
+	  result))
       (let* ((x2 (tmtm-eqnumber->nonumber x))
 	     (x3 (tmtm-match-brackets x2)))
 	(tmtex-initialize opts)
