@@ -116,6 +116,11 @@
 	      ((or (== h '(mathvariant "bold"))
 		   (== h '(mathvariant "bold-italic")))
 	       (cons* "math-font-series" "bold" r))
+	      ((or (== h '(mathvariant "sans-serif"))
+		   (== h '(mathvariant "sans-serif--italic")))
+	       (cons* "math-font-family" "ms" r))
+	      ((== h '(mathvariant "monospace"))
+	       (cons* "math-font-family" "mt" r))
 	      (else r)))))
 
 (define (mathtm-mstyle env a c)
@@ -135,7 +140,7 @@
 (define (mathtm-mfenced env a c)
   (let* ((open (car (or (assoc-ref a 'open) '("("))))
 	 (close (car (or (assoc-ref a 'close) '(")"))))
-	 (seps (string-tokenize (car (or (assoc-ref a 'separators) '("")))
+	 (seps (string-tokenize-by-char (car (or (assoc-ref a 'separators) '("")))
 				#\space)))
     (if (== seps '("")) (set! seps '()))
     (mathtm env `(m:mrow (m:mo ,open)
@@ -145,7 +150,7 @@
 (define (mathtm-menclose env a c)
   (let* ((args (mathtm-args env c))
 	 (notation (car (or (assoc-ref a 'notation) '(""))))
-	 (l (if (== notation "") '() (string-tokenize notation #\space))))
+	 (l (if (== notation "") '() (string-tokenize-by-char notation #\space))))
     (if (in? "updiagonalstrike" l)
 	`((neg ,(mathtm-serial env args)))
 	args)))
@@ -309,7 +314,7 @@
 
 (define (mathtm-row-format a)
   (cond ((func? a 'columnalign)
-	 (with l (string-tokenize (cadr a) #\space)
+	 (with l (string-tokenize-by-char (cadr a) #\space)
 	   (mathtm-row-halign l 1)))
 	((and (func? a 'rowalign) (mathtm-valign (cadr a)))
 	 `((cwith "1" "-1" "cell-valign" ,(mathtm-valign (cadr a)))))
@@ -345,10 +350,10 @@
 
 (define (mathtm-table-format a)
   (cond ((func? a 'columnalign)
-	 (with l (string-tokenize (cadr a) #\space)
+	 (with l (string-tokenize-by-char (cadr a) #\space)
 	   (mathtm-table-halign l 1)))
 	((func? a 'rowalign)
-	 (with l (string-tokenize (cadr a) #\space)
+	 (with l (string-tokenize-by-char (cadr a) #\space)
 	   (mathtm-table-valign l 1)))
 	(else '())))
 
