@@ -17,6 +17,19 @@
 ;; TODO: scheme file caching using (set! primitive-load ...) and
 ;; (set! %search-load-path)
 
+;; (debug-enable 'backtrace 'debug)
+;; (define load-indent 0)
+;; (define old-primitive-load primitive-load)
+;; (define (new-primitive-load . x)
+;;   (for-each display (make-list load-indent "  "))
+;;   (display "Load ") (apply display x) (display "\n")
+;;   (set! load-indent (+ load-indent 1))
+;;   (apply old-primitive-load x)
+;;   (set! load-indent (- load-indent 1))
+;;   (for-each display (make-list load-indent "  "))
+;;   (display "Done\n"))
+;; (set! primitive-load new-primitive-load)
+
 ;(display "Booting TeXmacs kernel functionality\n")
 (load (url-concretize "$TEXMACS_PATH/progs/kernel/boot/boot.scm"))
 (inherit-modules (kernel boot compat) (kernel boot abbrevs)
@@ -30,9 +43,11 @@
 (inherit-modules (kernel texmacs tm-overload) (kernel texmacs tm-define)
 		 (kernel texmacs tm-preferences) (kernel texmacs tm-modes)
 		 (kernel texmacs tm-plugins) (kernel texmacs tm-secure)
-		 (kernel texmacs tm-convert) (kernel texmacs tm-dialogue))
+		 (kernel texmacs tm-convert) (kernel texmacs tm-dialogue)
+		 (kernel texmacs tm-file-system))
 (inherit-modules (kernel gui menu-define) (kernel gui menu-widget)
-		 (kernel gui kbd-define))
+		 (kernel gui kbd-define) (kernel gui gui-widget)
+		 (kernel gui gui-factory) (kernel gui gui-form))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting utilities\n")
@@ -90,7 +105,8 @@
 
 ;(display "Booting table mode\n")
 (lazy-keyboard (table table-kbd) in-table?)
-(lazy-menu (table table-menu) table-menu table-icons insert-table-menu)
+(lazy-menu (table table-menu) table-menu table-icons insert-table-menu
+	   horizontal-table-cell-menu vertical-table-cell-menu)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting graphics mode\n")
@@ -109,11 +125,13 @@
 (lazy-menu (dynamic fold-menu) insert-fold-menu)
 (lazy-menu (dynamic session-menu) insert-session-menu
 	   session-menu session-icons session-help-icons help-icons)
-(lazy-menu (dynamic scripts-menu)
+(lazy-menu (dynamic scripts-menu) scripts-eval-menu
 	   plugin-eval-menu plugin-eval-toggle-menu plugin-plot-menu)
+(lazy-define (dynamic form-edit) form-ref form-set! form-toggle)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting documentation\n")
+(lazy-keyboard (doc tmdoc-kbd) in-tmdoc?)
 (lazy-menu (doc help-menu) help-menu)
 (lazy-define (doc tmdoc) tmdoc-expand-help tmdoc-expand-help-manual
 	     tmdoc-expand-this tmdoc-include)
@@ -129,24 +147,47 @@
 (lazy-format (convert html init-html) html)
 (lazy-format (convert images init-images)
 	     postscript pdf xfig xmgrace svg xpm jpeg ppm gif png pnm)
+(lazy-define (convert rewrite init-rewrite) texmacs->verbatim)
 (lazy-define (convert html tmhtml-expand) tmhtml-env-patch)
 (lazy-define (convert latex latex-drd) latex-arity latex-type)
 (lazy-define (convert latex textm) textm-finalize)
 (lazy-define (convert latex tmtex) tmtex-env-patch)
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
+;(display "Booting remote facilities\n")
+(lazy-define (remote tmfs-remote) remote-load remote-save
+	     remote-name remote-permission? remote-project-load-by-name)
+(lazy-menu (remote remote-menu) remote-menu)
+(lazy-menu (remote chat-menu) chat-menu)
+(lazy-menu (remote tmfs-menu) remote-file-menu)
+(tmfs-handler #t 'load remote-load)
+(tmfs-handler #t 'save remote-save)
+(tmfs-handler #t 'name remote-name)
+(tmfs-handler #t 'permission? remote-permission?)
+;(display* "time: " (- (texmacs-time) boot-start) "\n")
+
+;(display "Booting linking facilities\n")
+(lazy-menu (link link-menu) link-menu)
+(lazy-keyboard (link link-kbd) with-linking-tool?)
+(lazy-define (link link-edit) create-unique-id)
+(lazy-define (link link-navigate) link-active-upwards link-active-ids
+	     link-follow-ids)
+(lazy-define (link link-extern) get-constellation
+	     get-link-locations register-link-locations)
+;(display* "time: " (- (texmacs-time) boot-start) "\n")
+
 ;(display "Booting plugins\n")
 (for-each lazy-plugin-initialize (plugin-list))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "Booting fonts\n")
 (use-modules (fonts fonts-ec) (fonts fonts-adobe) (fonts fonts-x)
-	     (fonts fonts-math) (fonts fonts-foreign) (fonts fonts-misc))
+	     (fonts fonts-math) (fonts fonts-foreign) (fonts fonts-misc)
+	     (fonts fonts-composite))
 (if (> (get-font-type) 0) (use-modules (fonts fonts-truetype)))
 ;(display* "time: " (- (texmacs-time) boot-start) "\n")
 
 ;(display "------------------------------------------------------\n")
-(delayed (:idle 200) (recover-auto-save))
 (delayed (:idle 10000) (delayed-auto-save))
 (texmacs-banner)
 ;(display "Initialization done\n")
