@@ -559,11 +559,12 @@
   (symbol "<mathd>")
   (symbol "<mathi>")
   (symbol "<mathe>")
+  (symbol "<matheuler>")
   (symbol "<mathpi>")
   (symbol "<imath>")
   (symbol "<jmath>")
   (symbol "<ell>")
-  (symbol "<wp>")
+
   (symbol "<aleph>")
   (symbol "<beth>")
   (symbol "<gimel>")
@@ -571,9 +572,17 @@
   (symbol "<Re>")
   (symbol "<Im>")
   (symbol "<Mho>")
+  (symbol "<wp>")
+
   (symbol "<emptyset>")
   (symbol "<varnothing>")
+  (symbol "<infty>")
+  (symbol "<partial>")
   (symbol "<nabla>")
+  (symbol "<forall>")
+  (symbol "<exists>")
+  (symbol "<neg>")
+
   (symbol "<top>")
   (symbol "<bot>")
   (symbol "<vdash>")
@@ -582,15 +591,7 @@
   (symbol "<vDash>")
   (symbol "<dashv>")
   (symbol "<angle>")
-  (symbol "<forall>")
-  (symbol "<exists>")
-  (symbol "<neg>")
-  (symbol "<flat>")
-  (symbol "<natural>")
-  (symbol "<sharp>")
-  (symbol "<backslash>")
-  (symbol "<partial>")
-  (symbol "<infty>")
+
   (symbol "<box>")
   (symbol "<diamond>")
   (symbol "<vartriangle>")
@@ -598,6 +599,17 @@
   (symbol "<diamondsuit>")
   (symbol "<heartsuit>")
   (symbol "<spadesuit>")
+  (symbol "<backslash>")
+
+  (symbol "<flat>")
+  (symbol "<natural>")
+  (symbol "<sharp>")
+  (symbol "<eighthnote>")
+  (symbol "<quarternote>")
+  (symbol "<halfnote>")
+  (symbol "<fullnote>")
+  (symbol "<twonotes>")
+
   (symbol "<sun>")
   (symbol "<leftmoon>")
   (symbol "<rightmoon>")
@@ -606,16 +618,12 @@
   (symbol "<female>")
   (symbol "<maltese>")
   (symbol "<kreuz>")
+
   (symbol "<recorder>")
   (symbol "<phone>")
   (symbol "<checked>")
   (symbol "<pointer>")
-  (symbol "<bell>")
-  (symbol "<eighthnote>")
-  (symbol "<quarternote>")
-  (symbol "<halfnote>")
-  (symbol "<fullnote>")
-  (symbol "<twonotes>"))
+  (symbol "<bell>"))
 
 (menu-bind dots-menu
   (symbol "<ldots>")
