@@ -18,6 +18,113 @@
     (generic format-edit)
     (text std-text-edit)))
 
+(kbd-symbols
+  "alpha" "beta" "gamma" "delta" "epsilon"
+  "zeta" "eta" "theta" "iota" "kappa"
+  "lambda" "mu" "nu" "xi" "omicron" "pi" "rho"
+  "sigma" "tau" "upsilon" "phi" "chi"
+  "psi" "omega" "varepsilon" "vartheta"
+  "varpi" "varrho" "varsigma" "varphi"
+  "leftharpoonup" "leftharpoondown"
+  "rightharpoonup" "rightharpoondown" "lefthook" "righthook"
+  "triangleright" "triangleleft" "star" "partial"
+  "flat" "natural" "sharp" "smile" "frown" "ell"
+  "imath" "jmath" "wp" "vect"
+  
+  "Alpha" "Beta" "Gamma" "Delta" "Epsilon"
+  "Zeta" "Eta" "Theta" "Iota" "Kappa" "Lambda"
+  "Mu" "Nu" "Xi" "Omicron" "Pi" "Rho"
+  "Sigma" "Tau" "Upsilon" "Phi" "Chi" "Psi" "Omega"
+  "grave" "acute" "check" "breve" "bar"
+  "cedille"
+  "AA" "AE" "DH" "L" "NG" "O" "OE" "S" "SS" "TH"
+  "aa" "ae" "dh" "dj" "i" "j" "l" "ng" "o" "oe" "ss" "sz" "th"
+
+  "ast" "asterisk"
+  
+  "cdot" "times" "asterisk" "div" "diamond"
+  "pm" "mp" "oplus" "ominus" "otimes" "oslash"
+  "odot" "bigcirc" "circ" "bullet" "asymp" "equiv"
+  "subseteq" "supseteq" "leq" "geq" "preceq" "succeq"
+  "sim" "approx" "subset" "supset" "ll" "gg"
+  "prec" "succ" "leftarrow" "rightarrow" "uparrow"
+  "downarrow" "leftrightarrow" "nearrow" "searrow"
+  "simeq" "Leftarrow" "Rightarrow" "Uparrow"
+  "Downarrow" "Leftrightarrow" "nwarrow" "swarrow"
+  "propto" "infty" "in" "ni" "mid" "bigtriangleup" "triangle"
+  "bigtriangledown" "negate" "varshortmid" "forall" "exists"
+  "neg" "emptyset" "Re" "Im" "top" "bot" "aleph" "perp"
+  "cup" "cap" "uplus" "wedge" "vee" "vdash" "dashv"
+  "lfloor" "rfloor" "lceil" "rceil" "langle" "rangle"
+  "updownarrow" "Updownarrow" "wr" "amalg" "nabla"
+  "sqcup" "sqcap" "sqsubseteq" "sqsupseteq" "dag" "ddag"
+  "endofline" "clubsuit" "diamondsuit" "heartsuit" "spadesuit"
+  "backslash" "setminus"
+
+  "boxdot" "boxplus" "boxtimes" "box" "blacksquare"
+  "centerdot" "lozenge" "blacklozenge" "circlearrowright"
+  "circlearrowleft" "rightleftharpoons" "leftrightharpoons"
+  "boxminus" "Vdash" "Vvdash" "dashV" "twoheadrightarrow"
+  "twoheadleftarrow" "leftleftarrows" "rightrightarrows"
+  "upuparrows" "downdownarrows" "upharpoonright"
+  "downharpoonright" "upharpoonleft" "downharpoonleft"
+  "rightarrowtail" "leftarrowtail" "leftrightarrows"
+  "rightleftarrows" "Lsh" "Rsh" "rightsquigarrow"
+  "leftrightsquigarrow" "looparrowleft" "looparrowright"
+  "circeq" "succsim" "gtrsim" "gtrapprox" "multimap"
+  "therefore" "because" "doteqdot" "triangleq"
+  "precsim" "lesssim" "lessapprox" "eqslantless"
+  "eqslantgtr" "curlyeqprec" "curlyeqsucc" "preccurlyeq"
+  "leqq" "leqslant" "lessgtr" "smalldash"
+  "risingdotseq" "fallingdotseq" "succcurlyeq" "geqq"
+  "geqslant" "gtrless" "sqsubset" "sqsupset"
+  "vartriangleright" "vartriangleleft" "trianglerighteq"
+  "trianglelefteq" "bigstar" "between"
+  "blacktriangledown" "blacktriangleright" "blacktriangleleft"
+  "shortrightarrow" "shortleftarrow" "vartriangle"
+  "blacktriangle" "triangledown" "eqcirc"
+  "lesseqgtr" "gtreqless" "lesseqqgtr" "gtreqqless"
+  "yen" "Rrightarrow" "Lleftarrow" "checkmark"
+  "veebar" "barwedge" "doublebarwedge" "angle" "measuredangle"
+  "sphericalangle" "varpropto" "smallsmile" "smallfrown" "Subset"
+  "Supset" "Cup" "Cap" "curlyvee" "curlywedge"
+  "leftthreetimes" "rightthreetimes" "subseteqq" "supseteqq"
+  "bumpeq" "Bumpeq" "lll" "ggg" "ulcorner" "urcorner"
+  "circledR" "circledS" "pitchfork" "dotplus" "backsim"
+  "backsimeq" "llcorner" "lrcorner" "maltese"
+  "complement" "intercal" "circledcirc" "circledast" "circleddash"
+  
+  "lvertneqq" "gvertneqq" "nleq" "ngeq" "nless" "ngtr"
+  "nprec" "nsucc" "lneqq" "gneqq" "nleqslant" "ngeqslant"
+  "lneq" "gneq" "npreceq" "nsucceq" "precnsim"
+  "succnsim" "lnsim" "gnsim" "nleqq" "ngeqq"
+  "precneqq" "succneqq" "precnapprox" "succnapprox"
+  "lnapprox" "gnapprox" "nsim" "ncong" "diagup"
+  "diagdown" "varsubsetneq" "varsupsetneq"
+  "nsubseteqq" "nsupseteqq" "subsetneqq" "supsetneqq"
+  "varsubsetneqq" "varsupsetneqq" "subsetneq" "supsetneq"
+  "nsubseteq" "nsupseteq" "parallel" "nparallel" "nmid" "nshortmid"
+  "nshortparallel" "nvdash" "nVdash" "nvDash" "nVDash" "models"
+  "ntrianglerighteq" "ntrianglelefteq" "ntriangleleft"
+  "ntriangleright" "nleftarrow" "nrightarrow" "nLeftarrow"
+  "nRightarrow" "nLeftrightarrow" "nleftrightarrow"
+  "divideontimes" "varnothing" "nexists"
+  "Mho" "thorn" "eth" "eqsim" "beth" "gimel" "daleth"
+  "lessdot" "gtrdot" "ltimes" "rtimes" "shortmid"
+  "shortparallel" "smallsetminus" "thicksim" "thickapprox"
+  "approxeq" "succapprox" "precapprox" "curvearrowleft"
+  "curvearrowright" "digamma" "varkappa" "" "hslash"
+  "hbar" "backepsilon"
+  
+  "mapsto" "longmapsto" "longrightarrow" "longleftarrow"
+  "longleftrightarrow" "longRightarrow" "longLeftarrow"
+  "longLeftrightarrow" "ldots" "cdots" "vdots"
+  "ddots" "dotamalg" "dottimes" "dotoplus" "dototimes" "venus"
+  "mars" "earth" "aries" "fullmoon" "newmoon" "astrosun"
+  "leo" "LEFTcircle" "RIGHTcircle" "Square" "CheckedBox" "photon" "gluon"
+  "neq" "notin" "nin" "notni" "nni" "nll" "nlll" "ngg" "nggg"
+  "dagger" "ddagger" "boxbar" "checked" "obar" "kreuz")
+
 (kbd-commands
   ("#" "Insert sharp" (insert "#"))
   ("$" "Insert dollar" (insert "$"))
@@ -41,8 +148,8 @@
 
   ("cC" "Make " (emulate-keyboard "cedilla C"))
   ("cc" "Make " (emulate-keyboard "cedilla c"))
-  ("\'E" "Insert " (emulate-keyboard "acute E"))
-  ("\'e" "Insert " (emulate-keyboard "acute e"))
+  ("'E" "Insert " (emulate-keyboard "acute E"))
+  ("'e" "Insert " (emulate-keyboard "acute e"))
   ("`A" "Insert " (emulate-keyboard "grave A"))
   ("`E" "Insert " (emulate-keyboard "grave E"))
   ("`U" "Insert " (emulate-keyboard "grave U"))
@@ -121,7 +228,7 @@
   ("not" "Make negation" (make-neg))
   ("acute" "Make acute" (make-wide "<acute>"))
   ("bar" "Make bar" (make-wide "<bar>"))
-  ("breve" "Make bar" (make-wide "<breve>"))
+  ("breve" "Make breve" (make-wide "<breve>"))
   ("check" "Make check" (make-wide "<check>"))
   ("grave" "Make grave" (make-wide "<grave>"))
   ("hat" "Make hat" (make-wide "^"))
@@ -277,110 +384,6 @@
   ("footnote" "Insert a footnote" (make 'footnote))
   ("input" "Include a document" (make 'include)))
 
-(kbd-symbols
-  "alpha" "beta" "gamma" "delta" "epsilon"
-  "zeta" "eta" "theta" "iota" "kappa"
-  "lambda" "mu" "nu" "xi" "omicron" "pi" "rho"
-  "sigma" "tau" "upsilon" "phi" "chi"
-  "psi" "omega" "varepsilon" "vartheta"
-  "varpi" "varrho" "varsigma" "varphi"
-  "leftharpoonup" "leftharpoondown"
-  "rightharpoonup" "rightharpoondown" "lefthook" "righthook"
-  "triangleright" "triangleleft" "star" "partial"
-  "flat" "natural" "sharp" "smile" "frown" "ell"
-  "imath" "jmath" "wp" "vect"
-  
-  "Alpha" "Beta" "Gamma" "Delta" "Epsilon"
-  "Zeta" "Eta" "Theta" "Iota" "Kappa" "Lambda"
-  "Mu" "Nu" "Xi" "Omicron" "Pi" "Rho"
-  "Sigma" "Tau" "Upsilon" "Phi" "Chi" "Psi" "Omega"
-  "grave" "acute" "check" "breve" "bar"
-  "cedille" "sz" "ae" "oe" "AE" "OE"
-  "ast" "asterisk"
-  
-  "cdot" "times" "asterisk" "div" "diamond"
-  "pm" "mp" "oplus" "ominus" "otimes" "oslash"
-  "odot" "bigcirc" "circ" "bullet" "asymp" "equiv"
-  "subseteq" "supseteq" "leq" "geq" "preceq" "succeq"
-  "sim" "approx" "subset" "supset" "ll" "gg"
-  "prec" "succ" "leftarrow" "rightarrow" "uparrow"
-  "downarrow" "leftrightarrow" "nearrow" "searrow"
-  "simeq" "Leftarrow" "Rightarrow" "Uparrow"
-  "Downarrow" "Leftrightarrow" "nwarrow" "swarrow"
-  "propto" "infty" "in" "ni" "mid" "bigtriangleup" "triangle"
-  "bigtriangledown" "negate" "varshortmid" "forall" "exists"
-  "neg" "emptyset" "Re" "Im" "top" "bot" "aleph" "perp"
-  "cup" "cap" "uplus" "wedge" "vee" "vdash" "dashv"
-  "lfloor" "rfloor" "lceil" "rceil" "langle" "rangle"
-  "updownarrow" "Updownarrow" "wr" "amalg" "nabla"
-  "sqcup" "sqcap" "sqsubseteq" "sqsupseteq" "dag" "ddag"
-  "endofline" "clubsuit" "diamondsuit" "heartsuit" "spadesuit"
-  "backslash" "setminus"
-
-  "boxdot" "boxplus" "boxtimes" "box" "blacksquare"
-  "centerdot" "lozenge" "blacklozenge" "circlearrowright"
-  "circlearrowleft" "rightleftharpoons" "leftrightharpoons"
-  "boxminus" "Vdash" "Vvdash" "dashV" "twoheadrightarrow"
-  "twoheadleftarrow" "leftleftarrows" "rightrightarrows"
-  "upuparrows" "downdownarrows" "upharpoonright"
-  "downharpoonright" "upharpoonleft" "downharpoonleft"
-  "rightarrowtail" "leftarrowtail" "leftrightarrows"
-  "rightleftarrows" "Lsh" "Rsh" "rightsquigarrow"
-  "leftrightsquigarrow" "looparrowleft" "looparrowright"
-  "circeq" "succsim" "gtrsim" "gtrapprox" "multimap"
-  "therefore" "because" "doteqdot" "triangleq"
-  "precsim" "lesssim" "lessapprox" "eqslantless"
-  "eqslantgtr" "curlyeqprec" "curlyeqsucc" "preccurlyeq"
-  "leqq" "leqslant" "lessgtr" "smalldash"
-  "risingdotseq" "fallingdotseq" "succcurlyeq" "geqq"
-  "geqslant" "gtrless" "sqsubset" "sqsupset"
-  "vartriangleright" "vartriangleleft" "trianglerighteq"
-  "trianglelefteq" "bigstar" "between"
-  "blacktriangledown" "blacktriangleright" "blacktriangleleft"
-  "shortrightarrow" "shortleftarrow" "vartriangle"
-  "blacktriangle" "triangledown" "eqcirc"
-  "lesseqgtr" "gtreqless" "lesseqqgtr" "gtreqqless"
-  "yen" "Rrightarrow" "Lleftarrow" "checkmark"
-  "veebar" "barwedge" "doublebarwedge" "angle" "measuredangle"
-  "sphericalangle" "varpropto" "smallsmile" "smallfrown" "Subset"
-  "Supset" "Cup" "Cap" "curlyvee" "curlywedge"
-  "leftthreetimes" "rightthreetimes" "subseteqq" "supseteqq"
-  "bumpeq" "Bumpeq" "lll" "ggg" "ulcorner" "urcorner"
-  "circledR" "circledS" "pitchfork" "dotplus" "backsim"
-  "backsimeq" "llcorner" "lrcorner" "maltese"
-  "complement" "intercal" "circledcirc" "circledast" "circleddash"
-  
-  "lvertneqq" "gvertneqq" "nleq" "ngeq" "nless" "ngtr"
-  "nprec" "nsucc" "lneqq" "gneqq" "nleqslant" "ngeqslant"
-  "lneq" "gneq" "npreceq" "nsucceq" "precnsim"
-  "succnsim" "lnsim" "gnsim" "nleqq" "ngeqq"
-  "precneqq" "succneqq" "precnapprox" "succnapprox"
-  "lnapprox" "gnapprox" "nsim" "ncong" "diagup"
-  "diagdown" "varsubsetneq" "varsupsetneq"
-  "nsubseteqq" "nsupseteqq" "subsetneqq" "supsetneqq"
-  "varsubsetneqq" "varsupsetneqq" "subsetneq" "supsetneq"
-  "nsubseteq" "nsupseteq" "parallel" "nparallel" "nmid" "nshortmid"
-  "nshortparallel" "nvdash" "nVdash" "nvDash" "nVDash" "models"
-  "ntrianglerighteq" "ntrianglelefteq" "ntriangleleft"
-  "ntriangleright" "nleftarrow" "nrightarrow" "nLeftarrow"
-  "nRightarrow" "nLeftrightarrow" "nleftrightarrow"
-  "divideontimes" "varnothing" "nexists"
-  "Mho" "thorn" "eth" "eqsim" "beth" "gimel" "daleth"
-  "lessdot" "gtrdot" "ltimes" "rtimes" "shortmid"
-  "shortparallel" "smallsetminus" "thicksim" "thickapprox"
-  "approxeq" "succapprox" "precapprox" "curvearrowleft"
-  "curvearrowright" "digamma" "varkappa" "" "hslash"
-  "hbar" "backepsilon"
-  
-  "mapsto" "longmapsto" "longrightarrow" "longleftarrow"
-  "longleftrightarrow" "longRightarrow" "longLeftarrow"
-  "longLeftrightarrow" "ldots" "cdots" "vdots"
-  "ddots" "dotamalg" "dottimes" "dotoplus" "dototimes" "venus"
-  "mars" "earth" "aries" "fullmoon" "newmoon" "astrosun"
-  "leo" "LEFTcircle" "RIGHTcircle" "Square" "CheckedBox" "photon" "gluon"
-  "neq" "notin" "nin" "notni" "nni" "nll" "nlll" "ngg" "nggg"
-  "dagger" "ddagger" "boxbar" "checked" "obar" "kreuz")
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Extensions to LaTeX
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -392,4 +395,4 @@
 
 (kbd-symbols
   "ddots" "mho" "Backepsilon" "homsim"
-  "mathd" "mathe" "mathi" "mathpi")
+  "mathcatalan" "mathd" "mathe" "matheuler" "mathi" "mathpi")
