@@ -102,15 +102,88 @@
 (tm-define (go-to-previous-tag lab)
   (go-to-same-buffer (lambda (t p) (path-previous-tag t p lab))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Cursor history
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define cursor-history (make-ahash-table))
+(define cursor-future (make-ahash-table))
+
+(define (history-get) (ahash-ref* cursor-history (window-get-id) '()))
+(define (history-set l) (ahash-set! cursor-history (window-get-id) l))
+(define (future-get) (ahash-ref* cursor-future (window-get-id) '()))
+(define (future-set l) (ahash-set! cursor-future (window-get-id) l))
+
+(define (cursor-same? l p)
+  (and (nnull? l) (== (position-get (car l)) p)))
+
+(tm-define (cursor-history-add p)
+  (:synopsis "Add current cursor position into the history")
+  (if (cursor-same? (future-get) p)
+      (with pos (car (future-get))
+	(future-set (cdr (future-get)))
+	(history-set (cons pos (history-get))))
+      (when (not (cursor-same? (history-get) p))
+	(with pos (position-new)
+	  (position-set pos p)
+	  (history-set (cons pos (history-get)))))))
+
+(define (position-valid? pos)
+  (and-with t (path->tree (cDr (position-get pos)))
+    (not (tm-func? t 'uninit))))
+
+(tm-define (cursor-has-history?)
+  (:synopsis "Does there exist a previous position in history?")
+  (nnull? (history-get)))
+
+(tm-define (cursor-history-backward)
+  (:synopsis "Go to previous position in history")
+  (when (nnull? (history-get))
+    (with pos (car (history-get))
+      (history-set (cdr (history-get)))
+      (if (position-valid? pos)
+	  (begin
+	    (future-set (cons pos (future-get)))
+	    (if (== (cursor-path) (position-get pos))
+		(cursor-history-backward)
+		(begin
+		  (go-to (position-get pos))
+		  (cursor-show-if-hidden))))
+	  (begin
+	    (position-delete pos)
+	    (cursor-history-backward))))))
+
+(tm-define (cursor-has-future?)
+  (:synopsis "Does there exist a next position in history?")
+  (nnull? (future-get)))
+
+(tm-define (cursor-history-forward)
+  (:synopsis "Go to next position in history")
+  (when (nnull? (future-get))
+    (with pos (car (future-get))
+      (future-set (cdr (future-get)))
+      (if (position-valid? pos)
+	  (begin
+	    (history-set (cons pos (history-get)))
+	    (if (== (cursor-path) (position-get pos))
+		(cursor-history-forward)
+		(begin
+		  (go-to (position-get pos))
+		  (cursor-show-if-hidden))))
+	  (begin
+	    (position-delete pos)
+	    (cursor-future-backward))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Changing buffers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (go-to-buffer name)
   (:argument  name "Switch to buffer")
-  (:proposals name (map car (get-buffer-menu)))
-  (let* ((l1 (assoc name (get-buffer-menu)))
-	 (l2 (assoc (string-append name " *") (get-buffer-menu))))
+  (:proposals name (map car (list-filter (get-buffer-menu) pair?)))
+  (let* ((m  (list-filter (get-buffer-menu) pair?))
+	 (l1 (assoc name m))
+	 (l2 (assoc (string-append name " *") m)))
     (cond (l1 ((cadr l1)))
 	  (l2 ((cadr l2)))
 	  (else (set-message (string-append "No buffer#" name)
@@ -122,8 +195,8 @@
     (eval cmd)
     (switch-to-active-buffer old)))
 
-(tm-define-macro (with-active-buffer . l)
-  (with-active-buffer-sub (car l) (cons 'begin (cdr l))))
+(tm-define-macro (with-active-buffer buf . body)
+  `(with-active-buffer-sub ,buf `(begin ,@body)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Search and replace
