@@ -18,6 +18,8 @@
 
 tree upgrade_tex (tree t);
 static bool textm_appendices= false;
+static bool textm_unicode   = false;
+static bool textm_natbib    = false;
 
 /******************************************************************************
 * Preprocess preamble
@@ -67,6 +69,7 @@ filter_preamble (tree t) {
 #define t2e parsed_text_to_tree
 #define m2e latex_modifier_to_tree
 #define var_m2e var_latex_modifier_to_tree
+#define v2e latex_verbarg_to_string
 tree l2e (tree);
 tree latex_command_to_tree (tree t);
 
@@ -78,14 +81,29 @@ latex_symbol_to_tree (string s) {
       if (s == "\\ ") return " ";
       if (s == "\\-") return "";
       if (s == "\\/") return "";
-      if (s == "\\i") return "\031";
-      if (s == "\\j") return "\032";
-      if (s == "\\oe") return "\367";
-      if (s == "\\ae") return "\346";
-      if (s == "\\ss") return "\377";
-      if (s == "\\OE") return "\327";
-      if (s == "\\AE") return "\306";
-      if (s == "\\SS") return "\337";
+      if (s == "\\AA") return "\xC5";
+      if (s == "\\AE") return "\xC6";
+      if (s == "\\DH") return "\xD0";
+      if (s == "\\L") return "\x8A";
+      if (s == "\\NG") return "\x8D";
+      if (s == "\\O") return "\xD8";
+      if (s == "\\OE") return "\xD7";
+      if (s == "\\S") return "\x9F";
+      if (s == "\\SS") return "\xDF";
+      if (s == "\\TH") return "\xDE";
+      if (s == "\\aa") return "\xE5";
+      if (s == "\\ae") return "\xE6";
+      if (s == "\\dh") return "\xF0";
+      if (s == "\\dj") return "\x9E";
+      if (s == "\\i") return "\x19";
+      if (s == "\\j") return "\x1A";
+      if (s == "\\l") return "\xAA";
+      if (s == "\\ng") return "\xAD";
+      if (s == "\\o") return "\xF8";
+      if (s == "\\oe") return "\xF7";
+      if (s == "\\ss") return "\xFF";
+      if (s == "\\th") return "\xFE";
+      if (s == "\\pounds") return "\xBF";
       if (s == "\\\\") return tree (FORMAT, "next line");
       if (s == "\\cr") return tree (FORMAT, "next line");
       if (s == "\\noindent")  return tree (FORMAT, "no first indentation");
@@ -119,6 +137,8 @@ latex_symbol_to_tree (string s) {
       if (s == "\\notin") return "<nin>";
       if (s == "\\addots") return "<udots>";
       if (s == "\\dots") return "<ldots>";
+      if (s == "\\tableofcontents")
+	return compound ("table-of-contents", "toc", tree (DOCUMENT, ""));
     }
 
     if (latex_type (s) == "texmacs") {
@@ -208,6 +228,10 @@ latex_symbol_to_tree (string s) {
     if ((N(s) > 5) && (s(0,5) == "\\end-"))
       return tree (END, s(5,N(s)));
 
+    if (starts (s, "\\#") && s != "\\#") {
+      textm_unicode= true;
+      return "<" * s (1, N(s)) * ">";
+    }
     return tree (APPLY, s(1,N(s)));
   }
   if ((N(s) == 2) && (s[0] == '#') && (s[1] >= '0') && (s[1] <= '9'))
@@ -354,19 +378,9 @@ var_m2e (tree t, string var, string val) {
 	       tree (RESET, copy (var)));
 }
 
-tree
-latex_verbarg_to_tree (tree t) {
-  t= l2e (t);
-  if (is_concat (t)) {
-    string s;
-    int i, n= N(t);
-    for (i=0; i<n; i++)
-      if (is_atomic (t[i])) s << t[i]->label;
-      else if (is_func (t[i], RSUB, 1) && is_atomic (t[i][0]))
-	s << "_" << t[i][0]->label;
-    return s;
-  }
-  else return "";
+string
+v2e (tree t) {
+  return string_arg (t2e (t, false));
 }
 
 static bool
@@ -499,10 +513,12 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\~", 1)) return latex_accent (t[1], "~");
   if (is_tuple (t, "\\`", 1)) return latex_accent (t[1], "<grave>");
   if (is_tuple (t, "\\'", 1)) return latex_accent (t[1], "<acute>");
-  if (is_tuple (t, "\\\"", 1)) return latex_accent (t[1], "<ddot>");
+  if (is_tuple (t, "\\\"", 1)) return latex_accent (t[1], "<ddot>"); // diaeresis
   if (is_tuple (t, "\\.", 1)) return latex_accent (t[1], "<dot>");
   if (is_tuple (t, "\\u", 1)) return latex_accent (t[1], "<breve>");
-  if (is_tuple (t, "\\v", 1)) return latex_accent (t[1], "<check>");
+  if (is_tuple (t, "\\v", 1)) return latex_accent (t[1], "<check>"); // caron
+  if (is_tuple (t, "\\=", 1)) return latex_accent (t[1], "<bar>");   // macron
+
 
   if (is_tuple (t, "\\textrm", 1)) return m2e (t, FONT_FAMILY, "rm");
   if (is_tuple (t, "\\texttt", 1)) return m2e (t, FONT_FAMILY, "tt");
@@ -614,19 +630,49 @@ latex_command_to_tree (tree t) {
   }
   if (is_tuple (t, "\\cite", 1) || is_tuple (t, "\\nocite", 1)) {
     string cite_type= t[0]->label (1, N(t[0]->label));
-    string s= string_arg (t2e(t[1]));
+    string s= v2e (t[1]);
     return latex_cite_to_tree (cite_type, s);
   }
   if (is_tuple (t, "\\cite*", 2)) {
     tree   ot= t2e(t[1])->label;
-    string s = string_arg (t2e(t[2]));
+    string s = v2e (t[2]);
     tree   ct= latex_cite_to_tree ("cite", s);
+    if (N(ct) == 2) return compound ("cite-detail", ct[1], ot);
     return tree (CONCAT, ct, " (", ot, ")");
   }
   if (is_tuple (t, "\\citedetail", 2))
     return compound ("cite-detail", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\citet", 1) || is_tuple (t, "\\citep", 1) ||
+      is_tuple (t, "\\citet*", 1) || is_tuple (t, "\\citep*", 1) ||
+      is_tuple (t, "\\citealt", 1) || is_tuple (t, "\\citealp", 1) ||
+      is_tuple (t, "\\citealt*", 1) || is_tuple (t, "\\citealp*", 1))
+    {
+      textm_natbib= true;
+      string star= "";
+      string cite_type= t[0]->label (1, N(t[0]->label));
+      if (ends (cite_type, "*")) {
+	star= "*"; cite_type= cite_type (0, N (cite_type) - 1); }
+      if (cite_type == "citet") cite_type= "cite-textual" * star;
+      if (cite_type == "citep") cite_type= "cite-parenthesized" * star;
+      if (cite_type == "citealt") cite_type= "cite-raw" * star;
+      if (cite_type == "citealp") cite_type= "cite-raw" * star;
+      string s= v2e (t[1]);
+      return latex_cite_to_tree (cite_type, s);
+    }
+  if (is_tuple (t, "\\citetext", 1))
+    return compound ("render-cite", l2e (t[1]));
+  if (is_tuple (t, "\\citeauthor", 1)) {
+    textm_natbib= true; return compound ("cite-author-link", t2e (t[1])); }
+  if (is_tuple (t, "\\citeauthor*", 1)) {
+    textm_natbib= true; return compound ("cite-author*-link", t2e (t[1])); }
+  if (is_tuple (t, "\\citeyear", 1)) {
+    textm_natbib= true; return compound ("cite-year-link", t2e (t[1])); }
+  if (is_tuple (t, "\\bibitem", 1))
+    return compound ("bibitem", v2e (t[1]));
+  if (is_tuple (t, "\\bibitem*", 2))
+    return compound ("bibitem*", v2e (t[1]), v2e (t[2]));
   if (is_tuple (t, "\\index", 1)) {
-    string s= string_arg (t2e (t[1]));
+    string s= v2e (t[1]);
     return latex_index_to_tree (s);
   }
   if (is_tuple (t, "\\displaylines", 1)) {
@@ -641,7 +687,7 @@ latex_command_to_tree (tree t) {
     return r;
   }
   if (is_tuple (t, "\\includegraphics", 1)) {
-    tree name= latex_verbarg_to_tree (t[1]);
+    tree name= v2e (t[1]);
     if (name == "") return "";
     else {
       tree g (POSTSCRIPT, 7);
@@ -651,6 +697,13 @@ latex_command_to_tree (tree t) {
   }
   if (is_tuple (t, "\\noalign", 1))
     return ""; // FIXME: for larger space in maple matrices
+  if (is_tuple (t, "\\etalchar", 1)) return t2e (t[1]);
+  if (is_tuple (t, "\\natexlab", 1)) return t2e (t[1]);
+  if (is_tuple (t, "\\penalty", 1)) return "";
+  if (is_tuple (t, "\\url", 1))
+    return tree (APPLY, "slink", t2e (t[1]));
+  if (is_tuple (t, "\\href", 2))
+    return tree (APPLY, "hlink", l2e (t[2]), t2e (t[1]));
 
   // Start TeXmacs specific markup
   if (is_tuple (t, "\\tmmathbf", 1))
@@ -1262,10 +1315,16 @@ finalize_textm (tree t) {
 
 tree
 latex_to_tree (tree t1) {
-  string style;
+  string style, lan= "";
   bool is_document= is_compound (t1, "!file", 1);
   if (is_document) t1= t1[0];
+  if (is_compound (t1, "!language", 2)) {
+    lan= t1[1]->label;
+    t1 = t1[0];
+  }
   textm_appendices= false;
+  textm_unicode   = false;
+  textm_natbib    = false;
   command_type ("!em") = "false";
   // cout << "\n\nt1= " << t1 << "\n\n";
   tree t2= is_document? filter_preamble (t1): t1;
@@ -1289,9 +1348,34 @@ latex_to_tree (tree t1) {
   // cout << "\n\nt10= " << t10 << "\n\n";
   tree t11= simplify_correct (t10);
   // cout << "\n\nt11= " << t11 << "\n\n";
+
   if (!exists (url ("$TEXMACS_STYLE_PATH", style * ".ts")))
     style= "generic";
-  if (is_document)
-    return tree (DOCUMENT, compound ("body", t11), compound ("style", style));
-  else return t10;
+  tree initial (COLLECTION), mods (WITH);
+  if (textm_unicode) {
+    string name= "modern";
+    if (lan == "chinese") name= "fireflysung";
+    //if (lan == "japanese") name= "ipa";
+    //if (lan == "korean") name= "unbatang";
+    if (lan == "taiwanese") name= "fireflysung";
+    initial << tree (ASSOCIATE, FONT, name);
+    mods << tree (FONT) << tree (name);
+  }
+  if (lan != "") {
+    initial << tree (ASSOCIATE, LANGUAGE, lan);
+    mods << tree (LANGUAGE) << tree (lan);
+  }
+  if (is_document) {
+    tree the_body   = compound ("body", t11);
+    tree the_style  = compound ("style", style);
+    tree the_initial= compound ("initial", initial);
+    if (textm_natbib)
+      the_style= compound ("style", tuple (style, "cite-author-year"));
+    if (N (initial) == 0) return tree (DOCUMENT, the_style, the_body);
+    else return tree (DOCUMENT, the_style, the_body, the_initial);
+  }
+  else {
+    if (N (mods) > 0) { mods << t10; return mods; }
+    return t10;
+  }
 }
