@@ -38,6 +38,20 @@ get_codes (string version) {
   hashmap<string,int> H (UNKNOWN);
   H->join (STD_CODE);
 
+  if (version_inf ("1.0.6.9", version)) return H;
+
+  rename_feature (H, "frozen", "freeze");
+
+  if (version_inf ("1.0.6.2", version)) return H;
+
+  new_feature (H, "expand-as");
+  new_feature (H, "locus");
+  new_feature (H, "id");
+  new_feature (H, "hard-id");
+  new_feature (H, "link");
+  new_feature (H, "url");
+  new_feature (H, "script");
+
   if (version_inf ("1.0.4.1", version)) return H;
 
   new_feature (H, "copy");
@@ -396,7 +410,7 @@ upgrade_textual (tree t, path& mode_stack) {
 	else s << ss;
       }
       else if (((t->label[i] == '\'') || (t->label[i] == '`')) &&
-	       (!nil (mode_stack)) && (mode_stack->item == 1))
+	       (!is_nil (mode_stack)) && (mode_stack->item == 1))
 	{
 	  int start= i++;
 	  while ((i<n) && (t->label[i] == t->label[i-1])) i++;
@@ -422,7 +436,7 @@ upgrade_textual (tree t, path& mode_stack) {
       }
       else if (is_func (u, RESET)) {
 	if (u == tree (RESET, "mode"))
-	  if (!nil (mode_stack))
+	  if (!is_nil (mode_stack))
 	    mode_stack= mode_stack->next;
       }
       else if (is_func (u, BEGIN, 1)) {
@@ -437,7 +451,7 @@ upgrade_textual (tree t, path& mode_stack) {
 	    (u[0] == "equation*") ||
 	    (u[0] == "eqnarray*") ||
 	    (u[0] == "leqnarray*"))
-	  if (!nil (mode_stack))
+	  if (!is_nil (mode_stack))
 	    mode_stack= mode_stack->next;
       }
       if (is_concat (t) && is_concat (u)) r << A(u);
@@ -468,7 +482,7 @@ upgrade_apply_expand_value (tree t, hashset<string> H) {
   }
 }
 
-typedef char* charp;
+typedef const char* charp;
 static charp apply_expand_value_strings[]= {
   "part", "part*", "chapter", "chapter*", "appendix",
   "section", "section*", "subsection", "subsection*",
@@ -2636,6 +2650,94 @@ upgrade_textat (tree t) {
   return r;
 }
 
+/******************************************************************************
+* Upgrade cell alignment
+******************************************************************************/
+
+tree
+upgrade_cell_alignment (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  if (is_func (t, CWITH) && (N(t) >= 2))
+    if (t[N(t)-2] == CELL_HALIGN)
+      if (t[N(t)-1] == "." || t[N(t)-1] == ",") {
+	tree r= copy (t);
+	r[N(t)-1]= "L" * t[N(t)-1]->label;
+	return r;
+      }
+  int n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= upgrade_cell_alignment (t[i]);
+  return r;
+}
+
+/******************************************************************************
+* Renaming primitives
+******************************************************************************/
+
+tree
+rename_primitive (tree t, string which, string by) {
+  int i;
+  if (is_atomic (t)) return t;
+  int n= N(t);
+  tree r (t, n);
+  if (is_compound (t, which))
+    r= tree (make_tree_label (by), n);
+  for (i=0; i<n; i++)
+    r[i]= rename_primitive (t[i], which, by);
+  return r;
+}
+
+/******************************************************************************
+* Upgrade label assignment
+******************************************************************************/
+
+tree
+upgrade_label_assignment (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_func (t, ASSIGN, 2) && t[0] == "the-label")
+    return tree (SET_BINDING, t[1]);
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_label_assignment (t[i]);
+    return r;
+  }
+}
+
+/******************************************************************************
+* Upgrade scheme documentation
+******************************************************************************/
+
+tree
+upgrade_scheme_doc (tree t) {
+  int i;
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "scm-fun", 1) ||
+	   is_compound (t, "scm-macro", 1))
+    return compound ("scm", t[0]);
+  else if (is_compound (t, "explain-scm-fun") ||
+	   is_compound (t, "explain-scm-macro"))
+    {
+      tree r (CONCAT);
+      r << "(" << t[0];
+      for (int i=1; i<N(t); i++)
+	r << " " << t[i];
+      r << ")";
+      return compound ("scm", simplify_concat (r));
+    }
+  else {
+    int n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= upgrade_scheme_doc (t[i]);
+    return r;
+  }
+}
+
 /******************************************************************************
 * Upgrade from previous versions
 ******************************************************************************/
@@ -2669,6 +2771,7 @@ upgrade_tex (tree t) {
 
 tree
 upgrade (tree t, string version) {
+  // cout << "Upgrade from " << version << "\n";
   if (version_inf (version, "0.3.1.9")) {
     path p;
     t= upgrade_textual (t, p);
@@ -2739,5 +2842,13 @@ upgrade (tree t, string version) {
     t= upgrade_graphics (t);
   if (version_inf_eq (version, "1.0.5.11"))
     t= upgrade_textat (t);
+  if (version_inf_eq (version, "1.0.6.1"))
+    t= upgrade_cell_alignment (t);
+  if (version_inf_eq (version, "1.0.6.2"))
+    t= rename_primitive (t, "hyper-link", "hlink");
+  if (version_inf_eq (version, "1.0.6.2"))
+    t= upgrade_label_assignment (t);
+  if (version_inf_eq (version, "1.0.6.10"))
+    t= upgrade_scheme_doc (t);
   return t;
 }
