@@ -509,6 +509,16 @@ xml_unspace (string s, bool first, bool last) {
   return r;
 }
 
+bool
+contains_unicode_char (string s) {
+  int i= 0, n= N(s);
+  while (i+1<n) {
+    if (s[i] == '<' && s[i+1] == '#') return true;
+    tm_char_forwards (s, i);
+  }
+  return false;
+}
+
 /******************************************************************************
 * Roman and alpha numbers
 ******************************************************************************/
@@ -568,7 +578,7 @@ fnsymbol_nr (int nr) {
 * Conversions to and from hexadecimal
 ******************************************************************************/
 
-static char* hex_string= "0123456789ABCDEF";
+static const char* hex_string= "0123456789ABCDEF";
 
 string
 as_hexadecimal (int i) {
@@ -577,6 +587,14 @@ as_hexadecimal (int i) {
   return as_hexadecimal (i >> 4) * hex_string [i & 15];
 }
 
+string
+as_hexadecimal (pointer ptr) {
+  intptr_t i= (intptr_t) ptr;
+  if (i<0) return "-" * as_hexadecimal (-i);
+  if (i<16) return hex_string [i & 15];
+  return as_hexadecimal (i >> 4) * hex_string [i & 15];
+}
+
 string
 as_hexadecimal (int i, int len) {
   if (len==1) return hex_string [i & 15];
@@ -635,6 +653,24 @@ tm_decode (string s) {
   return r;
 }
 
+string
+tm_var_encode (string s) {
+  register int i, n= N(s);
+  string r;
+  for (i=0; i<n; i++) {
+    if (s[i]=='<') {
+      if (i+1 < n && s[i+1] == '#') {
+	while (i<n && s[i] != '>') r << s[i++];
+	if (i<n) r << s[i];
+      }
+      else r << "<less>";
+    }
+    else if (s[i]=='>') r << "<gtr>";
+    else r << s[i];
+  }
+  return r;
+}
+
 string
 tm_correct (string s) {
   register int i;
@@ -837,7 +873,7 @@ starts (string s, const string what) {
 
 bool
 ends (string s, const char* what) {
-  string r ((char*) what);
+  string r (what);
   if (N(r) > N(s)) return false;
   return s (N(s)-N(r), N(s)) == r;
 }
