@@ -17,6 +17,9 @@
 #include "analyze.hpp"
 #include "timer.hpp"
 #include "Bridge/impl_typesetter.hpp"
+#ifdef EXPERIMENTAL
+#include "../../Style/Environment/std_environment.hpp"
+#endif // EXPERIMENTAL
 
 //box empty_box (path ip, int x1=0, int y1=0, int x2=0, int y2=0);
 
@@ -35,7 +38,7 @@ edit_typeset_rep::edit_typeset_rep ():
   the_style (TUPLE),
   cur (hashmap<string,tree> (UNINIT)),
   pre (UNINIT), init (UNINIT), fin (UNINIT),
-  env (dis, drd, is_aux (buf->name)? buf->extra: buf->name,
+  env (drd, is_aux (buf->name)? buf->extra: buf->name,
        buf->ref, (buf->prj==NULL? buf->ref: buf->prj->ref),
        buf->aux, (buf->prj==NULL? buf->aux: buf->prj->aux)),
   ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {}
@@ -100,6 +103,16 @@ edit_typeset_rep::divide_lengths (string l1, string l2) {
 * Processing preamble
 ******************************************************************************/
 
+void
+use_modules (tree t) {
+  if (is_tuple (t))
+    for (int i=0; i<N(t); i++) {
+	string s= as_string (t[i]);
+	if (starts (s, "(")) eval ("(use-modules " * s * ")");
+	else if (s != "") eval ("(plugin-initialize '" * s * ")");
+    }
+}
+
 void
 edit_typeset_rep::typeset_style_use_cache (tree style) {
   bool ok;
@@ -108,7 +121,7 @@ edit_typeset_rep::typeset_style_use_cache (tree style) {
   SERVER (style_get_cache (style, H, t, ok));
   if (ok) {
     env->patch_env (H);
-    ok = drd->set_locals (t);
+    ok= drd->set_locals (t);
   }
   if (!ok) {
     if (!is_tuple (style))
@@ -120,6 +133,7 @@ edit_typeset_rep::typeset_style_use_cache (tree style) {
     drd->heuristic_init (H);
     SERVER (style_set_cache (style, H, drd->get_locals ()));
   }
+  use_modules (env->read (THE_MODULES));
 }
 
 void
@@ -147,6 +161,19 @@ edit_typeset_rep::drd_update () {
   drd->heuristic_init (cur[tp]);
 }
 
+#ifdef EXPERIMENTAL
+void
+edit_typeset_rep::environment_update () {
+  hashmap<string,tree> h;
+  typeset_prepare ();
+  env->assign ("base-file-name", as_string (env->base_file_name));
+  env->assign ("cur-file-name", as_string (env->cur_file_name));
+  env->assign ("secure", bool_as_tree (env->secure));
+  env->read_env (h);
+  ::primitive (ste, h);
+}
+#endif
+
 /******************************************************************************
 * Routines for getting information
 ******************************************************************************/
@@ -187,6 +214,16 @@ edit_typeset_rep::get_env_value (string var, path p) {
 
 tree
 edit_typeset_rep::get_env_value (string var) {
+ /* FIXME: tp is wrong (and consequently, crashes TeXmacs)
+  *   when we call this routine from inside the code which
+  *   is triggered by a button, for example.
+  *
+  * Test: fire TeXmacs, then open a new Graphics, then click
+  *   on the icon for going in spline mode. Then it crashes,
+  *   because we call (get-env-tree) from inside the Scheme.
+  *   If we call (get-env-tree-at ... (cDr (cursor-path))),
+  *   then it works.
+  */
   return get_env_value (var, tp);
 }
 
@@ -307,6 +344,11 @@ edit_typeset_rep::exec_texmacs (tree t, path p) {
   return exec (t, cur[p]);
 }
 
+tree
+edit_typeset_rep::exec_texmacs (tree t) {
+  return exec_texmacs (t, rp * 0);
+}
+
 tree
 edit_typeset_rep::exec_html (tree t, path p) {
   if (p == (rp * 0)) typeset_preamble ();
@@ -323,6 +365,22 @@ edit_typeset_rep::exec_html (tree t) {
   return exec_html (t, rp * 0);
 }
 
+static tree
+value_to_compound (tree t, hashmap<string,tree> h) {
+  if (is_atomic (t)) return t;
+  else if (is_func (t, VALUE, 1) &&
+	   is_atomic (t[0]) &&
+	   h->contains (t[0]->label))
+    return compound (t[0]->label);
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= value_to_compound (t[i], h);
+    return r;
+  }
+}
+
 tree
 edit_typeset_rep::exec_latex (tree t, path p) {
   string pref= "texmacs->latex:expand-macros";
@@ -336,11 +394,11 @@ edit_typeset_rep::exec_latex (tree t, path p) {
   if (is_document (t) && is_compound (t[0], "hide-preamble")) {
     tree r= copy (t);
     r[0]= "";
-    r= exec (r, H, false);
+    r= exec (value_to_compound (r, P), H, false);
     r[0]= exec (t[0], H, false);
     return r;
   }
-  else return exec (t, H, false);
+  else return exec (value_to_compound (t, P), H, false);
 }
 
 tree
@@ -359,10 +417,6 @@ edit_typeset_rep::texmacs_exec (tree t) {
 
 void
 edit_typeset_rep::init_style () {
-  bool old_need_save    = buf->need_save;
-  bool old_need_autosave= buf->need_autosave;
-  buf->need_save    = old_need_save;
-  buf->need_autosave= old_need_autosave;
   notify_change (THE_ENVIRONMENT);
 }
 
@@ -428,6 +482,14 @@ edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
   bench_end ("typeset");
 }
 
+void
+edit_typeset_rep::typeset_invalidate (path p) {
+  if (rp <= p) {
+    notify_change (THE_TREE);
+    ::notify_assign (ttt, p-rp, subtree (et, p));
+  }
+}
+
 void
 edit_typeset_rep::typeset_invalidate_all () {
   notify_change (THE_ENVIRONMENT);
