@@ -14,9 +14,15 @@
 #include "file.hpp"
 #include "convert.hpp"
 #include "server.hpp"
-#include "tm_buffer.hpp"
+#include "tm_window.hpp"
 #include "Metafont/tex_files.hpp"
 #include "data_cache.hpp"
+#include "drd_mode.hpp"
+#include "message.hpp"
+#ifdef EXPERIMENTAL
+#include "../../Style/Memorizer/clean_copy.hpp"
+#include "../../Style/Evaluate/evaluate_main.hpp"
+#endif
 
 extern void (*env_next_prog)(void);
 extern void selection_correct (tree t, path i1, path i2, path& o1, path& o2);
@@ -42,20 +48,24 @@ edit_interface_rep::edit_interface_rep ():
   do_animate (false), next_animate (last_change-1),
   full_screen (false), got_focus (false),
   sh_s (""), sh_len (0),
-  popup_win (NULL),
+  popup_win (),
   sfactor (sv->get_default_shrinking_factor ()),
   pixel (sfactor*PIXEL), copy_always (),
   last_click (0), last_x (0), last_y (0), dragging (false),
   made_selection (false), table_selection (false),
-  oc (0, 0), shadow (NULL), stored (NULL)
+  oc (0, 0), temp_invalid_cursor (false),
+  shadow (NULL), stored (NULL),
+  cur_sb (2), cur_wb (2)
 {
   input_mode= INPUT_NORMAL;
+  gui_root_extents (cur_wx, cur_wy);
 }
 
 edit_interface_rep::~edit_interface_rep () {
-  if (attached ()) {
-    win->delete_shadow (shadow);
-    win->delete_shadow (stored);
+  if (is_attached (this)) {
+    renderer ren= get_renderer (this);
+    ren->delete_shadow (shadow);
+    ren->delete_shadow (stored);
   }
 }
 
@@ -67,9 +77,10 @@ void
 edit_interface_rep::suspend () {
   got_focus= false;
   notify_change (THE_FOCUS);
-  if (attached ()) {
-    win->delete_shadow (shadow);
-    win->delete_shadow (stored);
+  if (is_attached (this)) {
+    renderer ren= get_renderer (this);
+    ren->delete_shadow (shadow);
+    ren->delete_shadow (stored);
   }
 }
 
@@ -80,20 +91,11 @@ edit_interface_rep::resume () {
   SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
   SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
   SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
+  cur_sb= 2;
   tp= make_cursor_accessible (tp, true);
   notify_change (THE_FOCUS + THE_EXTENTS + THE_CURSOR);
 }
 
-display
-edit_interface_rep::get_display () {
-  return dis;
-}
-
-widget
-edit_interface_rep::get_widget () {
-  return widget (this);
-}
-
 /******************************************************************************
 * Routines for dealing with shrinked coordinates
 ******************************************************************************/
@@ -110,19 +112,18 @@ edit_interface_rep::set_shrinking_factor (int sf) {
     pixel  = sf*PIXEL;
     init_env (SFACTOR, as_string (sf));
     notify_change (THE_ENVIRONMENT);
-    notify_change (THE_AUTOMATIC_SIZE);
   }
 }
 
 void
 edit_interface_rep::invalidate (SI x1, SI y1, SI x2, SI y2) {
-  this << emit_invalidate ((x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
-			   (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor);
+  send_invalidate (this, (x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
+		         (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor);
 }
 
 void
 edit_interface_rep::invalidate (rectangles rs) {
-  while (!nil (rs)) {
+  while (!is_nil (rs)) {
     invalidate (rs->item->x1-pixel, rs->item->y1-pixel,
 		rs->item->x2+pixel, rs->item->y2+pixel);
     rs= rs->next;
@@ -162,17 +163,67 @@ edit_interface_rep::set_extents (SI x1, SI y1, SI x2, SI y2) {
 
 void
 edit_interface_rep::cursor_visible () {
+  path sp= find_innermost_scroll (eb, tp);
   cursor cu= get_cursor ();
-  cu->y1 -= 2*pixel; cu->y2 += 2*pixel;
-  update_visible ();
-  if ((cu->ox+ ((SI) (cu->y1 * cu->slope)) <  vx1) ||
-      (cu->ox+ ((SI) (cu->y2 * cu->slope)) >= vx2) ||
-      (cu->oy+ cu->y1 <  vy1) ||
-      (cu->oy+ cu->y2 >= vy2))
-    {
-      scroll_to (cu->ox- ((vx2-vx1)>>1), cu->oy+ ((vy2-vy1)>>1));
-      this << emit_invalidate_all ();
-    }
+  if (is_nil (sp)) {
+    update_visible ();
+    cu->y1 -= 2*pixel; cu->y2 += 2*pixel;
+    if ((cu->ox+ ((SI) (cu->y1 * cu->slope)) <  vx1) ||
+	(cu->ox+ ((SI) (cu->y2 * cu->slope)) >= vx2) ||
+	(cu->oy+ cu->y1 <  vy1) ||
+	(cu->oy+ cu->y2 >= vy2))
+      {
+	scroll_to (cu->ox- ((vx2-vx1)>>1), cu->oy+ ((vy2-vy1)>>1));
+	send_invalidate_all (this);
+      }
+  }
+  else {
+    SI x, y, sx, sy;
+    rectangle outer, inner;
+    find_canvas_info (eb, sp, x, y, sx, sy, outer, inner);
+    if ((cu->ox+ ((SI) (cu->y1 * cu->slope)) < x + outer->x1) ||
+	(cu->ox+ ((SI) (cu->y2 * cu->slope)) > x + outer->x2))
+      {
+	SI tx= inner->x2 - inner->x1;
+	SI cx= outer->x2 - outer->x1;
+	if (tx > cx) {
+	  SI outer_cx= cu->ox - x;
+	  SI inner_cx= outer_cx - sx;
+	  SI dx= inner_cx - inner->x1;
+	  double p= 100.0 * ((double) (dx - (cx>>1))) / ((double) (tx-cx));
+	  p= max (min (p, 100.0), 0.0);
+	  tree old_xt= eb[path_up (sp)]->get_info ("scroll-x");
+	  tree new_xt= as_string (p) * "%";
+	  if (new_xt != old_xt && is_accessible (obtain_ip (old_xt))) {
+	    object fun= symbol_object ("tree-set");
+	    object cmd= list_object (fun, old_xt, new_xt);
+	    eval_delayed (cmd);
+	    temp_invalid_cursor= true;
+	  }
+	}
+      }
+    if ((cu->oy+ cu->y1 < y + outer->y1) ||
+	(cu->oy+ cu->y2 > y + outer->y2))
+      {
+	SI ty= inner->y2 - inner->y1;
+	SI cy= outer->y2 - outer->y1;
+	if (ty > cy) {
+	  SI outer_cy= cu->oy + ((cu->y1 + cu->y2) >> 1) - y;
+	  SI inner_cy= outer_cy - sy;
+	  SI dy= inner_cy - inner->y1;
+	  double p= 100.0 * ((double) (dy - (cy>>1))) / ((double) (ty-cy));
+	  p= max (min (p, 100.0), 0.0);
+	  tree old_yt= eb[path_up (sp)]->get_info ("scroll-y");
+	  tree new_yt= as_string (p) * "%";
+	  if (new_yt != old_yt && is_accessible (obtain_ip (old_yt))) {
+	    object fun= symbol_object ("tree-set");
+	    object cmd= list_object (fun, old_yt, new_yt);
+	    eval_delayed (cmd);
+	    temp_invalid_cursor= true;
+	  }
+	}
+      }
+  }
 }
 
 void
@@ -180,12 +231,9 @@ edit_interface_rep::selection_visible () {
   update_visible ();
   if ((vx2 - vx1 <= 80*pixel) || (vy2 - vy1 <= 80*pixel)) return;
 
-  bool scroll_x=
-    (end_x <  vx1 + 20*pixel) ||
-    (end_x >= vx2 - 20*pixel);
-  bool scroll_y=
-    (end_y <  vy1 + 20*pixel) ||
-    (end_y >= vy2 - 20*pixel);
+  SI extra= (cur_sb == 1? 20 * pixel: 0);
+  bool scroll_x= (end_x < vx1 + extra) || (end_x >= vx2 - extra);
+  bool scroll_y= (end_y < vy1 + extra) || (end_y >= vy2 - extra);
   SI new_x= vx1;
   if (scroll_x) new_x= end_x - ((vx2-vx1)>>1);
   SI new_y= vy2;
@@ -193,7 +241,7 @@ edit_interface_rep::selection_visible () {
 
   if (scroll_x || scroll_y) {
     scroll_to (new_x, new_y);
-    this << emit_invalidate_all ();
+    send_invalidate_all (this);
     SI old_vx1= vx1, old_vy1= vy1;
     update_visible ();
     end_x += vx1- old_vx1;
@@ -231,17 +279,23 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
        is_compound (subtree (et, path_up (p)), "input")))
     compute_env_rects (p, rs, recurse);
   else {
-    bool right;
-    path p1= p * 0, p2= p * 1, q1, q2;
-    if (is_script (subtree (et, p), right)) {
-      p1= start (et, p * 0);
-      p2= end   (et, p * 0);
+    int new_mode= DRD_ACCESS_NORMAL;
+    if (get_init_string (MODE) == "src") new_mode= DRD_ACCESS_SOURCE;
+    int old_mode= set_access_mode (new_mode);
+    tree st= subtree (et, p);
+    if (is_accessible_cursor (et, p * right_index (st)) || in_source ()) {
+      bool right;
+      path p1= p * 0, p2= p * 1, q1, q2;
+      if (is_script (subtree (et, p), right)) {
+	p1= start (et, p * 0);
+	p2= end   (et, p * 0);
+      }
+      if (is_func (st, CELL)) { q1= p1; q2= p2; }
+      else selection_correct (et, p1, p2, q1, q2);
+      selection sel= eb->find_check_selection (q1, q2);
+      rs << outline (sel->rs, pixel);
     }
-    if (is_func (st, CELL)) { q1= p1; q2= p2; }
-    else selection_correct (et, p1, p2, q1, q2);
-    selection sel= eb->find_check_selection (q1, q2);
-    rs << simplify (::correct (thicken (sel->rs, pixel, 3*pixel) -
-			       thicken (sel->rs, 0, 2*pixel)));
+    set_access_mode (old_mode);
     if (recurse) compute_env_rects (p, rs, recurse);
   }
 }
@@ -263,8 +317,8 @@ edit_interface_rep::has_changed (int question) {
 int
 edit_interface_rep::idle_time (int event_type) {
   if (env_change == 0 &&
-      win->repainted () &&
-      (!dis->check_event (event_type)) &&
+      get_renderer (this) -> repainted () &&
+      (!check_event (event_type)) &&
       got_focus)
     return texmacs_time () - last_change;
   else return 0;
@@ -290,7 +344,7 @@ edit_interface_rep::apply_changes () {
 	SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
 	SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
 	set_footer ();
-	if (!win->interrupted ()) drd_update ();
+	if (!get_renderer (this) -> interrupted ()) drd_update ();
 	cache_memorize ();
 	last_update= last_change;
       }
@@ -303,18 +357,41 @@ edit_interface_rep::apply_changes () {
   update_visible ();
 
   // cout << "Handling automatic resizing\n";
-  if (env_change & THE_AUTOMATIC_SIZE) {
-    if (get_init_string (PAGE_MEDIUM) == "automatic") {
-      if (!attached())
-	fatal_error ("Window not attached",
-		     "edit_interface_rep::apply_changes");
-      SI wx, wy;
-      win->get_size (wx, wy);
-      init_env (PAGE_SCREEN_WIDTH, as_string ((wx-20*PIXEL)*sfactor) * "tmpt");
+  int sb= 1;
+  if (is_attached (this) && get_init_string (PAGE_MEDIUM) == "automatic") {
+    SI wx, wy;
+    ::get_size (get_window (this), wx, wy);
+    if (get_init_string (SCROLL_BARS) == "false") sb= 0;
+    if (get_server () -> in_full_screen_mode ()) sb= 0;
+    if (sb) wx -= 20 * PIXEL;
+    if (wx != cur_wx || wy != cur_wy) {
+      cur_wx= wx; cur_wy= wy;
+      init_env (PAGE_SCREEN_WIDTH, as_string (wx*sfactor) * "tmpt");
       init_env (PAGE_SCREEN_HEIGHT, as_string (wy*sfactor) * "tmpt");
       notify_change (THE_ENVIRONMENT);
     }
   }
+  if (sb != cur_sb) {
+    cur_sb= sb;
+    tm_window win= get_server () -> get_window ();
+    win -> set_scrollbars (sb);
+  }
+
+  // window decorations (menu bar, icon bars, footer)
+  int wb= 2;
+  if (is_attached (this)) {
+    string val= get_init_string (WINDOW_BARS);
+    if (val == "auto") wb= 2;
+    else if (val == "false") wb= 0;
+    else if (val == "true") wb= 1;
+    if (wb != cur_wb) {
+      cur_wb= wb;
+      if (wb != 2) {
+	get_server () -> show_header (wb);
+	get_server () -> show_footer (wb);
+      }
+    }
+  }
 
   // cout << "Handling selection\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION)) {
@@ -340,11 +417,29 @@ edit_interface_rep::apply_changes () {
     the_ghost_cursor()= eb->find_check_cursor (tp);
   }
 
+#ifdef EXPERIMENTAL
+  if (env_change & THE_ENVIRONMENT)
+    environment_update ();
+  if (env_change & THE_TREE) {
+    //global_trace_subtree (rp);
+    tree raw= global_get_subtree (rp);
+    cout << HRULE;
+    mem= evaluate (ste, raw);
+    tree rew= mem->get_tree ();
+    //cout << HRULE;
+    //print_tree (mem);
+    cout << HRULE;
+    cout << tree_to_texmacs (rew) << LF;
+    //print_tree (rew);
+  }
+#endif
+
   // cout << "Handling extents\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
     set_extents (eb->x1, eb->y1, eb->x2, eb->y2);
 
   // cout << "Cursor\n";
+  temp_invalid_cursor= false;
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS+
 		    THE_CURSOR+THE_FOCUS)) {
     SI /*P1= pixel,*/ P2= 2*pixel, P3= 3*pixel;
@@ -393,8 +488,19 @@ edit_interface_rep::apply_changes () {
     }
   }
 
+  // cout << "Handling locus highlighting\n";
+  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
+    update_active_loci ();
+  if (env_change & THE_LOCUS) {
+    if (locus_new_rects != locus_rects) {
+      invalidate (locus_rects);
+      invalidate (locus_new_rects);
+      locus_rects= locus_new_rects;
+    }
+  }
+
   // cout << "Handling backing store\n";
-  if (!nil (stored_rects)) {
+  if (!is_nil (stored_rects)) {
     if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION+THE_EXTENTS))
       stored_rects= rectangles ();
   }
@@ -402,14 +508,14 @@ edit_interface_rep::apply_changes () {
     SI gx1, gy1, gx2, gy2;
     if (find_graphical_region (gx1, gy1, gx2, gy2)) {
       rectangle gr= rectangle (gx1, gy1, gx2, gy2);
-      if (!nil (gr - stored_rects))
+      if (!is_nil (gr - stored_rects))
 	invalidate (gx1, gy1, gx2, gy2);
     }
   }
 
   // cout << "Handling environment changes\n";
   if (env_change & THE_ENVIRONMENT)
-    this << emit_invalidate_all ();
+    send_invalidate_all (this);
 
   // cout << "Applied changes\n";
   env_change  = 0;
@@ -447,7 +553,7 @@ edit_interface_rep::full_screen_mode (bool flag) {
 
 void
 edit_interface_rep::before_menu_action () {
-  buf->mark_undo_block ();
+  mark_undo_blocks ();
   set_input_normal ();
 }
 
@@ -461,25 +567,17 @@ edit_interface_rep::after_menu_action () {
 ******************************************************************************/
 
 void
-edit_interface_rep::handle_get_size (get_size_event ev) {
-  dis->get_extents (ev->w, ev->h);
+edit_interface_rep::handle_get_size_hint (SI& w, SI& h) {
+  gui_root_extents (w, h);
 }
 
 void
-edit_interface_rep::handle_attach_window (attach_window_event ev) {
-  basic_widget_rep::handle_attach_window (ev);
-}
-
-void
-edit_interface_rep::handle_resize (resize_event ev) { (void) ev;
-  if (get_init_string (PAGE_MEDIUM) == "automatic")
-    notify_change (THE_AUTOMATIC_SIZE);
+edit_interface_rep::handle_notify_resize (SI w, SI h) {
+  (void) w; (void) h;
   notify_change (THE_TREE);
 }
 
 void
-edit_interface_rep::handle_set_integer (set_integer_event ev) {
-  if (ev->which == "shrinking factor")
-    set_shrinking_factor (ev->i);
-  else a[0]->a[0] << ev;
+edit_interface_rep::handle_set_shrinking_factor (int sf) {
+  set_shrinking_factor (sf);
 }
