@@ -14,8 +14,10 @@
 #define EDIT_INTERFACE_H
 #include "editor.hpp"
 #include "timer.hpp"
-#include "window.hpp"
-#include "Event/attribute_event.hpp"
+#include "widget.hpp"
+#ifndef AQUATEXMACS
+#include "Widkit/Event/attribute_event.hpp"
+#endif
 
 #define INPUT_NORMAL      0
 #define INPUT_SEARCH      1
@@ -36,7 +38,7 @@ protected:
   bool          got_focus;     // do we have keyboard focus ?
   string        sh_s;          // current string for shorthands
   int           sh_len;        // length of translation
-  window        popup_win;     // the current popup window
+  widget        popup_win;     // the current popup window
   string        message_l;     // a left message to display
   string        message_r;     // a right message to display
   string        last_l;        // last displayed left message
@@ -58,13 +60,19 @@ protected:
   rectangles    selection_rects;
   rectangles    env_rects;
   cursor        oc;
+  bool          temp_invalid_cursor;
   array<string> completions;
   string        completion_prefix;
   int           completion_pos;
-  ps_device     shadow;
+  renderer      shadow;
   SI            vx1, vy1, vx2, vy2;
   rectangles    stored_rects;
-  ps_device     stored;
+  renderer      stored;
+  rectangles    locus_new_rects;
+  rectangles    locus_rects;
+  list<string>  active_ids;
+  int           cur_sb, cur_wb;
+  SI            cur_wx, cur_wy;
 
 public:
   edit_interface_rep ();
@@ -72,8 +80,7 @@ public:
   operator tree ();
   void suspend ();
   void resume ();
-  display get_display ();
-  widget  get_widget ();
+  void get_size (SI& wx, SI& wy);
 
   /* routines for dealing with shrinked coordinates */
   int  get_pixel_size ();
@@ -86,15 +93,15 @@ public:
   void set_extents (SI x1, SI y1, SI x2, SI y2);
 
   /* repainting the window */
-  void draw_text (ps_device dev, rectangles& l);
-  void draw_surround (ps_device dev, rectangle r);
-  void draw_context (ps_device dev, rectangle r);
-  void draw_env (ps_device dev);
-  void draw_cursor (ps_device dev);
-  void draw_selection (ps_device dev);
-  void draw_graphics (ps_device dev);
-  void draw_pre (ps_device dev, rectangle r);
-  void draw_post (ps_device dev, rectangle r);
+  void draw_text (renderer ren, rectangles& l);
+  void draw_surround (renderer ren, rectangle r);
+  void draw_context (renderer ren, rectangle r);
+  void draw_env (renderer ren);
+  void draw_cursor (renderer ren);
+  void draw_selection (renderer ren);
+  void draw_graphics (renderer ren);
+  void draw_pre (renderer ren, rectangle r);
+  void draw_post (renderer ren, rectangle r);
   void draw_with_shadow (rectangle r);
   void draw_with_stored (rectangle r);
 
@@ -130,20 +137,20 @@ public:
   void complete_message ();
   void complete_start (string prefix, array<string> compls);
   bool complete_keypress (string key);
-  unsigned int get_kbd_modifiers ();
 
   /* mouse handling */
-  void mouse_any (string s, SI x, SI y, time_t t);
+  void mouse_any (string s, SI x, SI y, int mods, time_t t);
   void mouse_click (SI x, SI y);
   bool mouse_extra_click (SI x, SI y);
   void mouse_drag (SI x, SI y);
-  void mouse_select (SI x, SI y);
+  void mouse_select (SI x, SI y, int mods);
   void mouse_paste (SI x, SI y);
   void mouse_adjust (SI x, SI y);
   void mouse_scroll (SI x, SI y, bool up);
   cursor get_cursor ();
   void set_pointer (string name);
   void set_pointer (string curs_name, string mask_name);
+  void update_active_loci ();
 
   /* the footer */
   string compute_text_footer (tree st);
@@ -161,15 +168,14 @@ public:
   void   recall_message ();
 
   /* event handlers */
-  void handle_get_size (get_size_event ev);
-  void handle_attach_window (attach_window_event ev);
-  void handle_clear (clear_event ev);
-  void handle_repaint (repaint_event ev);
-  void handle_keypress (keypress_event ev);
-  void handle_mouse (mouse_event ev);
-  void handle_keyboard_focus (keyboard_focus_event ev);
-  void handle_resize (resize_event ev);
-  void handle_set_integer (set_integer_event ev);
+  void handle_get_size_hint (SI& w, SI& h);
+  void handle_notify_resize (SI w, SI h);
+  void handle_keypress (string key, time_t t);
+  void handle_keyboard_focus (bool has_focus, time_t t);
+  void handle_mouse (string kind, SI x, SI y, int mods, time_t t);
+  void handle_set_shrinking_factor (int sf);
+  void handle_clear (SI x1, SI y1, SI x2, SI y2);
+  void handle_repaint (SI x1, SI y1, SI x2, SI y2);
 
   friend class interactive_command_rep;
   friend class tm_window_rep;
