@@ -28,6 +28,11 @@ empty_cell () {
   return "";
 }
 
+static bool
+is_empty_cell (tree t) {
+  return t == "" || t == tree (DOCUMENT, "");
+}
+
 static tree
 empty_row (int nr_cols) {
   int i;
@@ -99,7 +104,7 @@ edit_table_rep::search_format (path p) {
 path
 edit_table_rep::search_format (int& row, int& col) {
   path p= search_table (row, col);
-  if (nil (p)) return p;
+  if (is_nil (p)) return p;
   if (is_func (subtree (et, p), TFORMAT)) return p;
   if (is_func (subtree (et, path_up (p)), TFORMAT)) return path_up (p);
   insert_node (p * 0, TFORMAT);
@@ -123,13 +128,13 @@ path
 edit_table_rep::search_table (int& row, int& col) {
   row= col= 0;
   path p= search_table ();
-  if (nil (p)) return p;
+  if (is_nil (p)) return p;
   path q= p, r= tail (tp, N(p));
 
   if (N(r) <= 1) return path ();
   row= r->item;
   while (true) {
-    if (nil (r)) return r;
+    if (is_nil (r)) return r;
     q= q * r->item;
     r= r->next;
     if (is_func (subtree (et, q), ROW)) break;
@@ -138,7 +143,7 @@ edit_table_rep::search_table (int& row, int& col) {
   if (N(r) <= 1) return path ();
   col= r->item;
   while (true) {
-    if (nil (r)) return r;
+    if (is_nil (r)) return r;
     q= q * r->item;
     r= r->next;
     if (!is_func (subtree (et, q), TFORMAT)) break;
@@ -656,13 +661,13 @@ edit_table_rep::back_in_table (tree t, path p, bool forward) {
 
   int i, j, row, col, nr_rows, nr_cols;
   p= search_table (row, col);
-  if (nil (p)) return;
+  if (is_nil (p)) return;
   table_get_extents (p, nr_rows, nr_cols);
 
   bool flag=true;
   for (j=0; j<nr_cols; j++) {
     path q= search_cell (p, row, j);
-    flag= flag && (subtree (et, q) == empty_cell ());
+    flag= flag && is_empty_cell (subtree (et, q));
   }
   if (flag) {
     int i1, j1, i2, j2;
@@ -677,7 +682,7 @@ edit_table_rep::back_in_table (tree t, path p, bool forward) {
   flag= true;
   for (i=0; i<nr_rows; i++) {
     path q= search_cell (p, i, col);
-    flag= flag && (subtree (et, q) == empty_cell ());
+    flag= flag && is_empty_cell (subtree (et, q));
   }
   if (flag) {
     int i1, j1, i2, j2;
@@ -693,7 +698,7 @@ edit_table_rep::back_in_table (tree t, path p, bool forward) {
   for (i=0; i<nr_rows; i++)
     for (j=0; j<nr_cols; j++) {
       path q= search_cell (p, i, j);
-      flag= flag && (subtree (et, q) == empty_cell ());
+      flag= flag && is_empty_cell (subtree (et, q));
     }
   if (flag) {
     destroy_table ();
@@ -887,7 +892,7 @@ table_undecorate (tree st, int row, int col) {
   for (k=0; k<n-1; k++)
     if ((as_int (st[k][0]) == (row+1)) && (as_int (st[k][2]) == (col+1)))
       if (is_func (st[k], CWITH, 6) && (st[k][4] == CELL_DECORATION)) {
-	int dec_row, dec_col;
+	int dec_row= 0, dec_col= 0;
 	tree T= copy (st[k][5]);
 	search_decoration (T, dec_row, dec_col);
 	table_set (T, dec_row, dec_col, table_get (st[n-1], row, col));
@@ -1026,13 +1031,14 @@ edit_table_rep::make_table (int nr_rows, int nr_cols) {
     }
   }
 
+  table_correct_block_content ();
   set_message ("E-down: new row, E-right: new column", "table");
 }
 
 void
 edit_table_rep::make_subtable (int nr_rows, int nr_cols) {
   path cp= search_upwards (CELL);
-  if (nil (cp)) return;
+  if (is_nil (cp)) return;
   tree T= empty_table (nr_rows, nr_cols);
   path p (0, 0, 0, 0);
   T= tree (TFORMAT, T);
@@ -1041,13 +1047,14 @@ edit_table_rep::make_subtable (int nr_rows, int nr_cols) {
   p= path (0, p);
   assign (cp * 0, T);
   go_to (cp * path (0, p));
+  table_correct_block_content ();
   set_message ("E-down: new row, E-right: new column", "table");
 }
 
 void
 edit_table_rep::destroy_table () {
   path fp= search_format ();
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   while (rp < fp) {
     tree st= subtree (et, path_up (fp));
     if (!is_func (st, TFORMAT)) break;
@@ -1069,7 +1076,7 @@ edit_table_rep::destroy_table () {
 void
 edit_table_rep::table_disactivate () {
   path fp= search_format ();
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   tree st= subtree (et, fp);
   if (!is_func (st, TFORMAT)) return;
   insert_node (fp * 0, INACTIVE);
@@ -1079,7 +1086,7 @@ edit_table_rep::table_disactivate () {
 void
 edit_table_rep::table_extract_format () {
   path fp= search_format ();
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   tree fm= table_get_format (fp);
   fm << "";
   if (is_extension (subtree (et, path_up (fp)), 1)) fp= path_up (fp);
@@ -1091,33 +1098,35 @@ void
 edit_table_rep::table_insert_row (bool forward) {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   int nr_rows, nr_cols, i1, j1, i2, j2;
   table_get_extents (fp, nr_rows, nr_cols);
   table_get_limits (fp, i1, j1, i2, j2);
   if (nr_rows+1 > i2) return;
   table_insert (fp, row + (forward? 1: 0), col, 1, 0);
   table_go_to (fp, row + (forward? 1: 0), col);
+  table_correct_block_content ();
 }
 
 void
 edit_table_rep::table_insert_column (bool forward) {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   int nr_rows, nr_cols, i1, j1, i2, j2;
   table_get_extents (fp, nr_rows, nr_cols);
   table_get_limits (fp, i1, j1, i2, j2);
   if (nr_cols+1 > j2) return;
   table_insert (fp, row, col + (forward? 1: 0), 0, 1);
   table_go_to (fp, row, col + (forward? 1: 0));
+  table_correct_block_content ();
 }
 
 void
 edit_table_rep::table_remove_row (bool forward, bool flag) {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   int nr_rows, nr_cols, i1, j1, i2, j2;
   table_get_extents (fp, nr_rows, nr_cols);
   table_get_limits (fp, i1, j1, i2, j2);
@@ -1135,13 +1144,14 @@ edit_table_rep::table_remove_row (bool forward, bool flag) {
     if (row < nr_rows-1 && forward) table_go_to (fp, row, col, forward);
     else if (forward || row < 0) table_go_to_border (fp, !forward);
   }
+  table_correct_block_content ();
 }
 
 void
 edit_table_rep::table_remove_column (bool forward, bool flag) {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   int nr_rows, nr_cols, i1, j1, i2, j2;
   table_get_extents (fp, nr_rows, nr_cols);
   table_get_limits (fp, i1, j1, i2, j2);
@@ -1159,13 +1169,14 @@ edit_table_rep::table_remove_column (bool forward, bool flag) {
     if (col < nr_cols-1 && forward) table_go_to (fp, row, col, forward);
     else if (forward || col < 0) table_go_to_border (fp, !forward);
   }
+  table_correct_block_content ();
 }
 
 int
 edit_table_rep::table_nr_rows () {
   int nr_rows, nr_cols;
   path fp= search_format ();
-  if (nil (fp)) return -1;
+  if (is_nil (fp)) return -1;
   table_get_extents (fp, nr_rows, nr_cols);
   return nr_rows;
 }
@@ -1174,7 +1185,7 @@ int
 edit_table_rep::table_nr_columns () {
   int nr_rows, nr_cols;
   path fp= search_format ();
-  if (nil (fp)) return -1;
+  if (is_nil (fp)) return -1;
   table_get_extents (fp, nr_rows, nr_cols);
   return nr_cols;
 }
@@ -1183,7 +1194,7 @@ int
 edit_table_rep::table_which_row () {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return 0;
+  if (is_nil (fp)) return 0;
   return row+1;
 }
 
@@ -1191,7 +1202,7 @@ int
 edit_table_rep::table_which_column () {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return 0;
+  if (is_nil (fp)) return 0;
   return col+1;
 }
 
@@ -1199,7 +1210,7 @@ path
 edit_table_rep::table_search_cell (int row, int col) {
   int nr_rows, nr_cols;
   path fp= search_format ();
-  if (nil (fp)) return path ();
+  if (is_nil (fp)) return path ();
   table_get_extents (fp, nr_rows, nr_cols);
   if (row>0) row--; else row+=nr_rows;
   if (col>0) col--; else col+=nr_cols;
@@ -1211,7 +1222,7 @@ void
 edit_table_rep::table_go_to (int row, int col) {
   int nr_rows, nr_cols;
   path fp= search_format ();
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   table_get_extents (fp, nr_rows, nr_cols);
   if (row>0) row--; else row+=nr_rows;
   if (col>0) col--; else col+=nr_cols;
@@ -1244,7 +1255,7 @@ void
 edit_table_rep::table_format_center () {
   int row, col;
   path fp= search_format (row, col);
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   table_format_center (fp, row, col);
 }
 
@@ -1266,6 +1277,28 @@ edit_table_rep::table_column_decoration (bool forward) {
   if (forward && (col < (nr_cols-1))) table_hor_decorate (fp, col, 0, 1);
 }
 
+void
+edit_table_rep::table_correct_block_content () {
+  int nr_rows, nr_cols;
+  path fp= search_format ();
+  if (is_nil (fp)) return;
+  table_get_extents (fp, nr_rows, nr_cols);
+  int row, col;
+  for (row= 0; row < nr_rows; row++)
+    for (col= 0; col < nr_cols; col++) {
+      path cp= search_cell (fp, row, col);
+      tree st= subtree (et, cp);
+      tree t1= table_get_format (fp, row+1, col+1, row+1, col+1, CELL_BLOCK);
+      tree t2= table_get_format (fp, row+1, col+1, row+1, col+1, CELL_HYPHEN);
+      bool f1= (t1 == "no" || (t1 == "auto" && t2 == "n"));
+      bool f2= (t1 == "yes" || (t1 == "auto" && is_atomic (t2) && t2 != "n"));
+      if (f1 && is_document (st) && N(st) == 1)
+	remove_node (cp * 0);
+      else if (f2 && !is_document (st))
+	insert_node (cp * 0, DOCUMENT);
+    }
+}
+
 void
 edit_table_rep::set_cell_mode (string mode) {
   cell_mode= mode;
@@ -1278,33 +1311,31 @@ edit_table_rep::get_cell_mode () {
 
 void
 edit_table_rep::cell_set_format (string var, string val) {
-  if (val == "") cell_del_format (var);
-  else {
-    if (selection_active_table ()) {
-      int row1, col1, row2, col2;
-      path fp= selection_get_subtable (row1, col1, row2, col2);
-      table_set_format (fp, row1+1, col1+1, row2+1, col2+1, var, val);
-    }
-    else {
-      int row, col;
-      path fp= search_format (row, col); row++; col++;
-      if (nil (fp)) return;
-      if (cell_mode=="row")
-	table_set_format (fp, row, 1, row, -1, var, val);
-      else if (cell_mode=="column")
-	table_set_format (fp, 1, col, -1, col, var, val);
-      else if (cell_mode=="table")
-	table_set_format (fp, 1, 1, -1, -1, var, val);
-      else table_set_format (fp, row, col, row, col, var, val);
-    }
+  if (selection_active_table ()) {
+    int row1, col1, row2, col2;
+    path fp= selection_get_subtable (row1, col1, row2, col2);
+    table_set_format (fp, row1+1, col1+1, row2+1, col2+1, var, val);
   }
+  else {
+    int row, col;
+    path fp= search_format (row, col); row++; col++;
+    if (is_nil (fp)) return;
+    if (cell_mode=="row")
+      table_set_format (fp, row, 1, row, -1, var, val);
+    else if (cell_mode=="column")
+      table_set_format (fp, 1, col, -1, col, var, val);
+    else if (cell_mode=="table")
+      table_set_format (fp, 1, 1, -1, -1, var, val);
+    else table_set_format (fp, row, col, row, col, var, val);
+  }
+  table_correct_block_content ();
 }
 
 string
 edit_table_rep::cell_get_format (string var) {
   int row, col;
   path fp= search_format (row, col); row++; col++;
-  if (nil (fp)) return "";
+  if (is_nil (fp)) return "";
   if (cell_mode=="row")
     return as_string (table_get_format (fp, row, 1, row, -1, var));
   else if (cell_mode=="column")
@@ -1324,41 +1355,18 @@ edit_table_rep::cell_del_format (string var) {
   else {
     int row, col;
     path fp= search_format (row, col); row++; col++;
-    if (nil (fp)) return;
+    if (is_nil (fp)) return;
     if (cell_mode=="row") table_del_format (fp, row, 1, row, -1, var);
     else if (cell_mode=="column") table_del_format (fp, 1, col, -1, col, var);
     else if (cell_mode=="table") table_del_format (fp, 1, 1, -1, -1, var);
     else table_del_format (fp, row, col, row, col, var);
   }
-}
-
-void
-edit_table_rep::cell_multi_paragraph (bool flag) {
-  int row, col;
-  path fp= search_format (row, col);
-  if (nil (fp)) return;
-  path cp= search_cell (fp, row, col);
-  tree st= subtree (et, cp);
-
-  if (flag && (!is_document (st)))
-    insert_node (cp * 0, DOCUMENT);
-  else if ((!flag) && is_document (st) && (N(st) == 1))
-    remove_node (cp * 0);
-}
-
-bool
-edit_table_rep::cell_is_multi_paragraph () {
-  int row, col;
-  path fp= search_format (row, col);
-  if (nil (fp)) return false;
-  path cp= search_cell (fp, row, col);
-  tree st= subtree (et, cp);
-  return is_document (st);
+  table_correct_block_content ();
 }
 
 void
 edit_table_rep::table_test () {
   path fp= search_format ();
-  if (nil (fp)) return;
+  if (is_nil (fp)) return;
   cout << table_get_format (fp) << "\n";
 }
