@@ -17,6 +17,8 @@
 #include "timer.hpp"
 #include "merge_sort.hpp"
 #include "data_cache.hpp"
+#include "web_files.hpp"
+#include "Scheme/object.hpp"
 
 #include <stddef.h>
 #include <stdio.h>
@@ -49,7 +51,7 @@ load_string (url u, string& s, bool fatal) {
     bool doc_flag= do_cache_doc (name);
     string cache_type= doc_flag? string ("doc_cache"): string ("file_cache");
     if (doc_flag) cache_load ("doc_cache");
-    if (is_cached (cache_type, name)) {
+    if (is_cached (cache_type, name) && is_up_to_date (url_parent (r))) {
       s= cache_get (cache_type, name) -> label;
       return false;
     }
@@ -94,6 +96,13 @@ load_string (url u, string& s, bool fatal) {
 
 bool
 save_string (url u, string s, bool fatal) {
+  if (is_rooted_tmfs (u)) {
+    bool err= save_to_server (u, s);
+    if (err && fatal)
+      fatal_error (as_string (u) * " not writeable", "save_string");
+    return err;
+  }
+
   // cout << "Save " << u << LF;
   url r= u;
   if (!is_rooted_name (r)) r= resolve (r, "");
@@ -122,8 +131,10 @@ save_string (url u, string s, bool fatal) {
     if (!err && N(s) <= 10000)
       if (file_flag || doc_flag)
 	cache_set (cache_type, name, s);
+    (bool) is_up_to_date (url_parent (r), true);
     // End caching
   }
+
   if (err && fatal)
     fatal_error (as_string (u) * " not writeable", "save_string");
   return err;
@@ -134,17 +145,22 @@ save_string (url u, string s, bool fatal) {
 ******************************************************************************/
 
 static bool
-get_attributes (url name, struct stat* buf, bool link_flag=false) {
+get_attributes (url name, struct stat* buf,
+		bool link_flag=false, bool cache_flag= true)
+{
   // cout << "Stat " << name << LF;
   string name_s= concretize (name);
 
   // Stat result in cache?
-  if (is_cached ("stat_cache.scm", name_s)) {
-    string r= cache_get ("stat_cache.scm", name_s) -> label;
-    if (r == "#f") return true;
-    buf->st_mode= ((unsigned int) as_int (r));
-    return false;
-  }
+  if (cache_flag &&
+      is_cached ("stat_cache.scm", name_s) &&
+      is_up_to_date (url_parent (name)))
+    {
+      string r= cache_get ("stat_cache.scm", name_s) -> label;
+      if (r == "#f") return true;
+      buf->st_mode= ((unsigned int) as_int (r));
+      return false;
+    }
   // End caching
 
   bench_start ("stat");
@@ -162,13 +178,15 @@ get_attributes (url name, struct stat* buf, bool link_flag=false) {
   bench_cumul ("stat");
 
   // Cache stat results
-  if (flag) {
-    if (do_cache_stat_fail (name_s))
-      cache_set ("stat_cache.scm", name_s, "#f");
-  }
-  else {
-    if (do_cache_stat (name_s))
-      cache_set ("stat_cache.scm", name_s, as_string ((int) buf->st_mode));
+  if (cache_flag) {
+    if (flag) {
+      if (do_cache_stat_fail (name_s))
+	cache_set ("stat_cache.scm", name_s, "#f");
+    }
+    else {
+      if (do_cache_stat (name_s))
+	cache_set ("stat_cache.scm", name_s, as_string ((int) buf->st_mode));
+    }
   }
   // End caching
 
@@ -178,11 +196,52 @@ get_attributes (url name, struct stat* buf, bool link_flag=false) {
 bool
 is_of_type (url name, string filter) {
   if (filter == "") return true;
+  int i, n= N(filter);
+
+  // Files from the web
+  if (is_rooted_web (name)) {
+    // cout << "  try " << name << "\n";
+    url from_web= get_from_web (name);
+    // cout << "  --> " << from_web << "\n";
+    if (is_none (from_web)) return false;
+    for (i=0; i<n; i++)
+      switch (filter[i]) {
+      case 'd': return false;
+      case 'l': return false;
+      case 'w': return false;
+      case 'x': return false;
+      }
+    return true;
+  }
+
+  // Files from a remote server
+  if (is_rooted_tmfs (name)) {
+    for (i=0; i<n; i++)
+      switch (filter[i]) {
+      case 'd': return false;
+      case 'l': return false;
+      case 'r':
+	if (!as_bool (call ("tmfs-permission?", name, "read")))
+	  return false;
+	break;
+      case 'w':
+	if (!as_bool (call ("tmfs-permission?", name, "write")))
+	  return false;
+	break;
+      case 'x': return false;
+      }
+    return true;
+  }
+
+  // Files from the ramdisk
+  if (is_ramdisc (name))
+    return true;
+
+  // Normal files
 #ifdef OS_WIN32
   if ((filter == "x") && (suffix(name) != "exe") && (suffix(name) != "bat"))
     name = glue (name, ".exe");
 #endif
-  int i, n= N(filter);
   bool preserve_links= false;
   for (i=0; i<n; i++)
     preserve_links= preserve_links || (filter[i] == 'l');
@@ -220,12 +279,21 @@ bool is_regular (url name) { return is_of_type (name, "f"); }
 bool is_directory (url name) { return is_of_type (name, "d"); }
 bool is_symbolic_link (url name) { return is_of_type (name, "l"); }
 
+int
+last_modified (url u, bool cache_flag) {
+  struct stat u_stat;
+  if (get_attributes (u, &u_stat, true, cache_flag)) return 0;
+  return u_stat.st_mtime;
+}
+
 bool
 is_newer (url which, url than) {
   struct stat which_stat;
   struct stat than_stat;
+  // FIXME: why was this? 
   if (is_cached ("stat_cache.scm", concretize (which))) return false;
   if (is_cached ("stat_cache.scm", concretize (than))) return false;
+  // end FIXME
   if (get_attributes (which, &which_stat, true)) return false;
   if (get_attributes (than , &than_stat , true)) return false;
   return which_stat.st_mtime > than_stat.st_mtime;
@@ -246,6 +314,22 @@ url_temp (string suffix) {
   return u;
 }
 
+url
+url_scratch (string prefix, string postfix, int i) {
+  url dir ("$TEXMACS_HOME_PATH/texts/scratch");
+  if (!exists (dir)) mkdir (dir);
+  for (; true; i++) {
+    url name= dir * (prefix * as_string (i) * postfix);
+    if (!exists (name)) return name;
+  }
+  return dir * (prefix * "x" * postfix);
+}
+
+bool
+is_scratch (url u) {
+  return head (u) == url ("$TEXMACS_HOME_PATH/texts/scratch");
+}
+
 /******************************************************************************
 * Reading directories
 ******************************************************************************/
@@ -273,7 +357,7 @@ read_directory (url u, bool& error_flag) {
   string name= concretize (u);
 
   // Directory contents in cache?
-  if (is_cached ("dir_cache.scm", name))
+  if (is_cached ("dir_cache.scm", name) && is_up_to_date (u))
     return cache_dir_get (name);
   bench_start ("read directory");
   // End caching
