@@ -16,7 +16,7 @@
 #include "sys_utils.hpp"
 #include "analyze.hpp"
 
-bool load_string (url file_name, string& s, bool fatal=false);
+bool load_string (url file_name, string& s, bool fatal);
 bool save_string (url file_name, string s, bool fatal=false);
 
 bool is_of_type (url name, string filter);
@@ -24,7 +24,10 @@ bool is_regular (url name);
 bool is_directory (url name);
 bool is_symbolic_link (url name);
 bool is_newer (url which, url than);
+int  last_modified (url u, bool cache_flag= true);
 url  url_temp (string suffix= "");
+url  url_scratch (string prefix= "no_name_", string postfix= ".tm", int i=1);
+bool is_scratch (url u);
 
 array<string> read_directory (url name, bool& error_flag);
 
@@ -35,9 +38,9 @@ inline void system (string which, url u1) {
   system (which * " " * sys_concretize (u1)); }
 inline void system (string which, url u1, url u2) {
   system (which * " " * sys_concretize (u1) * " " * sys_concretize (u2)); }
-inline void system (string which, url u1, char* post) {
+inline void system (string which, url u1, const char* post) {
   system (which * " " * sys_concretize (u1) * " " * post); }
-inline void system (string which, url u1, char* sep, url u2) {
+inline void system (string which, url u1, const char* sep, url u2) {
   system (which * " " * sys_concretize (u1) * " " * sep *
 	          " " * sys_concretize (u2)); }
 inline string eval_system (string which, url u1) {
