@@ -13,6 +13,13 @@
 #include "tm_data.hpp"
 #include "convert.hpp"
 #include "file.hpp"
+#include "web_files.hpp"
+#include "socket_server.hpp"
+#include "message.hpp"
+#include "dictionary.hpp"
+#ifdef EXPERIMENTAL
+#include "../../Style/Memorizer/clean_copy.hpp"
+#endif
 
 url tm_init_buffer_file= url_none ();
 url my_init_buffer_file= url_none ();
@@ -22,13 +29,22 @@ int max_undo_depth= 100; // should actually be part of tm_data_rep
 * Constructor and destructor
 ******************************************************************************/
 
-tm_data_rep::tm_data_rep (): history (0), hist_pos (-1) {}
+tm_data_rep::tm_data_rep () {}
 tm_data_rep::~tm_data_rep () {}
 
 /******************************************************************************
 * Low level functions for maintaining the buffer menu
 ******************************************************************************/
 
+int
+tm_data_rep::find_buffer (path p) {
+  int i;
+  for (i=0; i<N(bufs); i++)
+    if (bufs[i]->rp <= p)
+      return i;
+  return -1;
+}
+
 int
 tm_data_rep::find_buffer (url name) {
   int i;
@@ -41,10 +57,20 @@ tm_data_rep::find_buffer (url name) {
 string
 tm_data_rep::new_menu_name (url u) {
   string name= as_string (tail (u));
+  if (starts (name, "no_name_") && ends (name, ".tm")) {
+    string lan= get_output_language ();
+    string no_name= translate ("No name", "english", lan);
+    for (int i=0; i<N(no_name); i++)
+      if (((unsigned char) (no_name[i])) >= (unsigned char) 128)
+	{ no_name= "No name"; break; }
+    name= no_name * " [" * name (8, N(name) - 3) * "]";
+  }
   if ((name == "") || (name == "."))
     name= as_string (tail (u * url_parent ()));
   if ((name == "") || (name == "."))
     name= as_string (u);
+  if (is_rooted_tmfs (u))
+    name= as_string (call ("tmfs-name", as_string (u)));
 
   int i, j;
   for (j=1; true; j++) {
@@ -57,20 +83,52 @@ tm_data_rep::new_menu_name (url u) {
   }
 }
 
+static void
+menu_append_buffer (string& s, tm_buffer buf) {
+  if (buf->in_menu) {
+    s << " (\"" << buf->abbr;
+    if (buf->needs_to_be_saved ()) s << " *"; 
+    s << "\" (switch-to-buffer \"" * as_string (buf->name) * "\"))";
+  }
+}
+
 object
 tm_data_rep::get_buffer_menu () {
-  int i;
-  string s ("(menu-dynamic ");
-  for (i=0; i<N(bufs); i++) {
-    if (i>0) s << " ";
-    s << "(\"" << bufs[i]->abbr;
-    if (bufs[i]->needs_to_be_saved ()) s << " *"; 
-    s << "\" (switch-to-buffer \"" * as_string (bufs[i]->name) * "\"))";
+  int i, count;
+  bool two_types= false;;
+  string s ("(menu-dynamic");
+  for (i=0, count=0; i<N(bufs); i++) {
+    if (is_none (bufs[i]->extra) == is_none (bufs[0]->extra)) {
+      menu_append_buffer (s, bufs[i]);
+      count++;
+    }
+    else two_types= true;
+    if (count == 10) break;
+  }
+  if (two_types) {
+    s << " ---";
+    for (i=0, count=0; i<N(bufs); i++) {
+      if (is_none (bufs[i]->extra) != is_none (bufs[0]->extra)) {
+	menu_append_buffer (s, bufs[i]);
+	count++;
+      }
+      if (count == 10) break;
+    }
   }
   s << ")";
   return eval (s);
 }
 
+bool
+tm_data_rep::buffer_in_menu (url u, bool flag) {
+  int nr= find_buffer (u);
+  if (nr == -1) return false;
+  tm_buffer buf= bufs[nr];
+  bool old= buf->in_menu;
+  buf->in_menu= flag;
+  return old;
+}
+
 void
 tm_data_rep::menu_insert_buffer (tm_buffer buf) {
   // bufs << ((pointer) buf); // causes compilation error
@@ -177,6 +235,12 @@ tm_data_rep::set_name_buffer (url name) {
   set_abbr_buffer (new_menu_name (name));
 }
 
+string
+tm_data_rep::get_abbr_buffer () {
+  tm_buffer buf= get_buffer ();
+  return buf->abbr;
+}
+
 void
 tm_data_rep::set_abbr_buffer (string abbr) {
   int i;
@@ -186,7 +250,7 @@ tm_data_rep::set_abbr_buffer (string abbr) {
   for (i=0; i<N(buf->vws); i++) {
     tm_view vw2= buf->vws[i];
     if (vw2->win != NULL)
-      vw2->win->win->set_name (buf->abbr);
+      vw2->win->set_window_name (buf->abbr);
   }
 }
 
@@ -197,6 +261,21 @@ tm_data_rep::get_name_buffer () {
   return buf->name;
 }
 
+url
+tm_data_rep::get_name_buffer (path p) {
+  int nr= find_buffer (p);
+  if (nr == -1) return url_none ();
+  return bufs[nr]->name;
+}
+
+url
+tm_data_rep::get_all_buffers () {
+  url u= url_none ();
+  for (int i=N(bufs)-1; i>=0; i--)
+    u= bufs[i]->name | u;
+  return u;
+}
+
 /******************************************************************************
 * Low level view routines
 ******************************************************************************/
@@ -216,7 +295,6 @@ tm_data_rep::new_view (url name) {
   ed->notify_page_change ();
   ed->add_init (buf->init);
   ed->notify_change (THE_DECORATIONS);
-  ed->notify_change (THE_AUTOMATIC_SIZE);
   ed->typeset_invalidate_env ();
 
   tm_view temp_vw= get_view (false);
@@ -261,13 +339,12 @@ void
 tm_data_rep::attach_view (tm_window win, tm_view vw) {
   // cout << "Attach view " << vw->buf->name << "\n";
   vw->win= win;
-  widget wid= (widget) win->wid;
-  wid ["canvas"] << set_widget ("scrollable", vw->ed);
-  if (wid->attached ()) {
-    vw->ed->resume ();
-    win->wid->set_window_name (vw->buf->abbr);
-    wid ["canvas"] << emit_update ();
-  }
+  widget wid= win->wid;
+  set_canvas (wid, vw->ed);
+  if (!is_attached (wid))
+    fatal_error ("widget should be attached", "tm_data_rep::attach_view");
+  vw->ed->resume ();
+  win->set_window_name (vw->buf->abbr);
   // cout << "View attached\n";
 }
 
@@ -277,14 +354,12 @@ tm_data_rep::detach_view (tm_view vw) {
   tm_window win= vw->win;
   if (win == NULL) return;
   vw->win= NULL;
-  widget wid= (widget) win->wid;
-  wid ["canvas"] << set_widget ("scrollable", glue_widget ());
-  if (wid->attached ()) {
-    vw->ed->suspend ();
-    vw->ed << emit_attach_window (NULL);
-    win->wid->set_window_name ("TeXmacs");
-    wid ["canvas"] << emit_update ();
-  }
+  widget wid= win->wid;
+  if (!is_attached (wid))
+    fatal_error ("widget should be attached", "tm_data_rep::attach_view");
+  vw->ed->suspend ();
+  set_canvas (wid, glue_widget ());
+  win->set_window_name ("TeXmacs");
   // cout << "View detached\n";
 }
 
@@ -292,10 +367,24 @@ tm_data_rep::detach_view (tm_view vw) {
 * Low level window routines
 ******************************************************************************/
 
+class kill_window_command_rep: public command_rep {
+public:
+  inline kill_window_command_rep () {}
+  inline void apply () { exec_delayed (scheme_cmd ("(safely-kill-window)")); }
+  ostream& print (ostream& out) { return out << "kill window"; }
+};
+
 tm_window
-tm_data_rep::new_window (display dis, bool map_flag) {
-  tm_window win= new tm_window_rep (new tm_widget_rep (this, dis));
-  if (map_flag) win->win->map ();
+tm_data_rep::new_window (bool map_flag, tree geom) {
+  int mask= 0;
+  if (get_preference ("header") == "on") mask += 1;
+  if (get_preference ("main icon bar") == "on") mask += 2;
+  if (get_preference ("context dependent icons") == "on") mask += 4;
+  if (get_preference ("user provided icons") == "on") mask += 8;
+  if (get_preference ("status bar") == "on") mask += 16;
+  command quit= new kill_window_command_rep ();
+  tm_window win= new tm_window_rep (texmacs_widget (mask, quit), geom);
+  if (map_flag) win->map ();
   return win;
 }
 
@@ -319,8 +408,8 @@ tm_data_rep::delete_view_from_window (tm_window win) {
 void
 tm_data_rep::delete_window (tm_window win) {
   while (delete_view_from_window (win));
-  win->win->unmap ();
-  delete win->win;
+  win->unmap ();
+  destroy_window_widget (win->win);
   delete win;
 }
 
@@ -339,8 +428,8 @@ tm_data_rep::new_buffer_in_this_window (url name, tree doc) {
 }
 
 void
-tm_data_rep::new_buffer_in_new_window (url name, tree doc) {
-  tm_window win= new_window (get_display ());
+tm_data_rep::new_buffer_in_new_window (url name, tree doc, tree geom) {
+  tm_window win= new_window (true, geom);
   tm_buffer buf= new_buffer (name, doc);
   tm_view   vw = get_passive_view (buf);
   attach_view (win, vw);
@@ -362,15 +451,24 @@ tm_data_rep::get_buf (int i) {
   return (tm_buffer) bufs[i];
 }
 
-void
+tm_buffer
+tm_data_rep::get_buf (path p) {
+  int nr= find_buffer (p);
+  if (nr >= 0) return bufs[nr];
+  else return NULL;
+}
+
+url
 tm_data_rep::new_buffer () {
-  int i;
-  for (i=1; true; i++) {
-    string name= "no name " * as_string (i);
-    if (i==1) name= "no name";
-    if (find_buffer (name) != -1) continue;
-    new_buffer_in_this_window (name, tree (DOCUMENT));
-    return;
+  int i=1;
+  while (true) {
+    url name= url_scratch ("no_name_", ".tm", i);
+    int nr= find_buffer (name);
+    if (nr == -1) {
+      new_buffer_in_this_window (name, tree (DOCUMENT));
+      return name;
+    }
+    else i++;
   }
 }
 
@@ -385,11 +483,18 @@ tm_data_rep::switch_to_buffer (int nr) {
   attach_view (win, new_vw);
   set_view (new_vw);
   menu_focus_buffer (buf);
-  tm_widget meta= new_vw->win->wid;
-  meta->set_shrinking_factor (meta->get_shrinking_factor ());
+  tm_window nwin= new_vw->win;
+  nwin->set_shrinking_factor (nwin->get_shrinking_factor ());
   // cout << "Switched to buffer " << nr << "\n";
 }
 
+bool
+tm_data_rep::switch_to_buffer (path p) {
+  int nr= find_buffer (p);
+  if (nr != -1) switch_to_buffer (nr);
+  return nr != -1;
+}
+
 void
 tm_data_rep::switch_to_buffer (url name) {
   int nr= find_buffer (name);
@@ -427,6 +532,7 @@ tm_data_rep::switch_to_active_buffer (url name) {
 void
 tm_data_rep::revert_buffer () {
   tm_buffer buf= get_buffer ();
+  web_cache_invalidate (buf->name);
   tree doc= load_tree (buf->name, buf->fm);
   if (doc == "error") set_message ("Error: file not found", "revert buffer");
   else revert_buffer (buf->name, doc);
@@ -458,21 +564,23 @@ tm_data_rep::kill_buffer () {
   delete_buffer (buf);
 }
 
-void
-tm_data_rep::open_window () {
-  int i;
-  for (i=1; true; i++) {
-    string name= "no name " * as_string (i);
-    if (i==1) name= "no name";
-    if (find_buffer (name) != -1) continue;
-    new_buffer_in_new_window (name, tree (DOCUMENT));
-    return;
+url
+tm_data_rep::open_window (tree geom) {
+  int i=1;
+  while (true) {
+    url name= url_scratch ("no_name_", ".tm", i);
+    int nr= find_buffer (name);
+    if (nr == -1) {
+      new_buffer_in_new_window (name, tree (DOCUMENT), geom);
+      return name;
+    }
+    else i++;
   }
 }
 
 void
 tm_data_rep::clone_window () {
-  tm_window win= new_window (get_display ());
+  tm_window win= new_window ();
   tm_buffer buf= get_buffer ();
   tm_view   vw = get_passive_view (buf);
   attach_view (win, vw);
@@ -483,7 +591,7 @@ tm_data_rep::clone_window () {
 void
 tm_data_rep::kill_window () {
   int i, j;
-  tm_window win    = get_window ();
+  tm_window win= get_window ();
   for (i=0; i<N(bufs); i++) {
     tm_buffer buf= bufs[i];
     for (j=0; j<N(buf->vws); j++) {
@@ -496,7 +604,23 @@ tm_data_rep::kill_window () {
       }
     }
   }
-  quit ();
+  if (number_of_servers () == 0) quit ();
+  else delete_window (win);
+}
+
+void
+tm_data_rep::kill_window_and_buffer () {
+  if (N(bufs) <= 1) quit();
+  int i;
+  bool kill= true;
+  tm_buffer buf= get_buffer();
+  tm_window win= get_window ();
+  for (i=0; i<N(buf->vws); i++) {
+    tm_view old_vw= buf->vws[i];
+    if (old_vw->win != win) kill= false;
+  }
+  kill_window ();
+  if (kill) delete_buffer (buf);
 }
 
 void
@@ -517,15 +641,12 @@ tm_data_rep::no_bufs () {
 }
 
 void
-tm_data_rep::set_aux_buffer (string aux, url name, tree doc) {
+tm_data_rep::set_aux (string aux, url name) {
   int i, nr= find_buffer (aux);
-  if (nr == -1) new_buffer (aux, doc);
-  else revert_buffer (aux, doc);
-  nr= find_buffer (aux);
   if (nr != -1) {
     tm_buffer buf= bufs[nr];
     buf->extra= name;
-    if (aux == "* Help *") {
+    if (starts (aux, "Help - ")) {
       buf->fm= "help";
       buf->read_only= true;
     }
@@ -533,28 +654,65 @@ tm_data_rep::set_aux_buffer (string aux, url name, tree doc) {
       tm_view vw= buf->vws[i];
       vw->ed->set_base_name (name);
     }
-    switch_to_buffer (nr);
   }
 }
 
 void
-tm_data_rep::set_help_buffer (url name, tree doc) {
-  set_aux_buffer ("* Help *", name, doc);
-  tree t= tuple (name->t, doc);
-  hist_pos++;
-  if ((hist_pos>=0) && (hist_pos<N(history)) && (history[hist_pos]!=t))
-    history->resize (hist_pos);
-  if (hist_pos == N(history))
-    history << tuple (name->t, doc);
+tm_data_rep::set_aux_buffer (string aux, url name, tree doc) {
+  int nr= find_buffer (aux);
+  if (nr == -1) new_buffer (aux, doc);
+  else revert_buffer (aux, doc);
+  nr= find_buffer (aux);
+  if (nr != -1) {
+    set_aux (aux, name);
+    switch_to_buffer (nr);
+  }
+}
+
+static string
+get_doc_title (tree t) {
+  if (is_atomic (t)) return "";
+  if (is_compound (t, "doc-title") ||
+      is_compound (t, "tmdoc-title") ||
+      is_compound (t, "tmdoc-title*") ||
+      is_compound (t, "tmweb-title"))
+    return tree_to_verbatim (t[0]);
+  else {
+    for (int i=0; i<N(t); i++) {
+      string r= get_doc_title (t[i]);
+      if (r != "") return r;
+    }
+    return "";
+  }
+}
+
+string
+get_help_title (url name, tree t) {
+  string s= get_doc_title (t);
+  if (s == "") return "Help - " * as_string (tail (name));
+  else return "Help - " * s;
 }
 
 void
-tm_data_rep::browse_help (int delta) {
-  if (hist_pos + delta < 0) return;
-  if (hist_pos + delta >= N(history)) return;
-  hist_pos += delta-1;
-  tree t= history[hist_pos+1];
-  set_help_buffer (as_url (t[0]), t[1]);
+tm_data_rep::set_help_buffer (url name, tree doc) {
+  set_aux_buffer (get_help_title (name, doc), name, doc);
+}
+
+void
+tm_data_rep::set_buffer_tree (url name, tree doc) {
+  int nr= find_buffer (name);
+  if (nr == -1) new_buffer (name, tree (DOCUMENT));
+  nr= find_buffer (name);
+  tm_buffer buf= bufs[nr];
+  get_editor () -> assign (buf->rp, doc);
+}
+
+tree
+tm_data_rep::get_buffer_tree (url name) {
+  int nr= find_buffer (name);
+  if (nr == -1) return "";
+  tm_buffer buf= bufs[nr];
+  return subtree (the_et, buf->rp);
 }
 
 /******************************************************************************
@@ -617,22 +775,122 @@ path
 new_document () {
   int i, n= N(the_et);
   for (i=0; i<n; i++)
-    if (the_et[i] == UNINIT) {
+    if (the_et[i] == UNINIT && is_nil (the_et[i]->obs)) {
       assign (the_et[i], tree (DOCUMENT, ""));
       return path (i); // obtain_ip (the_et[i]);
     }
   insert (the_et, n, tuple (tree (DOCUMENT, "")));
+#ifdef EXPERIMENTAL
+  global_notify_insert (path (n), tuple (tree (DOCUMENT, "")));
+#endif
   return path (n); // obtain_ip (the_et[n]);
 }
 
 void
 delete_document (path rp) {
   assign (subtree (the_et, rp), UNINIT);
+#ifdef EXPERIMENTAL
+  global_notify_assign (rp, UNINIT);
+#endif
 }
 
 void
 set_document (path rp, tree t) {
+  //assign (subtree (the_et, rp), t);
   assign (subtree (the_et, rp), copy (t));
+#ifdef EXPERIMENTAL
+  global_notify_assign (rp, copy (t));
+#endif
+}
+
+/******************************************************************************
+* Window management
+******************************************************************************/
+
+static int  last_window= 1;
+static path the_windows;
+
+int
+create_window_id () {
+  the_windows= path (last_window, the_windows);
+  return last_window++;
+}
+
+static path
+remove (path p, int i) {
+  if (is_nil (p)) return p;
+  else if (p->item == i) return p->next;
+  else return path (p->item, remove (p->next, i));
+}
+
+void
+destroy_window_id (int i) {
+  the_windows= remove (the_windows, i);
+}
+
+int
+tm_data_rep::window_current () {
+  tm_window win= get_window ();
+  return win->id;
+}
+
+path
+tm_data_rep::windows_list () {
+  return the_windows;
+}
+
+path
+tm_data_rep::buffer_to_windows (url name) {
+  path p;
+  int nr= find_buffer (name);
+  if (nr == -1) return path ();
+  tm_buffer buf= bufs[nr];
+  for (int i=0; i<N(buf->vws); i++)
+    if (buf->vws[i]->win != NULL)
+      p= path (buf->vws[i]->win->id, p);
+  return p;
+}
+
+url
+tm_data_rep::window_to_buffer (int id) {
+  for (int i=0; i<N(bufs); i++)
+    for (int j=0; j<N(bufs[i]->vws); j++)
+      if (bufs[i]->vws[j]->win != NULL)
+	if (bufs[i]->vws[j]->win->id == id)
+	  return bufs[i]->name;
+  return url_none ();
+}
+
+tm_view
+tm_data_rep::window_find_view (int id) {
+  for (int i=0; i<N(bufs); i++)
+    for (int j=0; j<N(bufs[i]->vws); j++)
+      if (bufs[i]->vws[j]->win != NULL)
+	if (bufs[i]->vws[j]->win->id == id)
+	  return bufs[i]->vws[j];
+  return NULL;
+}
+
+void
+tm_data_rep::window_set_buffer (int id, url name) {
+  tm_view old_vw= window_find_view (id);
+  if (old_vw == NULL || old_vw->buf->name == name) return;
+  tm_window win= old_vw->win;
+  int nr= find_buffer (name);
+  if (nr == -1) return;
+  tm_buffer buf   = bufs[nr];
+  tm_view   new_vw= get_passive_view (buf);
+  detach_view (old_vw);
+  attach_view (win, new_vw);
+}
+
+void
+tm_data_rep::window_focus (int id) {
+  if (id == window_current ()) return;
+  tm_view vw= window_find_view (id);
+  if (vw == NULL) return;
+  set_view (vw);
+  menu_focus_buffer (vw->buf);
 }
 
 /******************************************************************************
@@ -721,6 +979,7 @@ tm_buffer_rep::truncate_undos (int nr) {
 
 bool
 tm_buffer_rep::needs_to_be_saved () {
+  if (!in_menu) return false;
   if (need_save) return true;
   if ((undo == "nil") || (undo[0] != ""))
     return (last_save != undo_depth);
@@ -729,6 +988,7 @@ tm_buffer_rep::needs_to_be_saved () {
 
 bool
 tm_buffer_rep::needs_to_be_autosaved () {
+  if (!in_menu) return false;
   if (!needs_to_be_saved ()) return false;
   if (need_autosave) return true;
   if ((undo == "nil") || (undo[0] != ""))
