@@ -15,6 +15,13 @@
 #include "server.hpp"
 #include "timer.hpp"
 #include "data_cache.hpp"
+#ifdef EXPERIMENTAL
+#include "../../Style/Memorizer/clean_copy.hpp"
+#include "../../Style/Environment/environment.hpp"
+#endif
+#ifdef AQUATEXMACS
+void mac_fix_paths ();
+#endif
 
 extern bool   char_clip;
 extern bool   reverse_colors;
@@ -29,6 +36,19 @@ extern int geometry_x, geometry_y;
 extern tree the_et;
 extern bool texmacs_started;
 
+/******************************************************************************
+* For testing
+******************************************************************************/
+
+//#define ENABLE_TESTS
+#ifdef ENABLE_TESTS
+void
+test_routines () {
+  extern void test_math ();
+  test_math ();
+}
+#endif
+
 /******************************************************************************
 * Real main program for encaptulation of guile
 ******************************************************************************/
@@ -133,6 +153,7 @@ TeXmacs_main (int argc, char** argv) {
 	       (s == "-delete-cache") || (s == "-delete-font-cache") ||
 	       (s == "-delete-style-cache") || (s == "-delete-file-cache") ||
 	       (s == "-delete-doc-cache"));
+      else if (starts (s, "-psn"));
       else {
 	cout << "\n";
 	cout << "Options for TeXmacs:\n\n";
@@ -165,10 +186,10 @@ TeXmacs_main (int argc, char** argv) {
   init_plugins ();
   bench_cumul ("initialize plugins");
   if (DEBUG_STD) cout << "TeXmacs] Opening display...\n";
-  display dis= open_display (argc, argv);
-  dis->set_default_font (the_default_font);
+  gui_open (argc, argv);
+  set_default_font (the_default_font);
   if (DEBUG_STD) cout << "TeXmacs] Starting server...\n";
-  server sv (dis);
+  server sv;
 
   for (i=1; i<argc; i++) {
     string s= argv[i];
@@ -206,10 +227,10 @@ TeXmacs_main (int argc, char** argv) {
 
   if (DEBUG_STD) cout << "TeXmacs] Starting event loop...\n";
   texmacs_started= true;
-  dis->event_loop ();
+  gui_start_loop ();
 
   if (DEBUG_STD) cout << "TeXmacs] Closing display...\n";
-  close_display (dis);
+  gui_close ();
   if (DEBUG_STD) cout << "TeXmacs] Good bye...\n";
 }
 
@@ -226,41 +247,53 @@ immediate_options (int argc, char** argv) {
     string s= argv[i];
     if ((N(s)>=2) && (s(0,2)=="--")) s= s (1, N(s));
     if ((s == "-S") || (s == "-setup")) {
-      remove ("$TEXMACS_HOME_PATH/system/settings.scm");
-      remove ("$TEXMACS_HOME_PATH/system/setup.scm");
-      remove ("$TEXMACS_HOME_PATH/system/cache" * url_wildcard ("*"));
-      remove ("$TEXMACS_HOME_PATH/fonts/error" * url_wildcard ("*"));
+      remove (url ("$TEXMACS_HOME_PATH/system/settings.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/system/setup.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("*"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/error") * url_wildcard ("*"));
     }
     else if (s == "-delete-cache")
-      remove ("$TEXMACS_HOME_PATH/system/cache" * url_wildcard ("*"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("*"));
     else if (s == "-delete-style-cache")
-      remove ("$TEXMACS_HOME_PATH/system/cache" * url_wildcard ("__*"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("__*"));
     else if (s == "-delete-font-cache")
-      remove ("$TEXMACS_HOME_PATH/system/cache/font_cache.scm");
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/font_cache.scm"));
     else if (s == "-delete-doc-cache") {
-      remove ("$TEXMACS_HOME_PATH/system/cache/doc_cache");
-      remove ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm");
-      remove ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm");
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/doc_cache"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm"));
     }
     else if (s == "-delete-file-cache") {
-      remove ("$TEXMACS_HOME_PATH/system/cache/doc_cache");
-      remove ("$TEXMACS_HOME_PATH/system/cache/file_cache");
-      remove ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm");
-      remove ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm");
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/doc_cache"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/file_cache"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm"));
     }
   }
 }
 
 int
 main (int argc, char** argv) {
-  // cout << "Bench  ] Started TeXmacs\n";
+#ifdef AQUATEXMACS
+  mac_fix_paths ();
+#endif
+  //cout << "Bench  ] Started TeXmacs\n";
   the_et     = tuple ();
   the_et->obs= ip_observer (path ());
+#ifdef EXPERIMENTAL
+  global_notify_assign (path (), tuple ());
+#endif
   immediate_options (argc, argv);
   cache_initialize ();
   bench_start ("initialize texmacs");
   init_texmacs ();
   bench_cumul ("initialize texmacs");
+#ifdef ENABLE_TESTS
+  test_routines ();
+#endif
+//#ifdef EXPERIMENTAL
+//  test_environments ();
+//#endif
   start_guile (argc, argv, TeXmacs_main);
   return 0;
 }
