@@ -14,8 +14,9 @@
 #include "formatter.hpp"
 #include "Graphics/point.hpp"
 #include "timer.hpp"
-#include "PsDevice/printer.hpp"
+#include "printer.hpp"
 #include "file.hpp"
+#include "merge_sort.hpp"
 
 /******************************************************************************
 * Default settings for virtual routines
@@ -23,6 +24,8 @@
 
 int box_rep::subnr () { return 0; }
 box box_rep::subbox (int i) { (void) i; return box (); }
+box box::operator [] (path p) {
+  if (is_nil (p)) return *this; else return rep->subbox(p->item)[p->next]; }
 double box_rep::left_slope () { return 0.0; }
 double box_rep::right_slope () { return 0.0; }
 SI box_rep::left_correction () { return (SI) (-min (0, y1) * left_slope ()); }
@@ -115,7 +118,11 @@ box_rep::find_right_box_path () {
 
 path
 box_rep::find_box_path (path p, bool& found) {
-  found= (!nil(p)) && is_accessible (ip);
+  // cout << "Find box path " << box (this) << ", " << p
+  //      << "; " << reverse (ip)
+  //      << ", " << reverse (find_lip ())
+  //      << " -- " << reverse (find_rip ()) << "\n";
+  found= (!is_nil(p)) && is_accessible (ip);
   if (last_item (p) == 0) return path (0);
   else return path (1);
 }
@@ -176,7 +183,7 @@ box_rep::find_check_selection (path lp, path rp) {
 void
 box_rep::relocate (path new_ip, bool force) {
   if (!force)
-    if (nil (ip) || (ip->item >= 0) || (ip == new_ip)) return;
+    if (is_nil (ip) || (ip->item >= 0) || (ip == new_ip)) return;
   ip= new_ip;
   int i, n= subnr ();
   for (i=0; i<n; i++) subbox (i)->relocate (ip, force);
@@ -188,6 +195,76 @@ box_rep::transform (frame fr) {
   return box ();
 }
 
+/******************************************************************************
+* Modified cursor routines in presence of scrolled boxes
+******************************************************************************/
+
+path
+find_innermost_scroll (box b, path p) {
+  // Given a box b and a logical path p, this routine returns 
+  // the longest box path sp such that b[sp] is a scroll node
+  path bp;
+  while (true) {
+    bool found= false;
+    bp= b->find_box_path (p, found);
+    if (found) break;
+    p= path_up (p);
+    if (is_nil (p)) return path ();
+  }
+  bp= path_up (bp);
+  path cp, sp;
+  while (!is_nil (bp)) {
+    if (b->get_type () == SCROLL_BOX) sp= reverse (cp);
+    b = b[bp->item];
+    cp= path (bp->item, cp);
+    bp= bp->next;
+  }
+  if (is_nil (sp)) return sp;
+  else return sp * 0;
+}
+
+path
+find_scrolled_box_path (box b, path sp, SI x, SI y, SI delta) {
+  if (is_nil (sp)) return b->find_box_path (x, y, delta, false);
+  else {
+    int m= sp->item;
+    SI xx= x - b->sx (m), yy= y - b->sy (m);
+    SI dd= delta + get_delta (xx, b[m]->x1, b[m]->x2);
+    return path (m, find_scrolled_box_path (b[m], sp->next, xx, yy, dd));
+  }
+}
+
+path
+find_scrolled_tree_path (box b, path sp, SI x, SI y, SI delta) {
+  path bp= find_scrolled_box_path (b, sp, x, y, delta);
+  //cout << "Find " << x << ", " << y << "; " << delta;
+  //cout << " -> " << bp << "\n";
+  return b->find_tree_path (bp);
+}
+
+void
+find_canvas_info (box b, path sp, SI& x, SI& y, SI& sx, SI& sy,
+		  rectangle& outer, rectangle& inner)
+{
+  if (is_nil (sp)) {
+    x= y= sx= sy= 0;
+    outer= inner= rectangle (0, 0, 0, 0);
+  }
+  else if (is_atom (sp)) {
+    x    = 0;
+    y    = 0;
+    sx   = b->sx (0);
+    sy   = b->sy (0);
+    outer= rectangle (b->x1, b->y1, b->x2, b->y2);
+    inner= rectangle (b[0]->x1, b[0]->y1, b[0]->x2, b[0]->y2);
+  }
+  else {
+    find_canvas_info (b[sp->item], sp->next, x, y, sx, sy, outer, inner);
+    x += b->sx (sp->item);
+    y += b->sy (sp->item);
+  }
+}
+
 /******************************************************************************
 * For graphical boxes
 ******************************************************************************/
@@ -208,18 +285,22 @@ box_rep::get_limits (point& lim1, point& lim2) {
 }
 
 frame
-box_rep::find_frame (path bp) {
+box_rep::find_frame (path bp, bool last) {
   SI    x= 0;
   SI    y= 0;
   box   b= this;
   frame f= get_frame ();
-  while (!nil (bp)) {
+  while (!is_nil (bp)) {
     x += b->sx (bp->item);
     y += b->sy (bp->item);
     b  = b->subbox (bp->item);
     bp = bp->next;
     frame g= b->get_frame ();
-    if (!nil (g)) f= scaling (1.0, point (x, y)) * g;
+    if (!is_nil (g))
+      if (last)
+	f= g;
+      else
+	f= scaling (1.0, point (x, y)) * g;
   }
   return f;
 }
@@ -228,11 +309,11 @@ grid
 box_rep::find_grid (path bp) {
   box   b= this;
   grid g= get_grid ();
-  while (!nil (bp)) {
+  while (!is_nil (bp)) {
     b  = b->subbox (bp->item);
     bp = bp->next;
     grid g2= b->get_grid ();
-    if (!nil (g2)) g= g2;
+    if (!is_nil (g2)) g= g2;
   }
   return g;
 }
@@ -241,7 +322,7 @@ void
 box_rep::find_limits (path bp, point& lim1, point& lim2) {
   box b= this;
   get_limits (lim1, lim2);
-  while (!nil (bp)) {
+  while (!is_nil (bp)) {
     point slim1, slim2;
     b  = b->subbox (bp->item);
     bp = bp->next;
@@ -274,6 +355,7 @@ box_rep::graphical_select (SI x, SI y, SI dist) {
     gs->cp << find_tree_path (x, y, dist);
     // FIXME: check whether this is correct: I do not remember whether
     // find_tree_path returns an absolute or a relative path
+    gs->c= curve ();
     res << gs;
   }
   return res;
@@ -288,6 +370,8 @@ box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
     SI dist= (SI)norm (point (x2-x1, y2-y1));
     gs->cp << find_tree_path (x1, y1, dist);
     // FIXME: as above, check whether this is correct or not
+    gs->pts= array<point> (0);
+    gs->c= curve ();
     res << gs;
   }
   return res;
@@ -303,7 +387,8 @@ box_rep::get_type () {
 }
 
 tree
-box_rep::get_info () {
+box_rep::get_info (tree in) {
+  (void) in;
   return "";
 }
 
@@ -376,11 +461,11 @@ box_rep::get_leaf_offset (string search) {
 int nr_painted= 0;
 
 void
-clear_rectangles (ps_device dev, rectangles l) {
-  while (!nil (l)) {
+clear_pattern_rectangles (renderer ren, rectangles l) {
+  while (!is_nil (l)) {
     rectangle r (l->item);
-    dev->clear (r->x1- dev->ox, r->y1- dev->oy,
-		r->x2- dev->ox, r->y2- dev->oy);
+    ren->clear_pattern (r->x1- ren->ox, r->y1- ren->oy,
+			r->x2- ren->ox, r->y2- ren->oy);
     l= l->next;
   }
 }
@@ -400,26 +485,26 @@ box_rep::reindex (int i, int item, int n) {
 }
 
 void
-box_rep::redraw (ps_device dev, path p, rectangles& l) {
-  if (((nr_painted&15) == 15) && dev->interrupted (true)) return;
-  dev->move_origin (x0, y0);
-  SI delta= dev->pixel; // adjust visibility to compensate truncation
-  if (dev->is_visible (x3- delta, y3- delta, x4+ delta, y4+ delta)) {
+box_rep::redraw (renderer ren, path p, rectangles& l) {
+  if (((nr_painted&15) == 15) && ren->interrupted (true)) return;
+  ren->move_origin (x0, y0);
+  SI delta= ren->pixel; // adjust visibility to compensate truncation
+  if (ren->is_visible (x3- delta, y3- delta, x4+ delta, y4+ delta)) {
     rectangles ll;
     l= rectangles();
-    pre_display (dev);
+    pre_display (ren);
 
     int i, item=-1, n=subnr (), i1= n, i2= -1;
-    if (!nil(p)) i1= i2= item= p->item;
+    if (!is_nil(p)) i1= i2= item= p->item;
     for (i=0; i<n; i++) {
       int k= reindex (i, item, n-1);
-      if (nil(p)) subbox (k)->redraw (dev, path (), ll);
+      if (is_nil(p)) subbox (k)->redraw (ren, path (), ll);
       else if (i!=0) {
-	if (k > item) subbox(k)->redraw (dev, path (0), ll);
-	else subbox(k)->redraw (dev, path (subbox(k)->subnr()-1), ll);
+	if (k > item) subbox(k)->redraw (ren, path (0), ll);
+	else subbox(k)->redraw (ren, path (subbox(k)->subnr()-1), ll);
       }
-      else subbox(k)->redraw (dev, p->next, ll);
-      if (!nil(ll)) {
+      else subbox(k)->redraw (ren, p->next, ll);
+      if (!is_nil(ll)) {
 	i1= min (i1, k);
 	i2= max (i2, k);
 	l = ll * l;
@@ -427,28 +512,28 @@ box_rep::redraw (ps_device dev, path p, rectangles& l) {
       }
     }
 
-    if (((nr_painted&15) == 15) && dev->interrupted ()) {
-      l= translate (l, -dev->ox, -dev->oy);
-      clear_incomplete (l, dev->pixel, item, i1, i2);
-      l= translate (l, dev->ox, dev->oy);
+    if (((nr_painted&15) == 15) && ren->interrupted ()) {
+      l= translate (l, -ren->ox, -ren->oy);
+      clear_incomplete (l, ren->pixel, item, i1, i2);
+      l= translate (l, ren->ox, ren->oy);
     }
     else {
-      l= rectangle (x3+ dev->ox, y3+ dev->oy, x4+ dev->ox, y4+ dev->oy);
-      display (dev);
-      if (nr_painted < 15) dev->apply_shadow (x1, y1, x2, y2);
+      l= rectangle (x3+ ren->ox, y3+ ren->oy, x4+ ren->ox, y4+ ren->oy);
+      display (ren);
+      if (nr_painted < 15) ren->apply_shadow (x1, y1, x2, y2);
       nr_painted++;
     }
 
-    post_display (dev);
+    post_display (ren);
   }
-  dev->move_origin (-x0, -y0);
+  ren->move_origin (-x0, -y0);
 }
 
 void
-box_rep::redraw (ps_device dev, path p, rectangles& l, SI x, SI y) {
-  dev->move_origin (x, y);
-  redraw (dev, p, l);
-  dev->move_origin (-x, -y);
+box_rep::redraw (renderer ren, path p, rectangles& l, SI x, SI y) {
+  ren->move_origin (x, y);
+  redraw (ren, p, l);
+  ren->move_origin (-x, -y);
 }
 
 void
@@ -457,13 +542,13 @@ box_rep::clear_incomplete (rectangles& rs, SI pixel, int i, int i1, int i2) {
 }
 
 void
-box_rep::pre_display (ps_device &dev) {
-  (void) dev;
+box_rep::pre_display (renderer &ren) {
+  (void) ren;
 }
 
 void
-box_rep::post_display (ps_device &dev) {
-  (void) dev;
+box_rep::post_display (renderer &ren) {
+  (void) ren;
 }
 
 /******************************************************************************
@@ -511,13 +596,13 @@ operator << (ostream& out, cursor cu) {
 * Selections
 ******************************************************************************/
 
-selection::selection (rectangles rs, path start, path end):
+selection::selection (rectangles rs, path start, path end, bool valid):
   rep (new selection_rep)
 {
   rep->rs   = rs;
   rep->start= start;
   rep->end  = end;
-  rep->valid= true;
+  rep->valid= valid;
 }
 
 bool
@@ -553,6 +638,20 @@ operator << (ostream& out, gr_selection sel) {
   return out << "gr_selection (" << sel->dist << ", " << sel->cp << ")";
 }
 
+struct less_eq_gr_selection {
+  static inline bool leq (gr_selection& a, gr_selection& b) {
+    return a->dist <= b->dist; }
+};
+
+tree as_tree (gr_selections sels) {
+  merge_sort_leq <gr_selection, less_eq_gr_selection> (sels);
+  int i, n= N(sels);
+  array<array<path> > res (n);
+  for (i=0; i<n; i++)
+    res[i]= sels[i]->cp;
+  return (tree) res;
+}
+
 /******************************************************************************
 * Animations
 ******************************************************************************/
@@ -650,6 +749,13 @@ box_rep::action (tree t, SI x, SI y, SI delta) {
   return "";
 }
 
+void
+box_rep::loci (SI x, SI y, SI delta, list<string>& ids, rectangles& rs) {
+  (void) x; (void) y; (void) delta;  
+  ids= list<string> ();
+  rs = rectangles ();
+}
+
 void
 box_rep::position_at (SI x, SI y, rectangles& change_log) {
   int i, n= subnr ();
@@ -676,7 +782,7 @@ ostream& operator << (ostream& out, box b) { return out << ((tree) b); }
 
 path
 descend_decode (path ip, int side) {
-  if (nil (ip)) return descend (ip, side);
+  if (is_nil (ip)) return descend (ip, side);
   else switch (ip->item) {
   case DECORATION       : return ip->next;
   case DECORATION_LEFT  : return descend (ip->next, 0);
@@ -710,17 +816,17 @@ attach_dip (tree ref, path dip) {
 ******************************************************************************/
 
 void
-make_eps (url name, display dis, box b, int dpi) {
+make_eps (url name, box b, int dpi) {
   double inch= ((double) dpi * PIXEL);
   double cm  = inch / 2.54;
   SI w= b->x4 - b->x3;
   SI h= b->y4 - b->y3;
   b->x0= -b->x3;
   b->y0= -b->y4;
-  ps_device dev= printer (dis, name, dpi, 1, "user", false, w/cm, h/cm);
-  dev->set_color (dis->black);
-  dev->set_background (dis->white);
+  renderer ren= printer (name, dpi, 1, "user", false, w/cm, h/cm);
+  ren->set_color (black);
+  ren->set_background (white);
   rectangles rs;
-  b->redraw (dev, path (0), rs);
-  delete dev;
+  b->redraw (ren, path (0), rs);
+  delete ren;
 }
