@@ -99,8 +99,8 @@ composite_box_rep::left_justify () {
 ******************************************************************************/
 
 void
-composite_box_rep::display (ps_device dev) {
-  (void) dev;
+composite_box_rep::display (renderer ren) {
+  (void) ren;
 }
 
 int
@@ -121,6 +121,18 @@ composite_box_rep::action (tree t, SI x, SI y, SI delta) {
 			     delta + get_delta (x, x1, x2));
 }
 
+void
+composite_box_rep::loci (SI x, SI y, SI delta,
+			 list<string>& ids, rectangles& rs)
+{
+  int m= find_child (x, y, delta, true);
+  if (m == -1) box_rep::loci (x, y, delta, ids, rs);
+  else {
+    bs[m]->loci (x- sx(m), y- sy(m), delta + get_delta (x, x1, x2), ids, rs);
+    rs= translate (rs, sx(m), sy(m));
+  }
+}
+
 void
 composite_box_rep::collect_page_numbers (hashmap<string,tree>& h, tree page) {
   int i, n= N(bs);
@@ -133,7 +145,7 @@ composite_box_rep::find_tag (string name) {
   int i, n= N(bs);
   for (i=0; i<n; i++) {
     path p= bs[i]->find_tag (name);
-    if (!nil (p)) return p;
+    if (!is_nil (p)) return p;
   }
   return path ();
 }
@@ -148,9 +160,9 @@ composite_box_rep::transform (frame fr) {
   int i;
   array<box> bs;
   for (i= 0; i<subnr(); i++) {
-    if (!nil (subbox (i))) {
+    if (!is_nil (subbox (i))) {
       box sb= subbox (i)->transform (fr);
-      if (!nil (sb)) bs << sb;
+      if (!is_nil (sb)) bs << sb;
     }
   }
   return N (bs)==0?box ():composite_box (ip, bs);
@@ -223,6 +235,10 @@ composite_box_rep::find_rip () {
 
 path
 composite_box_rep::find_box_path (path p, bool& found) {
+  // cout << "Find box path " << box (this) << ", " << p
+  //      << "; " << reverse (ip)
+  //      << ", " << reverse (find_lip ())
+  //      << " -- " << reverse (find_rip ()) << "\n";
   int n= subnr();
   // cout << "Search cursor " << p << " among " << n
   //      << " at " << box (this) << " " << reverse (ip) << "\n";
@@ -296,13 +312,13 @@ composite_box_rep::find_box_path (path p, bool& found) {
 
 path
 composite_box_rep::find_tree_path (path bp) {
-  if (atom (bp)) return box_rep::find_tree_path (bp);
+  if (is_atom (bp)) return box_rep::find_tree_path (bp);
   return bs[bp->item]->find_tree_path (bp->next);
 }
 
 cursor
 composite_box_rep::find_cursor (path bp) {
-  if (atom (bp)) return box_rep::find_cursor (bp);
+  if (is_atom (bp)) return box_rep::find_cursor (bp);
   else {
     int i= bp->item;
     cursor cu= bs[i]->find_cursor (bp->next);
@@ -315,7 +331,7 @@ composite_box_rep::find_cursor (path bp) {
 
 selection
 composite_box_rep::find_selection (path lbp, path rbp) {
-  if ((!atom (lbp)) && (!atom (rbp)) && (lbp->item == rbp->item)) {
+  if ((!is_atom (lbp)) && (!is_atom (rbp)) && (lbp->item == rbp->item)) {
     int i= lbp->item;
     selection sel= bs[i]->find_selection (lbp->next, rbp->next);
     return selection (translate (sel->rs, sx(i), sy(i)), sel->start, sel->end);
