@@ -30,9 +30,10 @@ struct graphics_box_rep: public composite_box_rep {
   grid get_grid ();
   void  get_limits (point& lim1, point& lim2);
   operator tree () { return "graphics"; }
-  void pre_display (ps_device &dev);
-  void post_display (ps_device &dev);
+  void pre_display (renderer &ren);
+  void post_display (renderer &ren);
   int reindex (int i, int item, int n);
+  virtual int find_child (SI x, SI y, SI delta, bool force);
   gr_selections graphical_select (SI x1, SI y1, SI x2, SI y2);
 };
 
@@ -45,6 +46,10 @@ graphics_box_rep::graphics_box_rep (
   y1= (SI) min (flim1[1], flim2[1]);
   x2= (SI) max (flim1[0], flim2[0]);
   y2= (SI) max (flim1[1], flim2[1]);
+  x3= max (x1, x3);
+  y3= max (y1, y3);
+  x4= min (x2, x4);
+  y4= min (y2, y4);
   finalize ();
 }
 
@@ -65,14 +70,14 @@ graphics_box_rep::get_limits (point& lim1b, point& lim2b) {
 }
 
 void
-graphics_box_rep::pre_display (ps_device &dev) {
-  dev->get_clipping (old_clip_x1, old_clip_y1, old_clip_x2, old_clip_y2);
-  dev->extra_clipping (x1, y1, x2, y2);
+graphics_box_rep::pre_display (renderer &ren) {
+  ren->get_clipping (old_clip_x1, old_clip_y1, old_clip_x2, old_clip_y2);
+  ren->extra_clipping (x1, y1, x2, y2);
 }
 
 void
-graphics_box_rep::post_display (ps_device &dev) {
-  dev->set_clipping (
+graphics_box_rep::post_display (renderer &ren) {
+  ren->set_clipping (
     old_clip_x1, old_clip_y1, old_clip_x2, old_clip_y2, true);
 }
 
@@ -81,13 +86,28 @@ graphics_box_rep::reindex (int i, int item, int n) {
   return i;
 }
 
+int
+graphics_box_rep::find_child (SI x, SI y, SI delta, bool force) {
+  int m= composite_box_rep::find_child (x, y, delta, force);
+  if (m==-1) return -1;
+  int i, n= subnr();
+  for (i=0; i<n; i++)
+    if (distance (i, x, y, delta)==0) {
+      tree ty= (tree)bs[i];
+      if ((bs[i]->accessible () || force)
+       && is_tuple (ty) && ty[0]=="textat")
+        return i;
+    }
+  return m;
+}
+
 gr_selections
 graphics_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
   gr_selections res;
   int i, n= subnr();
   for (i=0; i<n; i++)
     res << bs[i]->graphical_select (x1- sx(i), y1- sy(i),
-                                    x2- sx(i), y2- sy(i));
+				    x2- sx(i), y2- sy(i));
   return res;
 }
 /*NOTE: It seems that the dimensions of the boxes that inherit from
@@ -121,7 +141,10 @@ graphics_group_box_rep::graphical_select (SI x, SI y, SI dist) {
   if (graphical_distance (x, y) <= dist) {
     gr_selection gs;
     gs->dist= graphical_distance (x, y);
+  //gs->p= point (x, y); // The cursor moves freely inside the box
     gs->cp << reverse (path (0, ip));
+    gs->pts= array<point> (0);
+    gs->c= curve ();
     res << gs;
   }
   return res;
@@ -134,6 +157,8 @@ graphics_group_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
     gr_selection gs;
     gs->dist= graphical_distance (x1, y1);
     gs->cp << reverse (path (0, ip));
+    gs->pts= array<point> (0);
+    gs->c= curve ();
     res << gs;
   }
   return res;
@@ -159,7 +184,8 @@ struct point_box_rep: public box_rep {
     path ip, point p, SI radius, color col,
     int fill, color fill_col, string style);
   SI graphical_distance (SI x, SI y) { return (SI)norm (p - point (x, y)); }
-  void display (ps_device dev);
+  gr_selections graphical_select (SI x, SI y, SI dist);
+  void display (renderer ren);
   operator tree () { return "point"; }
 };
 
@@ -175,8 +201,23 @@ point_box_rep::point_box_rep (
   y2= y4= ((SI) p[1]) + r;
 }
 
+gr_selections
+point_box_rep::graphical_select (SI x, SI y, SI dist) {
+  gr_selections res;
+  if (graphical_distance (x, y) <= dist) {
+    gr_selection gs;
+    gs->dist= graphical_distance (x, y);
+    gs->p= p;
+    gs->cp << reverse (path (0, ip));
+    gs->pts << p;
+    gs->c= curve ();
+    res << gs;
+  }
+  return res;
+}
+
 void
-point_box_rep::display (ps_device dev) {
+point_box_rep::display (renderer ren) {
   array<SI> x (4), y (4);
   x[0]= ((SI) p[0]) - r;
   y[0]= ((SI) p[1]) - r;
@@ -186,34 +227,34 @@ point_box_rep::display (ps_device dev) {
   y[2]= ((SI) p[1]) + r;
   x[3]= ((SI) p[0]) + r;
   y[3]= ((SI) p[1]) - r;
-  dev->set_line_style (PIXEL);
+  ren->set_line_style (PIXEL);
   if (style == "square") {
     if (fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-      dev->set_color (fill_col);
-      dev->line (x[0], y[0], x[1], y[1]);
-      dev->line (x[1], y[1], x[2], y[2]);
-      dev->line (x[2], y[2], x[3], y[3]);
-      dev->line (x[3], y[3], x[0], y[0]);
-      dev->polygon (x, y, false);
+      ren->set_color (fill_col);
+      ren->line (x[0], y[0], x[1], y[1]);
+      ren->line (x[1], y[1], x[2], y[2]);
+      ren->line (x[2], y[2], x[3], y[3]);
+      ren->line (x[3], y[3], x[0], y[0]);
+      ren->polygon (x, y, false);
     }
     if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-      dev->set_color (col);
-      dev->line (x[0], y[0], x[1], y[1]);
-      dev->line (x[1], y[1], x[2], y[2]);
-      dev->line (x[2], y[2], x[3], y[3]);
-      dev->line (x[3], y[3], x[0], y[0]);
+      ren->set_color (col);
+      ren->line (x[0], y[0], x[1], y[1]);
+      ren->line (x[1], y[1], x[2], y[2]);
+      ren->line (x[2], y[2], x[3], y[3]);
+      ren->line (x[3], y[3], x[0], y[0]);
     }
   }
   else {
     if (style == "disk"
      || fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-      dev->set_color (style == "disk" ? col : fill_col);
-      dev->arc (x[0], y[0]+dev->pixel, x[2], y[2]+dev->pixel, 0, 64*360);
-      dev->fill_arc (x[0], y[0]+dev->pixel, x[2], y[2]+dev->pixel, 0, 64*360);
+      ren->set_color (style == "disk" ? col : fill_col);
+      ren->arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
+      ren->fill_arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
     }
     if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-      dev->set_color (col);
-      dev->arc (x[0], y[0]+dev->pixel, x[2], y[2]+dev->pixel, 0, 64*360);
+      ren->set_color (col);
+      ren->arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
     }
   }
 }
@@ -241,7 +282,7 @@ struct curve_box_rep: public box_rep {
   SI graphical_distance (SI x, SI y);
   gr_selections graphical_select (SI x, SI y, SI dist);
   gr_selections graphical_select (SI x1, SI y1, SI x2, SI y2);
-  void display (ps_device dev);
+  void display (renderer ren);
   operator tree () { return "curve"; }
   SI length ();
   void apply_style ();
@@ -269,31 +310,31 @@ curve_box_rep::curve_box_rep (path ip2, curve c2, SI W, color C,
   arrows= array<box>(2);
   point p1, p2;
   bool error;
-  if (N(arrows2)>0 && !nil (arrows2[0])) {
+  if (N(arrows2)>0 && !is_nil (arrows2[0])) {
     point tg= c->grad (0.0, error);
     if (!error) {
       frame fr= scaling (1.0, a[0]) *
-	        rotation_2D (point (0.0, 0.0), arg (tg));
+		rotation_2D (point (0.0, 0.0), arg (tg));
       arrows[0]= arrows2[0]->transform (fr);
-      if (!nil (arrows[0])) {
-        x1= min (x1, arrows[0]->x1);
-        y1= min (y1, arrows[0]->y1);
-        x2= max (x2, arrows[0]->x2);
-        y2= max (y2, arrows[0]->y2);
+      if (!is_nil (arrows[0])) {
+	x1= min (x1, arrows[0]->x1);
+	y1= min (y1, arrows[0]->y1);
+	x2= max (x2, arrows[0]->x2);
+	y2= max (y2, arrows[0]->y2);
       }
     }
   }
-  if (N(arrows2)>1 && !nil (arrows2[1])) {
+  if (N(arrows2)>1 && !is_nil (arrows2[1])) {
     point tg= c->grad (1.0, error);
     if (!error) {
       frame fr= scaling (1.0, a[N(a)-1]) *
-	        rotation_2D (point (0.0, 0.0), arg (tg));
+		rotation_2D (point (0.0, 0.0), arg (tg));
       arrows[1]= arrows2[1]->transform (fr);
-      if (!nil (arrows[1])) {
-        x1= min (x1, arrows[1]->x1);
-        y1= min (y1, arrows[1]->y1);
-        x2= max (x2, arrows[1]->x2);
-        y2= max (y2, arrows[1]->y2);
+      if (!is_nil (arrows[1])) {
+	x1= min (x1, arrows[1]->x1);
+	y1= min (y1, arrows[1]->y1);
+	x2= max (x2, arrows[1]->x2);
+	y2= max (y2, arrows[1]->y2);
       }
     }
   }
@@ -334,10 +375,13 @@ curve_box_rep::graphical_select (SI x, SI y, SI dist) {
     for (i=0; i<N(pts); i++) {
       SI n= (SI)norm (p - pts[i]);
       if (n <= dist) {
-        gr_selection gs;
-        gs->dist= n;
-        gs->cp << reverse (paths[i]);
-        res << gs;
+	gr_selection gs;
+	gs->dist= n;
+	gs->p= pts[i];
+	gs->cp << reverse (paths[i]);
+	gs->pts << pts[i];
+	gs->c= c;
+	res << gs;
       }
     }
     if (N(res) != 0) return res;
@@ -348,15 +392,19 @@ curve_box_rep::graphical_select (SI x, SI y, SI dist) {
       bool b;
       double t= c->find_closest_point (abs[i], abs[(i+1)%np], p, PIXEL, b);
       if (b) {
-        point p2= c->evaluate (t);
-        SI n= (SI)norm (p - p2);
-        if (n <= dist) {
-          gr_selection gs;
-          gs->dist= n;
-          gs->cp << reverse (paths[i]);
-          gs->cp << reverse (paths[(i+1)%np]);
-          res << gs;
-        }
+	point p2= c->evaluate (t);
+	SI n= (SI)norm (p - p2);
+	if (n <= dist) {
+	  gr_selection gs;
+	  gs->dist= n;
+	  gs->p= p2;
+	  gs->cp << reverse (paths[i]);
+	  gs->cp << reverse (paths[(i+1)%np]);
+	  gs->pts << pts[i];
+	  gs->pts << pts[(i+1)%np];
+	  gs->c= c;
+	  res << gs;
+	}
       }
     }
   }
@@ -370,31 +418,38 @@ curve_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
     gr_selection gs;
     gs->dist= graphical_distance (x1, y1);
     gs->cp << reverse (path (0, ip));
+    gs->pts= array<point> (0);
+    gs->c= c;
     res << gs;
   }
   return res;
 }
 
 void
-curve_box_rep::display (ps_device dev) {
+curve_box_rep::display (renderer ren) {
   int i, n;
   if (fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-    dev->set_color (fill_col);
+    ren->set_color (fill_col);
     n= N(a);
     array<SI> x (n), y (n);
     for (i=0; i<n; i++) {
       x[i]= (SI)a[i][0];
       y[i]= (SI)a[i][1];
     }
-    dev->polygon (x, y, false);
+    ren->polygon (x, y, false);
   }
   if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-    dev->set_color (col);
-    dev->set_line_style (width, 0, false);
+    ren->set_color (col);
+    ren->set_line_style (width, 0, false);
+ // TODO: Add options for handling round/nonround joins & line ends
     if (N (style) == 0) {
       n= N(a);
-      for (i=0; i<(n-1); i++)
-        dev->line ((SI) a[i][0], (SI) a[i][1], (SI) a[i+1][0], (SI) a[i+1][1]);
+      array<SI> x (n), y (n);
+      for (i=0; i<n; i++) {
+	x[i]= (SI) a[i][0];
+	y[i]= (SI) a[i][1];
+      }
+      ren->lines (x, y);
     }
     else {
       SI li=0, o=0;
@@ -402,53 +457,55 @@ curve_box_rep::display (ps_device dev) {
       int no;
       point prec= a[0];
       for (no=0; no<N(styled_n); no++) {
-        point seg= a[i+1]-a[i];
-        while (fnull (norm(seg),1e-6) && i+2<N(a)) {
-          i++;
-          seg= a[i+1]-a[i];
-        }
-        if (fnull (norm(seg),1e-6) && i+2>=N(a))
-          break;
-        SI lno= styled_n[no]*style_unit,
-           len= li+(SI)norm(seg);
-        while (i+2<N(a) && lno>len) {
-          li= len;
-          if (no%2!=0) {
-         // 1st subsegment of a dash
-            dev->line ((SI) prec[0], (SI) prec[1],
-                       (SI) a[i+1][0], (SI) a[i+1][1]);
-  	    prec= a[i+1];
-          }
-          i++;
-          seg= a[i+1]-a[i];
-          len= li+(SI)norm(seg);
-        }
-        o= lno-li;
-     /* We could also use this one in order to use lines with
-        round ends. But it doesn't work well when the width
-        of the line becomes bigger than the style unit length.
-        Anyway (although I don't know if there is a way to do
-        lines with round ends in PostScript), our current PostScript
-        output in GhostView uses square line ends, so we do the same.
-        SI inc= ((no%2==0?1:-1) * width)/2;
-        point b= a[i] + (o+inc)*(seg/norm(seg)); */
-        if (i<N(a)) {
-          point b= a[i] + o*(seg/norm(seg));
-          if (no%2==0)
-            prec= b;
-          else
-         // Last subsegment of a dash
-            dev->line ((SI) prec[0], (SI) prec[1], (SI) b[0], (SI) b[1]);
-         // TODO: Use XDrawLines() and the join style to draw correctly
-         //   the subsegments ; implement this for Postscript as well.
-        }
+	array<SI> x, y;
+	point seg= a[i+1]-a[i];
+	while (fnull (norm(seg),1e-6) && i+2<N(a)) {
+	  i++;
+	  seg= a[i+1]-a[i];
+	}
+	if (fnull (norm(seg),1e-6) && i+2>=N(a))
+	  break;
+	SI lno= styled_n[no]*style_unit,
+	   len= li+(SI)norm(seg);
+	while (i+2<N(a) && lno>len) {
+	  li= len;
+	  if (no%2!=0) {
+         // 1st subsegment of a dash, along with the next ones
+	    x << (SI) prec[0];
+	    y << (SI) prec[1];
+	    prec= a[i+1];
+	  }
+	  i++;
+	  seg= a[i+1]-a[i];
+	  len= li+(SI)norm(seg);
+	}
+	if (N(x)>0 && no%2!=0) {
+	  x << (SI) prec[0];
+	  y << (SI) prec[1];
+	}
+	o= lno-li;
+	if (i<N(a)) {
+	  point b= a[i] + o*(seg/norm(seg));
+	  if (no%2==0)
+	    prec= b;
+	  else {
+	 // Last subsegment of a dash
+	    if (N(x)==0) {
+	      x << (SI) prec[0];
+	      y << (SI) prec[1];
+	    }
+	    x << (SI) b[0];
+	    y << (SI) b[1];
+	  }
+	}
+	ren->lines (x, y);
       }
     }
   }
 
   rectangles ll;
-  if (!nil (arrows[0])) arrows[0]->redraw (dev, path (), ll);
-  if (!nil (arrows[1])) arrows[1]->redraw (dev, path (), ll);
+  if (!is_nil (arrows[0])) arrows[0]->redraw (ren, path (), ll);
+  if (!is_nil (arrows[1])) arrows[1]->redraw (ren, path (), ll);
 }
 
 SI
@@ -514,15 +571,15 @@ curve_box_rep::apply_style () {
     for (j=0; j<(i==n1?n2:n); j++) {
       int frag= style[j]?1:0;
       if (frag!=prevfrag) {
-        if (frag==1) {
-          styled_n[no]= nbu;
-          no++;
-        }
-        else
-        if (frag==0 && prevfrag!=-1) {
-          styled_n[no]= nbu;
-          no++;
-        }
+	if (frag==1) {
+	  styled_n[no]= nbu;
+	  no++;
+	}
+	else
+	if (frag==0 && prevfrag!=-1) {
+	  styled_n[no]= nbu;
+	  no++;
+	}
       }
       prevfrag= frag;
       nbu++;
