@@ -58,7 +58,10 @@ box composite_box (path ip, array<box> bs, array<SI> x, array<SI> y,
 box superpose_box (path ip, array<box> bs, bool bfl= true);
 box scatter_box (path ip, array<box> bs, array<SI> x, array<SI> y);
 box cell_box (path ip, box b, SI x0, SI y0, SI x1, SI y1, SI x2, SI y2,
-	      SI bl, SI br, SI bb, SI bt, color fg, color bg, bool transp);
+	      SI bl, SI br, SI bb, SI bt, color fg, tree bg);
+box remember_box (path ip, box b);
+box highlight_box (path ip, box b, SI w, SI xpad, SI ypad,
+		   tree bg, color sunny, color shadow);
 
 box frac_box (path ip, box b1, box b2, font fn, font sfn, color c);
 box sqrt_box (path ip, box b1, box b2, box sqrtb, font fn, color c);
@@ -74,6 +77,7 @@ box side_box (path ip, box ref, box l1, box l2, box r1, box r2, font f, int l);
 box specific_box (path ip, box b, bool printer_flag, font fn);
 box flag_box (path ip, box b, SI h, SI lw, color dark, color light);
 box info_box (path ip, SI h, SI lw, color dark, color light);
+box scrollbar_box (path ip, box b, bool vertical, SI span, tree t);
 
 box symbol_box (path ip, box b, int n);
 box shorter_box (path ip, box b, int n);
@@ -81,14 +85,18 @@ box frozen_box (path ip, box b);
 box move_box (path ip, box b, SI x, SI y, bool chf= false, bool bigf= false);
 box resize_box (path ip, box b, SI x1, SI y1, SI x2, SI y2,
 		bool chf= false, bool adjust= false);
+box clip_box (path ip, box b, SI x1, SI y1, SI x2, SI y2);
+box clip_box (path ip, box b, SI x1, SI y1, SI x2, SI y2,
+	      tree xt, tree yt, SI scx, SI scy);
 box vcorrect_box (path ip, box b, SI top_cor, SI bot_cor);
 box page_box (path ip, tree page, SI w, SI h,
 	      array<box> bs  , array<SI> bs_x  , array<SI> bs_y,
 	      array<box> decs, array<SI> decs_x, array<SI> decs_y);
 box action_box (path ip, box b, tree filter, command cmd, bool child_flag);
 box action_box (path ip, box b, tree f, command c, bool ch, path vip);
+box locus_box (path ip, box b, list<string> ids, SI pixel);
 box macro_box (path ip, box b, font big_fn= font ());
-box tag_box (path ip, box b, string name);
+box tag_box (path ip, box b, tree keys);
 
 box anim_compose_box (path ip, array<box> b);
 box anim_repeat_box (path ip, box b);
