@@ -12,6 +12,7 @@
 
 #include "concater.hpp"
 #include "formatter.hpp"
+#include "analyze.hpp"
 
 lazy make_lazy_vstream (edit_env env, tree t, path ip, tree channel);
 
@@ -25,6 +26,15 @@ concater_rep::typeset_substring (string s, path ip, int pos) {
   a << line_item (STRING_ITEM, b, HYPH_INVALID, env->lan);
 }
 
+void
+concater_rep::typeset_colored_substring
+  (string s, path ip, int pos, string col)
+{
+  color c= (col == ""? env->col: named_color (col));
+  box b= text_box (ip, pos, s, env->fn, c);
+  a << line_item (STRING_ITEM, b, HYPH_INVALID, env->lan);
+}
+
 #define PRINT_SPACE(spc_type) \
   switch (spc_type) { \
   case SPC_NONE: \
@@ -32,6 +42,9 @@ concater_rep::typeset_substring (string s, path ip, int pos) {
   case SPC_SPACE: \
     print (spc); \
     break; \
+  case SPC_DSPACE: \
+    print (space (spc->min << 1, spc->def << 1, spc->max << 1)); \
+    break; \
   case SPC_PERIOD: \
     print (spc+ extra); \
     break; \
@@ -68,7 +81,34 @@ concater_rep::typeset_substring (string s, path ip, int pos) {
   }
 
 void
-concater_rep::typeset_string (string s, path ip) {
+concater_rep::typeset_text_string (string s, path ip) {
+  int    start, pos=0;
+  space  spc= env->fn->spc;
+  space  extra= env->fn->extra;
+
+  do {
+    start= pos;
+    text_property tp= env->lan->advance (s, pos);
+    if ((pos>start) && (s[start]==' ')) { // spaces
+      if (start==0) typeset_substring ("", ip, 0);
+      penalty_min (tp->pen_after);
+      PRINT_SPACE (tp->spc_before);
+      PRINT_SPACE (tp->spc_after);
+      if ((pos==N(s)) || (s[pos]==' '))
+	typeset_substring ("", ip, pos);
+    }
+    else { // strings
+      penalty_max (tp->pen_before);
+      PRINT_SPACE (tp->spc_before)
+      typeset_substring (s (start, pos), ip, start);
+      penalty_min (tp->pen_after);
+      PRINT_SPACE (tp->spc_after)
+    }
+  } while (pos<N(s));
+}
+
+void
+concater_rep::typeset_math_string (string s, path ip) {
   int    start, pos=0;
   space  spc= env->fn->spc;
   space  extra= env->fn->extra;
@@ -98,6 +138,35 @@ concater_rep::typeset_string (string s, path ip) {
   } while (pos<N(s));
 }
 
+void
+concater_rep::typeset_prog_string (tree t, path ip) {
+  string s= t->label;
+  int    start, pos=0;
+  space  spc= env->fn->spc;
+  space  extra= env->fn->extra;
+
+  do {
+    start= pos;
+    text_property tp= env->lan->advance (s, pos);
+    if ((pos>start) && (s[start]==' ')) { // spaces
+      if (start==0) typeset_substring ("", ip, 0);
+      penalty_min (tp->pen_after);
+      PRINT_SPACE (tp->spc_before);
+      PRINT_SPACE (tp->spc_after);
+      if ((pos==N(s)) || (s[pos]==' '))
+	typeset_substring ("", ip, pos);
+    }
+    else { // strings
+      penalty_max (tp->pen_before);
+      PRINT_SPACE (tp->spc_before)
+      typeset_colored_substring (s (start, pos), ip, start,
+				 env->lan->get_color (t, start, pos));
+      penalty_min (tp->pen_after);
+      PRINT_SPACE (tp->spc_after)
+    }
+  } while (pos<N(s));
+}
+
 /******************************************************************************
 * Typesetting errors, documents and concatenations
 ******************************************************************************/
@@ -135,6 +204,7 @@ concater_rep::typeset_concat (tree t, path ip) {
 
 void
 concater_rep::typeset_hspace (tree t, path ip) {
+  if (N(a)==0) print (STD_ITEM, empty_box (ip, 0, 0, 0, env->fn->yx));
   if (N(t)==1) print (env->as_hspace (t[0]));
   else print (space (env->as_length (t[0]),
 		     env->as_length (t[1]),
@@ -168,7 +238,7 @@ concater_rep::typeset_move (tree t, path ip) {
   print (STD_ITEM, move_box (ip, b, x, y, true));
 }
 
-static SI
+SI
 resize (edit_env env, SI old, SI minimum, SI maximum, tree new_size) {
   if (!is_atomic (new_size)) return old;
   string s= new_size->label;
@@ -223,6 +293,18 @@ concater_rep::typeset_resize (tree t, path ip) {
   print (STD_ITEM, resize_box (ip, b, x1, y1, x2, y2, true, !fl));
 }
 
+void
+concater_rep::typeset_clipped (tree t, path ip) {
+  // IDEA: set left, right, bottom, top environment variables
+  //       and allow doing computations with them
+  box  b = typeset_as_concat (env, t[4], descend (ip, 4));
+  SI   x1= resize (env, b->x1, b->x1, b->x2, env->exec (t[0]));
+  SI   y1= resize (env, b->y1, b->y1, b->y2, env->exec (t[1]));
+  SI   x2= resize (env, b->x2, b->x1, b->x2, env->exec (t[2]));
+  SI   y2= resize (env, b->y2, b->y2, b->y2, env->exec (t[3]));
+  print (STD_ITEM, clip_box (ip, b, x1, y1, x2, y2));
+}
+
 /******************************************************************************
 * Floating objects
 ******************************************************************************/
@@ -235,7 +317,7 @@ concater_rep::typeset_float (tree t, path ip) {
   lazy lz= make_lazy_vstream (env, t[2], descend (ip, 2), ch);
   marker (descend (ip, 0));
   if (is_accessible (ip) && !env->read_only)
-    flag_ok (as_string (t1), decorate_middle (ip), env->dis->brown);
+    flag_ok (as_string (t1), decorate_middle (ip), brown);
   print (FLOAT_ITEM, control_box (decorate_middle (ip), lz, env->fn));
   marker (descend (ip, 1));
 }
@@ -277,7 +359,7 @@ void
 concater_rep::typeset_decorated_box (tree t, path ip) {
   (void) t; (void) ip;
   int n= N (env->decorated_boxes);
-  if ((n > 0) && (!nil (env->decorated_boxes [n-1]))) {
+  if ((n > 0) && (!is_nil (env->decorated_boxes [n-1]))) {
     print (STD_ITEM, env->decorated_boxes [n-1]);
     env->decorated_boxes [n-1]= box ();
   }
