@@ -13,10 +13,12 @@
 #include "env.hpp"
 #include "convert.hpp"
 #include "file.hpp"
+#include "image_files.hpp"
 #include "scheme.hpp"
-#include "PsDevice/page_type.hpp"
+#include "page_type.hpp"
 #include "typesetter.hpp"
 #include "drd_mode.hpp"
+#include "dictionary.hpp"
 
 extern int script_status;
 
@@ -51,10 +53,10 @@ edit_env_rep::rewrite (tree t) {
       object expr= null_object ();
       for (i=n-1; i>0; i--)
 	expr= cons (object (r[i]), expr);
-      string fun= as_string (exec (t[0]));
+      string fun= exec_string (t[0]);
       expr= cons (string_to_object (fun), expr);
       (void) eval ("(lazy-markup-modules-force)");
-      if (script_status < 2) {
+      if (!secure && script_status < 2) {
 	if (!as_bool (call ("secure?", expr)))
 	  return tree (ERROR, "insecure script");
       }
@@ -68,15 +70,15 @@ edit_env_rep::rewrite (tree t) {
     {
       if (!(is_atomic (t[0]) && is_atomic (t[1]) && is_atomic (t[2])))
 	return tree (ERROR, "invalid map arguments");
-      if (nil (macro_arg) || (!macro_arg->item->contains (t[2]->label)))
+      if (is_nil (macro_arg) || (!macro_arg->item->contains (t[2]->label)))
 	return tree (ERROR, "map arguments " * t[2]->label);
       tree v= macro_arg->item [t[2]->label];
       if (is_atomic (v))
 	return tree (ERROR, "map arguments " * t[2]->label);
       list<hashmap<string,tree> > old_var= macro_arg;
       list<hashmap<string,path> > old_src= macro_src;
-      if (!nil (macro_arg)) macro_arg= macro_arg->next;
-      if (!nil (macro_src)) macro_src= macro_src->next;
+      if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+      if (!is_nil (macro_src)) macro_src= macro_src->next;
 
       int start= 0, end= N(v);
       if (N(t)>=4) start= as_int (exec (t[3]));
@@ -94,14 +96,14 @@ edit_env_rep::rewrite (tree t) {
     }
   case INCLUDE:
     {
-      url file_name= url_system (as_string (t[0]));
+      url file_name= url_system (exec_string (t[0]));
       return load_inclusion (relative (base_file_name, file_name));
     }
   case REWRITE_INACTIVE:
     {
       if ((!is_func (t[0], ARG)) ||
 	  is_compound (t[0][0]) ||
-	  nil (macro_arg) ||
+	  is_nil (macro_arg) ||
 	  (!macro_arg->item->contains (t[0][0]->label)))
 	return tree (ERROR, "invalid rewrite-inactive");
       tree val= macro_arg->item [t[0][0]->label];
@@ -150,7 +152,7 @@ edit_env_rep::exec_until_rewrite (tree t, path p, string var, int level) {
 tree
 texmacs_exec (edit_env env, tree cmd) {
   // re-entrancy
-  if (!nil (current_rewrite_env)) env= current_rewrite_env;
+  if (!is_nil (current_rewrite_env)) env= current_rewrite_env;
   return env->exec (cmd);
 }
 
@@ -205,6 +207,8 @@ edit_env_rep::exec (tree t) {
     return exec_eval_args (t);
   case MARK:
     return tree (MARK, copy (t[0]), exec (t[1]));
+  case EXPAND_AS:
+    return exec (t[1]);
   case EVAL:
     return exec (exec (t[0]));
   case QUOTE:
@@ -233,6 +237,8 @@ edit_env_rep::exec (tree t) {
     return exec_rewrite (t);
   case USE_PACKAGE:
     return exec_use_package (t);
+  case USE_MODULE:
+    return exec_use_module (t);
 
   case OR:
     return exec_or (t);
@@ -252,6 +258,20 @@ edit_env_rep::exec (tree t) {
     return exec_divide (t);
   case MOD:
     return exec_modulo (t);
+  case MATH_SQRT:
+    return exec_math_sqrt (t);
+  case EXP:
+    return exec_exp (t);
+  case LOG:
+    return exec_log (t);
+  case POW:
+    return exec_pow (t);
+  case COS:
+    return exec_cos (t);
+  case SIN:
+    return exec_sin (t);
+  case TAN:
+    return exec_tan (t);
   case MERGE:
     return exec_merge (t);
   case LENGTH:
@@ -321,6 +341,10 @@ edit_env_rep::exec (tree t) {
     return exec_fns_length ();
   case BLS_LENGTH:
     return exec_bls_length ();
+  case FNBOT_LENGTH:
+    return exec_fnbot_length ();
+  case FNTOP_LENGTH:
+    return exec_fntop_length ();
   case SPC_LENGTH:
     return exec_spc_length ();
   case XSPC_LENGTH:
@@ -359,6 +383,21 @@ edit_env_rep::exec (tree t) {
   case REWRITE_INACTIVE:
     return exec_rewrite (t);
 
+  case HARD_ID:
+    return exec_hard_id (t);
+  case SCRIPT:
+    return exec_script (t);
+  case HLINK:
+  case ACTION:
+    return exec_compound (t);
+  case SET_BINDING:
+    return exec_set_binding (t);
+  case GET_BINDING:
+    return exec_get_binding (t);
+
+  case PATTERN:
+    return exec_pattern (t);
+
   case _POINT:
     return exec_point (t);
   case BOX_INFO:
@@ -523,21 +562,41 @@ edit_env_rep::exec_drd_props (tree t) {
 	if (val == "no") drd->set_no_border (l, true);
 	drd->freeze_no_border (l);
       }
-      if (prop == "unaccessible" || prop == "hidden" || prop == "accessible") {
-	int prop_code= ACCESSIBLE_NEVER;
-	if (prop == "hidden") prop_code= ACCESSIBLE_HIDDEN;
-	if (prop == "accessible") prop_code= ACCESSIBLE_ALWAYS;
-	if (val == "none") prop_code= ACCESSIBLE_NEVER;
-	if (is_int (val))
-	  drd->set_accessible (l, as_int (val), prop_code);
-	else if (val == "none" || val == "all") {
-	  int i, n= drd->get_nr_indices (l);
-	  for (i=0; i<n; i++) {
-	    drd->set_accessible (l, i, prop_code);
-	    drd->freeze_accessible (l, i);
+      if (prop == "unaccessible" ||
+	  prop == "hidden" ||
+	  prop == "accessible")
+	{
+	  int prop_code= ACCESSIBLE_NEVER;
+	  if (prop == "hidden") prop_code= ACCESSIBLE_HIDDEN;
+	  if (prop == "accessible") prop_code= ACCESSIBLE_ALWAYS;
+	  if (val == "none") prop_code= ACCESSIBLE_NEVER;
+	  if (is_int (val))
+	    drd->set_accessible (l, as_int (val), prop_code);
+	  else if (val == "none" || val == "all") {
+	    int i, n= drd->get_nr_indices (l);
+	    for (i=0; i<n; i++) {
+	      drd->set_accessible (l, i, prop_code);
+	      drd->freeze_accessible (l, i);
+	    }
+	  }
+	}
+      if (prop == "normal-writability" ||
+	  prop == "disable-writability" ||
+	  prop == "enable-writability")
+	{
+	  int prop_code= WRITABILITY_NORMAL;
+	  if (prop == "disable-writability") prop_code= WRITABILITY_DISABLE;
+	  if (prop == "enable-writability") prop_code= WRITABILITY_ENABLE;
+	  if (is_int (val))
+	    drd->set_writability (l, as_int (val), prop_code);
+	  else if (val == "all") {
+	    int i, n= drd->get_nr_indices (l);
+	    for (i=0; i<n; i++) {
+	      drd->set_writability (l, i, prop_code);
+	      drd->freeze_writability (l, i);
+	    }
 	  }
 	}
-      }
     }
   return t;
 }
@@ -568,13 +627,13 @@ edit_env_rep::exec_arg (tree t) {
   tree r= t[0];
   if (is_compound (r))
     return tree (ERROR, "bad argument application");
-  if (nil (macro_arg) || (!macro_arg->item->contains (r->label)))
+  if (is_nil (macro_arg) || (!macro_arg->item->contains (r->label)))
     return tree (ERROR, "argument " * r->label);
   r= macro_arg->item [r->label];
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
-  if (!nil (macro_arg)) macro_arg= macro_arg->next;
-  if (!nil (macro_src)) macro_src= macro_src->next;
+  if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+  if (!is_nil (macro_src)) macro_src= macro_src->next;
   if (N(t) > 1) {
     int i, n= N(t);
     for (i=1; i<n; i++) {
@@ -591,12 +650,14 @@ edit_env_rep::exec_arg (tree t) {
   return r;
 }
 
+static bool quote_substitute= false;
+
 tree
 edit_env_rep::exec_quote_arg (tree t) {
   tree r= t[0];
   if (is_compound (r))
     return tree (ERROR, "bad quoted argument application");
-  if (nil (macro_arg) || (!macro_arg->item->contains (r->label)))
+  if (is_nil (macro_arg) || (!macro_arg->item->contains (r->label)))
     return tree (ERROR, "quoted argument " * r->label);
   r= macro_arg->item [r->label];
   if (N(t) > 1) {
@@ -609,6 +670,13 @@ edit_env_rep::exec_quote_arg (tree t) {
       r= r[nr];
     }
   }
+  if (quote_substitute && !is_func (r, ARG)) {
+    int i, n= N(r);
+    tree s (r, n);
+    for (i=0; i<n; i++)
+      s[i]= tree (ARG, A(t)) * tree (ARG, as_string (i));
+    return s;
+  }
   return r;
 }
 
@@ -627,12 +695,12 @@ edit_env_rep::exec_get_arity (tree t) {
 tree
 edit_env_rep::exec_eval_args (tree t) {
   tree v= macro_arg->item [as_string (t[0])];
-  if (is_atomic (v) || nil (macro_arg))
+  if (is_atomic (v) || is_nil (macro_arg))
     return tree (ERROR, "eval arguments " * t[0]->label);
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
-  if (!nil (macro_arg)) macro_arg= macro_arg->next;
-  if (!nil (macro_src)) macro_src= macro_src->next;
+  if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+  if (!is_nil (macro_src)) macro_src= macro_src->next;
 
   int i, n= N(v);
   tree r (v, n);
@@ -769,6 +837,18 @@ edit_env_rep::exec_use_package (tree t) {
   return "";
 }
 
+tree
+edit_env_rep::exec_use_module (tree t) {
+  int i, n= N(t);
+  for (i=0; i<n; i++) {
+    string s= exec_string (t[i]);
+    if (starts (s, "(")) eval ("(use-modules " * s * ")");
+    else if (s != "") eval ("(plugin-initialize '" * s * ")");
+    assign (THE_MODULES, read (THE_MODULES) * tuple (s));
+  }
+  return "";
+}
+
 tree
 edit_env_rep::exec_or (tree t) {
   if (N(t)<2) return tree (ERROR, "bad or");
@@ -850,6 +930,19 @@ edit_env_rep::exec_plus_minus (tree t) {
   else return tree (ERROR, "bad plus/minus");
 }
 
+bool
+is_percentage (tree t, string s= "%") {
+  return
+    is_atomic (t) &&
+    ends (t->label, s) &&
+    is_double (t->label (0, N (t->label) - 1));
+}
+
+double
+as_percentage (tree t) {
+  return as_double (t->label (0, N (t->label) - 1)) / 100.0;
+}
+
 tree
 edit_env_rep::exec_times_over (tree t) {
   int i, n= N(t);
@@ -857,6 +950,7 @@ edit_env_rep::exec_times_over (tree t) {
   tree prod= exec (t[0]);
   if (is_double (prod));
   else if (is_anylen (prod)) prod= as_tmlen (prod);
+  else if (is_percentage (prod)) prod= as_tree (as_percentage (prod));
   else return tree (ERROR, "bad times/over");
   if ((n==1) && is_func (t, OVER)) {
     if (is_double (prod)) return as_string (1 / as_double (prod));
@@ -885,6 +979,12 @@ edit_env_rep::exec_times_over (tree t) {
 	prod= tmlen_times (as_double (prod), mul);
       else return tree (ERROR, "bad times/over");
     }
+    else if (is_percentage (mul)) {
+      double _mul= as_percentage (mul);
+      if (is_double (prod))
+	prod= as_string (_mul * as_double (prod));
+      else prod= tmlen_times (_mul, prod);
+    }
     else return tree (ERROR, "bad times/over");
     // cout << "  " << i << "\t" << prod << "\n";
   }
@@ -921,15 +1021,80 @@ edit_env_rep::exec_modulo (tree t) {
   return tree (ERROR, "bad modulo");
 }
 
+tree
+edit_env_rep::exec_math_sqrt (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad sqrt");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (sqrt (as_double (t1)));
+  return tree (ERROR, "bad sqrt");
+}
+
+tree
+edit_env_rep::exec_exp (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad exp");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (exp (as_double (t1)));
+  return tree (ERROR, "bad exp");
+}
+
+tree
+edit_env_rep::exec_log (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad log");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (log (as_double (t1)));
+  return tree (ERROR, "bad log");
+}
+
+tree
+edit_env_rep::exec_pow (tree t) {
+  if (N(t)!=2) return tree (ERROR, "bad pow");
+  tree t1= exec (t[0]);
+  tree t2= exec (t[1]);
+  if (is_double (t1) && is_double (t2))
+    return as_tree (pow (as_double (t1), as_double (t2)));
+  return tree (ERROR, "bad pow");
+}
+
+tree
+edit_env_rep::exec_cos (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad cos");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (cos (as_double (t1)));
+  return tree (ERROR, "bad cos");
+}
+
+tree
+edit_env_rep::exec_sin (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad sin");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (sin (as_double (t1)));
+  return tree (ERROR, "bad sin");
+}
+
+tree
+edit_env_rep::exec_tan (tree t) {
+  if (N(t)!=1) return tree (ERROR, "bad tan");
+  tree t1= exec (t[0]);
+  if (is_double (t1))
+    return as_tree (tan (as_double (t1)));
+  return tree (ERROR, "bad tan");
+}
+
 tree
 edit_env_rep::exec_merge (tree t) {
   int i, n= N(t);
   if (n == 0) return "";
   tree acc= exec (t[0]);
+  if (is_concat (acc)) acc= var_as_string (acc);
   for (i=1; i<n; i++) {
     tree add= exec (t[i]);
-    if (is_atomic (acc) && is_atomic (add))
-      acc= acc->label * add->label;
+    if (is_atomic (acc) && (is_atomic (add) || is_concat (add)))
+      acc= acc->label * var_as_string (add);
     else if (is_tuple (acc) && is_tuple (add))
       acc= acc * add;
     else if (is_func (acc, MACRO) && is_func (add, MACRO) &&
@@ -1013,8 +1178,7 @@ edit_env_rep::exec_date (tree t) {
     if (is_compound (u)) return tree (ERROR, "bad date");
     lan= u->label;
   }
-  string fm= "%d %B %Y";
-  if (lan == "english") fm= "%B %d, %Y";
+  string fm= "";
   if (N(t) != 0) {
     tree u= exec (t[0]);
     if (is_compound (u)) return tree (ERROR, "bad date");
@@ -1031,7 +1195,7 @@ edit_env_rep::exec_translate (tree t) {
   tree t3= exec (t[2]);
   if (is_compound (t1) || is_compound (t2) || is_compound (t3))
     return tree (ERROR, "bad translate");
-  return dis->translate (t1->label, t2->label, t3->label);
+  return translate (t1->label, t2->label, t3->label);
 }
 
 tree
@@ -1048,7 +1212,7 @@ edit_env_rep::exec_change_case (tree t, tree nc, bool exec_flag, bool first) {
     else if (nc == "UPCASE") { up= true; }
     else if (nc == "locase") { lo= true; }
 
-    for (i=0; i<n; lan->enc->token_forward (s, i))
+    for (i=0; i<n; tm_char_forwards (s, i))
       if (is_iso_alpha (s[i]) && (all || (first && (i==0)))) {
 	if (up && is_locase (s[i])) r->label[i]= upcase (s[i]);
 	if (lo && is_upcase (s[i])) r->label[i]= locase (s[i]);
@@ -1203,13 +1367,148 @@ edit_env_rep::exec_greatereq (tree t) {
   return tree (ERROR, "bad greater or equal");
 }
 
+tree
+edit_env_rep::exec_hard_id (tree t) {
+  if (N(t) == 0) {
+    pointer ptr= (pointer) this;
+    return "%" * as_hexadecimal (ptr);
+  }
+  else {
+    t= expand (t[0], true);
+    pointer ptr1= (pointer) this;
+    pointer ptr2= (pointer) t.operator -> ();
+    return "%" * as_hexadecimal (ptr1) * "-" * as_hexadecimal (ptr2);
+  }
+}
+
+tree
+edit_env_rep::exec_script (tree t) {
+  if (N(t) != 1 && N(t) != 2) return tree (ERROR, "bad script");
+  if (N(t) == 1) return tree (SCRIPT, exec (t[0]));
+  else return tree (SCRIPT, exec (t[0]), expand (t[1], true));
+}
+
+tree
+edit_env_rep::exec_set_binding (tree t) {
+  tree keys, value;
+  if (N(t) == 1) {
+    keys= read ("the-tags");
+    if (!is_tuple (keys))
+      return tree (ERROR, "bad set binding");
+    for (int i=0; i<N(keys); i++)
+      if (!is_atomic (keys[i]))
+	return tree (ERROR, "bad set binding");
+    value= exec (t[0]);
+    assign (string ("the-tags"), tree (TUPLE));
+    assign (string ("the-label"), copy (value));
+  }
+  else if (N(t) >= 2) {
+    tree key= exec (t[0]);
+    if (!is_atomic (key)) 
+      return tree (ERROR, "bad set binding");
+    keys= tuple (key);
+    value= exec (t[1]);
+  }
+  else return tree (ERROR, "bad set binding");
+
+  for (int i=0; i<N(keys); i++) {
+    string key= keys[i]->label;
+    tree old_value= local_ref[key];
+    string part= as_string (read ("current-part"));
+    if (is_func (old_value, TUPLE) && (N(old_value) >= 2))
+      local_ref (key)= tuple (copy (value), old_value[1]);
+    else local_ref (key)= tuple (copy (value), "?");
+    if (cur_file_name != base_file_name || N(part) != 0) {
+      string extra;
+      if (cur_file_name != base_file_name)
+	extra << as_string (delta (base_file_name, cur_file_name));
+      if (N(part) != 0)
+	extra << "#" << part (1, N(part));
+      local_ref (key) << extra;
+    }
+    if (complete && is_tuple (old_value) && N(old_value) >= 1) {
+      string old_s= var_as_string (old_value[0]);
+      string new_s= var_as_string (value);
+      if (new_s != old_s && !starts (key, "auto-")) {
+	if (new_s == "") system_warning ("Redefined", key);
+	else system_warning ("Redefined " * key * " as", new_s);
+      }
+    }
+  }
+  return keys;
+}
+
+tree
+edit_env_rep::exec_get_binding (tree t) {
+  if (N(t) != 1 && N(t) != 2) return tree (ERROR, "bad get binding");
+  string key= exec_string (t[0]);
+  tree value= local_ref->contains (key)? local_ref [key]: global_ref [key];
+  int type= (N(t) == 1? 0: as_int (exec_string (t[1])));
+  if (type != 0 && type != 1) type= 0;
+  if (is_func (value, TUPLE) && (N(value) >= 2)) value= value[type];
+  else if (type == 1) value= tree (UNINIT);
+  if (complete && value == tree (UNINIT))
+    system_warning ("Undefined reference", key);
+  return value;
+}
+
+bool is_magnification (string s);
+double get_magnification (string s);
+
+tree
+edit_env_rep::exec_pattern (tree t) {
+  url im= exec_string (t[0]);
+  url image= resolve (relative (base_file_name, im));
+  if (is_none (image))
+    image= resolve (url ("$TEXMACS_PATTERN_PATH") * im);
+  if (is_none (image)) return "white";
+  int imw_pt, imh_pt;
+  image_size (image, imw_pt, imh_pt);
+  double pt= ((double) dpi*PIXEL) / 72.0;
+  SI imw= (SI) (((double) imw_pt) * pt);
+  SI imh= (SI) (((double) imh_pt) * pt);
+  if (imw <= 0 || imh <= 0) return "white";
+  string w= exec_string (t[1]);
+  string h= exec_string (t[2]);
+  if (is_length (w))
+    w= as_string (as_length (w));
+  else if (is_magnification (w))
+    w= as_string ((SI) (get_magnification (w) * ((double) imw)));
+  if (is_length (h))
+    h= as_string (as_length (h));
+  else if (is_magnification (h))
+    h= as_string ((SI) (get_magnification (h) * ((double) imh)));
+  if (w == "" && h != "") {
+    if (is_int (h)) w= as_string ((SI) ((as_double (h) * imw) / imh));
+    else if (is_percentage (h))
+      w= as_string (100.0 * (as_percentage (h) * imw) / imh) * "@";
+    else return "white";
+  }
+  else if (h == "" && w != "") {
+    if (is_int (w)) h= as_string ((SI) ((as_double (w) * imh) / imw));
+    else if (is_percentage (w))
+      h= as_string (100.0 * (as_percentage (w) * imh) / imw) * "@";
+    else return "white";
+  }
+  else if (w == "" && h == "") {
+    w= as_string (imw);
+    h= as_string (imh);
+  }
+  else if ((!is_int (w) && !is_percentage (w)) ||
+	   (!is_int (h) && !is_percentage (h)))
+    return "white";
+  tree r (PATTERN, as_string (image), w, h);
+  if (N(t) == 4) r << exec (t[3]);
+  return r;
+}
+
 tree
 edit_env_rep::exec_point (tree t) {
   int i, n= N(t);
-  tree u (TUPLE, n);
+  tree u (_POINT, n);
   for (i=0; i<n; i++)
     u[i]= exec (t[i]);
-  if (is_tuple (u) && ((n==0) || is_double (u[0]))) return u;
+  if (n==0 || is_double (u[0])) return u;
   return as_tree (as_point (u));
 }
 
@@ -1225,13 +1524,13 @@ edit_env_rep::exec_box_info (tree t) {
 tree
 edit_env_rep::exec_frame_direct (tree t) {
   tree t1= exec (t[0]);
-  return as_tree (!nil (fr) ? fr (::as_point (t1)) : point ());
+  return as_tree (!is_nil (fr) ? fr (::as_point (t1)) : point ());
 }
 
 tree
 edit_env_rep::exec_frame_inverse (tree t) {
   tree t1= exec (t[0]);
-  return as_tree (!nil (fr) ? fr [::as_point (t1)] : point ());
+  return as_tree (!is_nil (fr) ? fr [::as_point (t1)] : point ());
 }
 
 /******************************************************************************
@@ -1241,8 +1540,8 @@ edit_env_rep::exec_frame_inverse (tree t) {
 void
 edit_env_rep::exec_until (tree t, path p) {
   // cout << "Execute " << t << " until " << p << "\n";
-  if (nil (p)) return;
-  if (atom (p)) {
+  if (is_nil (p)) return;
+  if (is_atom (p)) {
     if (p->item!=0)
       (void) exec (t);
     return;
@@ -1285,6 +1584,10 @@ edit_env_rep::exec_until (tree t, path p) {
   case VAR_INACTIVE:
     exec_until_compound (t, p);
     return;
+  case HLINK:
+  case ACTION:
+    exec_until_compound (t, p);
+    return;
   default:
     if (L(t) < START_EXTENSIONS) {
       int i;
@@ -1362,9 +1665,13 @@ edit_env_rep::exec_until_compound (tree t, path p) {
     f= read (fname);
   }
 
-  if ((p->item < d) || (p->item >= N(f)) ||
-      is_compound (f[p->item-d])) return;
-  string var= f[p->item-d]->label;
+  string var;
+  if (L(f) == XMACRO) var= f[0]->label;
+  else {
+    if ((p->item < d) || (p->item >= N(f)) ||
+	is_compound (f[p->item-d])) return;
+    var= f[p->item-d]->label;
+  }
 
   if (is_applicable (f)) {
     int i, n=N(f)-1, m=N(t)-d;
@@ -1455,8 +1762,12 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
   case MARK:
     return exec_until_mark (t, p, var, level);
   case EVAL:
+    return exec_until (exec (t), p, var, level);
   case QUOTE:
+    (void) exec (t);
+    return false;
   case QUASI:
+    return exec_until_quasi (t, p, var, level);
   case QUASIQUOTE:
   case UNQUOTE:
   case VAR_UNQUOTE:
@@ -1476,6 +1787,7 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
   case INCLUDE:
     return exec_until_rewrite (t, p, var, level);
   case USE_PACKAGE:
+  case USE_MODULE:
   case OR:
   case XOR:
   case AND:
@@ -1515,6 +1827,9 @@ edit_env_rep::exec_until (tree t, path p, string var, int level) {
     return exec_until_compound (t, p, var, level);
   case REWRITE_INACTIVE:
     return exec_until_rewrite (t, p, var, level);
+  case HLINK:
+  case ACTION:
+    return exec_until_compound (t, p, var, level);
   default:
     if (L(t) < START_EXTENSIONS) {
       int i, n= N(t);
@@ -1638,15 +1953,15 @@ bool
 edit_env_rep::exec_until_arg (tree t, path p, string var, int level) {
   // cout << "  " << macro_arg << "\n";
   tree r= t[0];
-  if (is_atomic (r) && (!nil (macro_arg)) &&
+  if (is_atomic (r) && (!is_nil (macro_arg)) &&
       macro_arg->item->contains (r->label))
     {
       bool found;
       tree arg= macro_arg->item [r->label];
       list<hashmap<string,tree> > old_var= macro_arg;
       list<hashmap<string,path> > old_src= macro_src;
-      if (!nil (macro_arg)) macro_arg= macro_arg->next;
-      if (!nil (macro_src)) macro_src= macro_src->next;
+      if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+      if (!is_nil (macro_src)) macro_src= macro_src->next;
       if (level == 0) {
 	found= (r->label == var);
 	if ((N(t) > 1) && found) {
@@ -1655,7 +1970,7 @@ edit_env_rep::exec_until_arg (tree t, path p, string var, int level) {
 	    tree u= exec (t[i]);
 	    if (!is_int (u)) { found= false; break; }
 	    int nr= as_int (u);
-	    if ((!is_compound (arg)) || (nr<0) || (nr>=N(arg)) || nil (p)) {
+	    if ((!is_compound (arg)) || (nr<0) || (nr>=N(arg)) || is_nil (p)) {
 	      found= false; break; }
 	    if (p->item != nr) found= false;
 	    arg= arg[nr];
@@ -1674,13 +1989,13 @@ edit_env_rep::exec_until_arg (tree t, path p, string var, int level) {
   /*
   cout << "  " << macro_arg << "\n";
   tree r= t[0];
-  if (is_atomic (r) && (r->label == var) && (!nil (macro_arg))) {
+  if (is_atomic (r) && (r->label == var) && (!is_nil (macro_arg))) {
     bool found= (level == 0) && macro_arg->item->contains (r->label);
     tree arg  = macro_arg->item [var];
     list<hashmap<string,tree> > old_var= macro_arg;
     list<hashmap<string,path> > old_src= macro_src;
-    if (!nil (macro_arg)) macro_arg= macro_arg->next;
-    if (!nil (macro_src)) macro_src= macro_src->next;
+    if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+    if (!is_nil (macro_src)) macro_src= macro_src->next;
     if (found) exec_until (arg, p);
     else found= exec_until (arg, p, var, level-1);
     macro_arg= old_var;
@@ -1697,10 +2012,10 @@ edit_env_rep::exec_until_mark (tree t, path p, string var, int level) {
     // cout << "\n\tTest: " << t[0] << ", " << p << "\n";
     path q= p;
     int i, n= N(t[0]);
-    for (i=1; (!nil (q)) && (i<n); i++, q= q->next)
+    for (i=1; (!is_nil (q)) && (i<n); i++, q= q->next)
       if (t[0][i] != as_string (q->item))
 	break;
-    border= (i == n) && atom (q);
+    border= (i == n) && is_atom (q);
     // FIXME: in order to be clean, we should check whether q->item
     // is on the border of the contents of the argument t[0].
     // Nevertheless, this only matters for strings and
@@ -1711,6 +2026,15 @@ edit_env_rep::exec_until_mark (tree t, path p, string var, int level) {
   else return exec_until (t[1], p, var, level);
 }
 
+bool
+edit_env_rep::exec_until_quasi (tree t, path p, string var, int level) {
+  bool old= quote_substitute;
+  quote_substitute= true;
+  tree u= exec_quasiquoted (t[0]);
+  quote_substitute= old;
+  return exec_until (u, p, var, level);
+}
+
 bool
 edit_env_rep::exec_until_if (tree t, path p, string var, int level) {
   if ((N(t)!=2) && (N(t)!=3)) return false;
@@ -1752,18 +2076,18 @@ edit_env_rep::exec_until_while (tree t, path p, string var, int level) {
 ******************************************************************************/
 
 tree
-edit_env_rep::expand (tree t) {
-  if (is_atomic (t) || nil (macro_arg)) return t;
+edit_env_rep::expand (tree t, bool search_accessible) {
+  if (is_atomic (t) || is_nil (macro_arg)) return t;
   else if (is_func (t, ARG) || is_func (t, QUOTE_ARG)) {
     if (is_compound (t[0]))
       return tree (ERROR, "bad argument application");
     if (!macro_arg->item->contains (t[0]->label))
       return tree (ERROR, "argument " * t[0]->label);
-    tree r= macro_arg->item [r->label];
+    tree r= macro_arg->item [t[0]->label];
     list<hashmap<string,tree> > old_var= macro_arg;
     list<hashmap<string,path> > old_src= macro_src;
-    if (!nil (macro_arg)) macro_arg= macro_arg->next;
-    if (!nil (macro_src)) macro_src= macro_src->next;
+    if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+    if (!is_nil (macro_src)) macro_src= macro_src->next;
     if (N(t) > 1) {
       int i, n= N(t);
       for (i=1; i<n; i++) {
@@ -1774,16 +2098,27 @@ edit_env_rep::expand (tree t) {
 	r= r[nr];
       }
     }
-    if (is_func (t, ARG)) r= expand (r);
+    if (is_func (t, ARG))
+      r= expand (r, search_accessible);
     macro_arg= old_var;
     macro_src= old_src;
     return r;
   }
+  else if (is_func (t, EXPAND_AS, 2))
+    return expand (t[0], search_accessible);
+  else if (search_accessible && is_accessible (obtain_ip (t)))
+    return t;
   else {
     int i, n= N(t);
     tree r (t, n);
-    for (i=0; i<n; i++)
-      r[i]= expand (t[i]);
+    for (i=0; i<n; i++) {
+      r[i]= expand (t[i], search_accessible);
+      if (search_accessible &&
+	  is_accessible (obtain_ip (r[i])) &&
+	  drd->is_accessible_child (t, i))
+	return r[i];
+    }
+    if (search_accessible) return t;
     return r;
   }
 }
@@ -1795,7 +2130,7 @@ edit_env_rep::depends (tree t, string s, int level) {
        << " " << macro_arg << "\n";
   */
 
-  if (is_atomic (t) || nil (macro_arg)) return false;
+  if (is_atomic (t) || is_nil (macro_arg)) return false;
   else if (is_func (t, ARG) ||
 	   is_func (t, QUOTE_ARG) ||
 	   is_func (t, MAP_ARGS) ||
@@ -1810,8 +2145,8 @@ edit_env_rep::depends (tree t, string s, int level) {
       tree r= macro_arg->item [v->label];
       list<hashmap<string,tree> > old_var= macro_arg;
       list<hashmap<string,path> > old_src= macro_src;
-      if (!nil (macro_arg)) macro_arg= macro_arg->next;
-      if (!nil (macro_src)) macro_src= macro_src->next;
+      if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
+      if (!is_nil (macro_src)) macro_src= macro_src->next;
       bool dep= depends (r, s, level-1);
       macro_arg= old_var;
       macro_src= old_src;
