@@ -14,7 +14,7 @@
 #define GRID_H
 #include "frame.hpp"
 #include "curve.hpp"
-#include "ps_device.hpp"
+#include "renderer.hpp"
 
 class grid_curve_rep: public concrete_struct {
 public:
@@ -46,8 +46,12 @@ public:
   virtual void set_aspect (tree aspect);
 
   virtual operator tree () = 0;
-  virtual array<grid_curve> get_curves (point lim1, point lim2, double u) = 0;
+  virtual array<grid_curve> get_curves (
+    point lim1, point lim2, double u, bool b= false) = 0;
+  virtual array<grid_curve> get_curves_around (
+    point p, double delta, frame f);
   virtual point find_closest_point (point p, point pmin, point pmax) = 0;
+  virtual point find_point_around (point p, double delta, frame f);
 };
 
 class grid {
@@ -59,9 +63,15 @@ ABSTRACT_NULL_CODE(grid);
 grid empty_grid();
 grid cartesian (array<SI> subd, array<string> col, point o, double step);
 grid polar (array<SI> subd, array<string> col, point o, double step, SI astep);
-grid logarithmic (array<SI> subd, array<string> col, point o, double step, SI base);
+grid logarithmic (array<SI> subd, array<string> col, point o,
+		  double step, SI base);
 
 grid as_grid (tree t);
 tree as_tree (grid g);
 
+inline bool operator == (grid g1, grid g2) {
+  return as_tree (g1) == as_tree(g2); }
+inline bool operator != (grid g1, grid g2) {
+  return as_tree (g1) != as_tree(g2); }
+
 #endif // defined GRID_H
