@@ -13,7 +13,6 @@
 #ifndef MATH_UTIL_H
 #define MATH_UTIL_H
 #include "tree.hpp"
-#include <math.h>
 
 const  double tm_infinity=3.40282347e+38F;
 const  double tm_PI=3.1415926535897932384626433832795029L;
@@ -25,7 +24,10 @@ inline double nearest (double x) { // round missing in some math.h
   return floor (x+0.5); }
 inline int    sign (double x) { return x>0?1:x<0?-1:0; } 
 inline bool   fnull (double x, double approx) { return norm(x) <= approx; }
+#ifndef __SUNPRO_CC
 inline double pow (double x, int n) {
   if (n<=0) return 1; n--; while (n--) x*=x; return x; }
+#endif
+
 
 #endif // defined MATH_UTIL_H
