@@ -23,6 +23,7 @@ point operator *  (double x, point p );
 point operator /  (point  p, double x);
 bool  operator == (point p1, point p2);
 
+bool is_point (tree t);
 inline point as_point(double x) {
   point p(1); p[0]=x; return p; }
 point as_point (tree t);
@@ -39,11 +40,11 @@ bool   orthogonalize (point &i, point &j, point p1, point p2, point p3);
 
 typedef struct { point p0, p1; } axis;
 
-point  proj      (axis a, point p);
-double dist      (axis a, point p);
-double seg_dist  (axis a, point p);
-axis   midperp   (point p1, point p2, point p3);
-point  intersect (axis A, axis B);
+point  proj         (axis a, point p);
+double dist         (axis a, point p);
+double seg_dist     (axis a, point p);
+axis   midperp      (point p1, point p2, point p3);
+point  intersection (axis A, axis B);
 
 bool inside_rectangle (point p, point p1, point p2);
 
