@@ -16,7 +16,7 @@
 #include "basic.hpp"
 #include "rectangles.hpp"
 #include "path.hpp"
-#include "ps_device.hpp"
+#include "renderer.hpp"
 #include "font.hpp"
 #include "language.hpp"
 #include "hashmap.hpp"
@@ -30,6 +30,7 @@
 #define STACK_BOX     1
 #define CONTROL_BOX   2
 #define MOVE_BOX      3
+#define SCROLL_BOX    4
 
 /******************************************************************************
 * The cursor class
@@ -70,7 +71,8 @@ struct selection_rep: concrete_struct {
 struct selection {
   CONCRETE(selection);
   selection (rectangles rs= rectangles(),
-	     path start= path(), path end= path ());
+	     path start= path(), path end= path (),
+	     bool valid= true);
 };
 CONCRETE_CODE(selection);
 
@@ -79,23 +81,19 @@ bool operator != (selection sel1, selection sel2);
 ostream& operator << (ostream& out, selection sel);
 
 /******************************************************************************
-* The selection class
+* The graphical selection class
 ******************************************************************************/
 
-struct gr_selection_rep: concrete_struct {
-  array<path> cp;
-  SI dist;
-};
-
+struct gr_selection_rep;
 struct gr_selection {
   CONCRETE(gr_selection);
   gr_selection (array<path> cp= array<path> (), SI dist= 0);
 };
-CONCRETE_CODE(gr_selection);
 
 ostream& operator << (ostream& out, gr_selection sel);
 
 typedef array<gr_selection> gr_selections;
+tree as_tree (gr_selections sels);
 
 /******************************************************************************
 * The box class
@@ -108,6 +106,7 @@ typedef array<double> point;
 class box {
   ABSTRACT_NULL(box);
   inline box operator [] (int i);
+  box operator [] (path p);
   operator tree ();
   bool operator == (box b2);
   bool operator != (box b2);
@@ -133,21 +132,22 @@ public:
   void              relocate (path p, bool force= false);
   virtual box	    transform (frame fr);
   virtual operator  tree () = 0;
-  virtual void      pre_display (ps_device& dev);
-  virtual void      post_display (ps_device& dev);
-  virtual void      display (ps_device dev) = 0;
+  virtual void      pre_display (renderer& ren);
+  virtual void      post_display (renderer& ren);
+  virtual void      display (renderer ren) = 0;
   virtual void      clear_incomplete (rectangles& rs, SI pixel,
 				      int i, int i1, int i2);
   virtual int       subnr ();
   virtual box       subbox (int i);
   virtual tree      action (tree t, SI x, SI y, SI delta);
+  virtual void      loci (SI x, SI y, SI d, list<string>& ids, rectangles& rs);
   virtual void      position_at (SI x, SI y, rectangles& change_log);
   virtual void      collect_page_numbers (hashmap<string,tree>& h, tree page);
   virtual path      find_tag (string name);
 
   virtual int  reindex (int i, int item, int n);
-  void redraw (ps_device dev, path p, rectangles& l);
-  void redraw (ps_device dev, path p, rectangles& l, SI x, SI y);
+  void redraw (renderer ren, path p, rectangles& l);
+  void redraw (renderer ren, path p, rectangles& l, SI x, SI y);
 
   /*************************** positioning routines **************************/
 
@@ -210,7 +210,7 @@ public:
   virtual grid      get_grid ();
   virtual void      get_limits (point& lim1, point& lim2);
 
-  frame     find_frame (path bp);
+  frame     find_frame (path bp, bool last= false);
   grid      find_grid (path bp);
   void      find_limits (path bp, point& lim1, point& lim2);
 
@@ -221,7 +221,7 @@ public:
   /************************** retrieving information *************************/
 
   virtual int       get_type ();
-  virtual tree      get_info ();
+  virtual tree      get_info (tree in);
   virtual int       get_leaf_left_pos ();
   virtual int       get_leaf_right_pos ();
   virtual string    get_leaf_string ();
@@ -248,7 +248,8 @@ public:
   friend struct page_box_rep; // temporary friends for accessing x0 and y0
   friend struct lazy_paragraph_rep;
   friend class  phrase_box_rep;
-  friend void make_eps (url dest, ::display dis, box b, int dpi= 600);
+  friend class  remember_box_rep;
+  friend void make_eps (url dest, box b, int dpi= 600);
 };
 ABSTRACT_NULL_CODE(box);
 
@@ -277,6 +278,11 @@ inline int N (box b) { return b.rep->subnr(); }
 ostream& operator << (ostream& out, box b);
 SI   get_delta (SI x, SI x1, SI x2);
 bool outside (SI x, SI delta, SI x1, SI x2);
+void make_eps (url dest, box b, int dpi);
+path find_innermost_scroll (box b, path p);
+path find_scrolled_tree_path (box b, path sp, SI x, SI y, SI delta);
+void find_canvas_info (box b, path sp, SI& x, SI& y, SI& sx, SI& sy,
+		       rectangle& outer, rectangle& inner);
 
 extern bool   refresh_needed;
 extern time_t refresh_next;
@@ -287,20 +293,20 @@ void          refresh_at (time_t t);
 #define DECORATION_MIDDLE (-3)
 #define DECORATION_RIGHT  (-4)
 #define DETACHED          (-5)
-#define is_accessible(p) ((nil (p)) || ((p)->item >= 0))
-#define is_decoration(p) ((!nil (p)) && ((p)->item < 0))
+#define is_accessible(p) ((is_nil (p)) || ((p)->item >= 0))
+#define is_decoration(p) ((!is_nil (p)) && ((p)->item < 0))
 inline path descend (path ip, int i) {
-  return (nil (ip) || (ip->item >= 0))? path (i, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (i, ip): ip; }
 inline path decorate () {
   return path (DECORATION); }
 inline path decorate (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION, ip): ip; }
 inline path decorate_left (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_LEFT, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_LEFT, ip): ip; }
 inline path decorate_middle (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_MIDDLE, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_MIDDLE, ip): ip; }
 inline path decorate_right (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_RIGHT, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_RIGHT, ip): ip; }
 path descend_decode (path ip, int side);
 
 inline bool box_rep::accessible () { return is_accessible (find_lip ()); }
@@ -314,4 +320,17 @@ tree attach_dip (tree ref, path ip);
   attach_dip(t,decorate_middle(ip)),decorate_middle(ip)
 #define attach_right(t,ip) attach_dip(t,decorate_right(ip)),decorate_right(ip)
 
+/******************************************************************************
+* The graphical selection class (continued)
+******************************************************************************/
+
+struct gr_selection_rep: concrete_struct {
+  array<path> cp;
+  array<point> pts;
+  point p;
+  SI dist;
+  curve c;
+};
+CONCRETE_CODE(gr_selection);
+
 #endif // defined BOXES_H
